%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%For an overview of the toolbox, see reference [2].
%The following functions for the end user are currently implemented:
%
% Definition of antennas.
%   defant      : Defines an antenna.
%   pantpat3    : Plots the antenna pattern (mechanical scan) of an antenna.
%   spantpat3   : Simple or short calling form of pantpat3.
%   gettap1     : Calculates taperings for conventional beamforming.
%
% Non-radar signal simulation.
%   compsim4    : Simulates received antenna signals. Only one range bin
%                 (passive mode).
%   simwidebandkernel : Simulates wideband radio signals received with array
%                 antennas. Only one range bin (passive mode).
%   simwideband : Like simwidebandkernel but with other input parameters.
%
% Radar signal simulation
%   simradarsig : Simulates received radar antenna signals (active mode).
%                 Multiple range bins and multiple modulation codes are used
%                 together with "simnoise" and "simtarget".
%   deftarget   : Defines a target for the function "simaradarsig".
%   defnoise    : Defines a noise for the function "simaradarsig".
%   defclutter  : Defines clutter for the function "simaradarsig".
%   defjammer   : Defines a jammer for the function "simaradarsig".
%   defsources  : Combines several signal source definitions from the
%                 functions "deftarget", "defnoise", "defclutter", 
%                 "defjammer" to a single input parameter to "simradarsig".
%   defwave     : Defines a waveform for use with, for example, "simradarsig".
%   getmod      : Returns a pulscode modulation for use with for example
%                 "defwave" and "pulscomp".
%
% Low level radar signal simulation (usually not called by the end user).
%   simnoise    : Simulates received noise signals. Called by simradarsig.
%   simtarget   : Simulates radarsignals from targets. Called by simradarsig.
%   simclutter  : Simulates clutter. Called by simradarsig.
%
% Using signals from the FOA experimental antenna.
%   expsig1     : Gets measured antenna signals.
%   setcal1     : Sets calibration compensation to use with sdoaspc,
%                sdoapar1 etc. via spastemat.
%   sigcomp2    : Does calibration compensation on the signals.
%   dbsfileinfo : Displays information about a DBS-file.
%   dbcfileinfo : Displays information about a DBC-file.
%
% Radar signal management and analysis.
%   basecorrm   : Base estimates of the antenna signal correlation matrix.
%   ecorrm      : Antenna signal correlation matrix estimate or modfication
%                 of the estimate.
%   sigsize     : Returns the size of a radar signal.
%   sigplot2    : Plots a 1D- or 2D-slice of a radar signal.
%   sigplot2win : Interactiv version of "sigplot2" via a GUI.
%   addsig      : Adds two radar signals.
%   sigsubix    : Gets some specified elements (indices) of a radar signal.
%   sigsplitdim : Splits a dimension in a radar signal into two dimensions.
%   sigswapdim  : Swaps two dimensions in a radar signal.
%   normcohpulse: Makes a radar signal coherent between pulses.
%   normsig     : Normalizes a radar signal or a spatial correlation matrix 
%                 relative the noise power.
%   estnoisepower:Estimates the noise power in a radar signal.
%   RxRadarSigT : Creates a variable of the data type "RxRadarSigT", i.e.
%                 a radar signal.
%   msigmat2    : Rearranges the received antenna signals to a multi-
%                 dimensional matrix with time, distance and direction as 
%                 indices.
%   examsigw    : Examines a received radar signal waveform in various ways.
%   RxCorrMatT  : Creates a variable of the data type "RxCorrMatT", i.e. 
%                 spatial correlation matrices.
%   eigplot1    : Plots the eigenvalues of correlation matrices.
%   correig2sig : Calculates the eigenvalues of correlation matrices and
%                 converts the result to a radar signal so that it can be
%                 plotted with functions intended for radar signals.
%   corrmplot   : Plots a spatial correlation matrix.
%
% Simulation of channel errors.
%   simcherr    : Simulates channel errors.
%
% Spectral DOA estimation ("beamforming").
%   sdoaspc     : Estimates a DOA-spectrum with different methods. 
%                 Implemented methods are Conventional beamforming, Capon, 
%                 Music, Min-norm.
%   doaspc1     : Like sdoapar but with som extra input parameters.
%   splot2      : Plots one or more DOA spectrums.
%   doapeak     : Finds the peaks of a DOA spectrum.
%   doaspc2sig  : Converts a DOA spectrum to a radar signal so that it can be
%                 plotted with functions intended for radar signals.
%   DoaSpecT    : Creates a variable of the data type "DoaSpecT", i.e.
%                 a DOA spectrum.
%
% Parametric DOA estimation.
%   sdoapar1    : Estimates DOA:s with different methods. Implemented
%                 methods are DML, SML, PTMF, WSF, SSF, Root Music, ESPRIT.
%   doapar1     : Like sdoapar but with som extra input parameters.
%   stardoa     : Finds the startvalues for parametric algorithms.
%   pdoapar     : Presents on the screen information about DOA:s.
%   pdoait      : Plots the DOA:s for all iterations from PTMF.
%   perrfun     : Plots the error function for all iterations from PTMF.
%
% Estimation of number of signal sources.
%   spanosrc    : Estimates the number of signals sources using spatial
%                 information.
%
% High level radar signal processing functions.
%   beamform    : Conventional beamforming with or without tapering and
%                 with or without side lobe cancellation.
%   dbf         : Performs conventional beamforming (old function).
%   pulscomp    : Performs pulse compression.
%   dopplerfilt : Dopplerfilter bank by correlation.
%   dfb2        : Doppler filter bank by FFT.
%   videoint    : Video integration (incoherent summation) of all coherent
%                 processing intervalls.
%   cfar        :
%
% Low level radar signal processing functions.
%   spafilt     : Spatial linear filtering (beamforming).
%   pulfilt     : Pulse (slow-time) linear filtering (doppler filtering).
%   ranfilt     : Range (fast-time) linear filtering (pulse compression).
%   spapulfilt  : Space-pulse (space-slow time) linear filtering 
%                 (slow-time STAP).
%   sparanfilt  : Space-range (space-fast time) linear filtering 
%                 (fast-time STAP).
%
% Weigting functions.
%   taylorlp    : Calculates Taylor weights.
%   chebylp     : Calculates Dolhp-Chebychev weights.
%   bartlett    : Calculates Bartlett weights (Signal Processing Toolbox).
%   blackman    : Calculates Blackman weights (Signal Processing Toolbox).
%   hamming     : Calculates Hamming weights (Signal Processing Toolbox).
%   hanning     : Calculates Hanning weights (Signal Processing Toolbox).
%   kaiser      : Calculates Kaiser weights (Signal Processing Toolbox).
%   triang      : Calculates triangular weights (Signal Processing Toolbox).
%
%
%Miscellaneous processing functions.
%   estlocalfreq: Estimates the local (instantaneous) frequency of a signal.
%
%Auxiliary functions.
%   helpd       : Displays Matlab helptext with 'more on'.
%   helpwin2	: Displays Matlab helptext in separate window.
%   dbtinfo     : Displays information messages. Can be turned on and off.
%   dbtwarning  : Displays warning messages. Can be turned on and off.
%   dbterror    : Displays an error message and halts execution.
%   dbtversion  : Return the version number of this release of DBT.
%   d2r         : Converts from degrees to radians.
%   r2d         : Converts from radians to degrees.
%   xmin,xmax   : Sets the minimum and maximum on the x-axis.
%   ymin,ymax   : Sets the minimum and maximum on the y-axis.
%   zmin,zmax   : Sets the minimum andr maximum on the z-axis.
%   xlim,ylim,zlim: Sets the minimum and maximum on the x-, y- and z-axis.
%   plotxline   : Plots straight lines at specified x-positions.
%   plotyline   : Plots straight lines at specified y-positions.
%   plotaline   : Plots straight lines at specified xyz-pos:s.
%   genfreqaxis : Generates values for a frequency axis used at plotting etc.
%   normplot2   : Normal probability plot.
%   printfm     : Create file output of a Matlab figure suitable to import in
%                 FrameMaker.
%   coa         : = "close all".
%   fig 5       : = "figure(5)".
%   irot        : = "rotate3d". Interactive 3D rotation with the mouse.
%   dbtrefguide : Shows the DBT reference guide on the screen.
%   fftshift1   : Shift DC component to or from center of spectrum for
%                 several 1D spektra.
%   orthbeam    : Orthogonalization of beamforming weights.
%   setr, getr, existr, isrecvar, recdir, showr, whosr:
%     Functions for the management of record variables (called structures
%     in Matlab 5). A part of the package RECVAR. See "help recvar" and
%     [3,4] for more information.
%   setm, getm, getm2, getm3, zerosm, sizem, ismulmat, reshapem
%     Functions for the management of multidimensional numerical matrices.
%     A part of the package RECVAR. See "help multimat" and [3,4] for more
%     information.
%   ran2ranbin    : Converts ranges to rangebins and calculates length of a
%                   range bin.
%   getranbinlen  : Gets the length of a range bin.
%   speedoflight  : Returns the speed of light in empty space.
%   eetimebegin,  : These four functions are used to estimate the execution
%   eetimeend,      time in advance.
%   eetimeloop, 
%   eetimepreloop
%
%
%The following scripts for the end user are currently implemented:
%
%   dbtex1.m    : An example with an ULA with isotropical elements.
%   dbtex2.m    : An example of DOA-estimation with coherent signals.
%   dbtex3.m    : An example with the AIMT application example.
%   dbtex4.m    : An example with PTMF.
%   dbtex51.m   : Demonstrates conventional radar signal processing.
%   dbtex5.m    : Demonstrates the conventional signal processing chain in
%                 the AIMT application example.
%   dbtex6.m    : An example doing Monte-Carlo simulations with DML and ULA.
%   dbtex7.m    : An example of beamforming on measured data from the AIMT
%                 experimental antenna.
%   dbtex8.m    : An example doing statistical analysis with the AIMT
%                 experimental antenna.
%   dbtex9.m    : An example of beamforming on measured data from the AIMT
%                 experimental antenna with different calibration compen-
%                 sations of the steering matrix.
%   dbtex10.m   : An example of beamforming on measured data from the AIMT
%                 experimental antenna with calibration compensation on the
%                 signals. NOTE: the implementation is not finnished. This
%                 example does not work yet.
%   dbtex15.m   : An example of plotting antenna patterns.
%   dbtex20.m   : An example for the simulator "simradarsig".
%   dbtex21.m   : An example of beamforming of radar signals.
%   dbtex22.m   : An example of pulse compression of radar signals.
%   dbtex25.m   : An example of conventional and high resolution doppler 
%                 filtering.
%   dbtex26.m   : An example of conventional and high resolution simultaneous 
%                 beamforming and doppler filtering (slow-time STAP).
%   dbtex28.m   : Another example of conventional and high resolution 
%                 simultaneous beamforming and doppler filtering (slow-time 
%                 STAP).
%   dbtex29.m   : An example of noncoherent and coherent integration.
%   dbtex30.m   : An example of radar signal simulation, including clutter,
%                 with a flying side-looking array radar.
%   dbtex60.m   : An example of beamforming and DOA estimation on measured
%                 signals that are reflected against a rough surface.
%   dbtex70.m   : An example of performing radar signal processing on and
%                 plotting of measured radar signals.
%   dbtex80.m   : An example of the creation of an array antenna with
%                 subarrays using antenna type BFArray.
%   dbtex81.m   : An example of the creation and use of a circular array
%                 antenna.
%   moderrgui.m : An example of a graphical tool for model error analysis
%                 using the DBT toolbox.
%
%   dbthelp.m   : This help file.
%
%
%The following functions for the extension programmer currently exist:
%
%   doa2rect    : Converts between DOA coordinates and rectangular 
%                 coordinates.
%   doa2wavevec : Computes the wavevector for several waves.
%   spastemat   : Returns the ideal spatial steering matrix.
%   dspastemat  : Returns the derivative of the ideal (spatial) steering
%                 matrix.
%   pulstemat   : Returns the ideal pulse (slow-time) steering matrix.
%   ranstemat   : Returns the ideal range (fast-time) steering matrix.
%   spapulstemat: Returns the ideal space-pulse (space-slow time)
%                 steering matrix.
%   sparanstemat: Returns the ideal space-range (space-fast time)
%                 steering matrix.
%   getcorrm    : Returns an antenna array correlation matrix.
%   modcorrmkernel: Correlation matrix modification method code.
%   getcorrmkernel: Gets a (possibly modified) correlation matrix from
%                   a signal matrix.
%   gettap1     : Calculates tapering weights for different tapering types.
%   chdtype     : Asserts that a variable is of the desired data type.
%   squeezeall  : Remove all singleton dimensions, including dimension one.
%   vecnorm2    : Computes the euclidean norm (2-norm) of several vectors.
%   weightrows  : Weights the rows of a matrix.
%   weightcols  : Weights the columns of a matrix.
%   multdiag    : Returns the diagonal of a product of two matrices.
%   dftopm      : Generates a DFT operator matrix, defined as the Matlab FFT.
%   dftopm2d    : Generates a 2D-DFT op. matrix, defined as the Matlab FFT2.
%   calcfiltinit: Preparing the linear filtering with 'calcfiltproc'.
%   calcfiltproc: Performing linear filtering of several signal vectors.
%   setpultrans : Set transformation matrix for the pulse (time) dimension
%                 in a radar signal.
%   setrantrans : Set transformation matrix for the range dimension in a
%                 radar signal.
%   setspatrans : Set spatial transformation matrix in a radar signal.
%   setsparantrans : Set space-range transformation matrix in a radar signal.
%
%  Every function has a help text. Input parameters are described with name,
%  data type and a description. For example:
%   method (StringT) : Method
%     = 'cbf': ...
%     = 'capon': ...
%  where "method" is the parameter name, "StringT" is the data type and the
%  rest the description.
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%  For more information about the functions use "help" for the corresponding
%  function.
%
%  There are some programs and files, which are using DBT, for the studies
%  of the AIMT application example [5]. Help about these can be found by
%    >> help aimtexhelp
%
%  There are some programs and files, which are using DBT, for the studies
%  of the AIMT experimental antenna [6]. Help about these can be found by
%    >> help expanthelp
%
%  There are some programs and files, which are using DBT, for the studies
%  of radio communication (see for example [7]). Help about these can be
%  found by
%    >> help radiohelp
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Bjrklund S., Rejdemyhr D.: "A MATLAB Toolbox for Radar Array 
%    Processing", Proceedings of ISSPA `99 (IEEE Fifth International 
%    Symposium on Signal Processing and its Applications), August 22-25,
%    1999, Brisbane, Australien. p 547-550.
%  [3]: Bjrklund S.:"A Software Package for new Data Structures in MATLAB",
%    Proceedings of Nordic MATLAB Conference `97, Stockholm 27-28 October 1997.
%  [4]: Bjrklund S.: "RECVAR 3.5. A Software Package for New data Structures
%    in Matlab. Reference Guide", FOA-D--98-00379-408--SE, May 1998.
%  [5]: Athley F.:"Tillmpningsexempel fr studie av modellbaserad
%    lobformning", Ericsson Microwave Systems 1996, No FY/XU-96:299.
%  [6]: Pettersson L., Danestig M., Sjstrm U.: "An Experimental S-Band
%    Digital Beamforming Antenna", IEEE AES Systems Magazine, November 1997,
%    p.19-26.
%  [7]: Bjrklund S.: "A MATLAB implementation of a Simulator for Wideband
%    Radio Signals Received by Array Antennas", FOA-D--99-00435-504--SE,
%    March 1999.



%  The contents of this file is copied from the file "dbthelp.m".
%  All help text lines in "dbthelp.m" except the two first is used.
%
%  Start        : 961120 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:19:53 $ $Author: svabj $.
%  $Revision: 1.13 $
% *****************************************************************************

help Contents.m
