function clutterdef = defclutter(clType, clTypeParam, clPower, ...
                                clPatchDOAs, transAnt, platHeight, ...
                                platVel)

%DEFCLUTTER Defines clutter in a structure of type ClutterDefT.
%
%Synopsis:
%  clutterdef = defclutter()
%  clutterdef = defclutter(clType, clTypeParam)
%  clutterdef = defclutter(clType, clTypeParam, clPower)
%  clutterdef = defclutter(clType, clTypeParam, clPower, clPatchDOAs)
%  clutterdef = defclutter(clType, clTypeParam, clPower, clPatchDOAs, ...
%                         transAnt)
%  clutterdef = defclutter(clType, clTypeParam, clPower, clPatchDOAs, ...
%                         transAnt, platHeight)
%  clutterdef = defclutter(clType, clTypeParam, clPower, clPatchDOAs, ...
%                         transAnt, platHeight, platVel)
%
%Description:
%  Defines a structure variable containing all information needed by
%  function "simradarsig" in combination with function "simclutter"
%  to simulate the radar signal from clutter.
%
%  It is possible to simulate clutter frome a side-looking moving radar
%  at a specified altitude above the ground. The orientation of the radar 
%  antenna can only be side-looking.
%
%Input:
%  clType (StringT)             : Type of clutter, available type are:
%     ='MIT-LCE': Exponential pektral density ref [2]. (Default).
%  clTypeParam (CellArrayT)     : Parameters which are specific to
%     chosen clutter type.
%  clPower (RealScalarT)        : Power of clutter at recieving antenna in
%     the direction where the clutter is strongest. Default is 1 [W].
%  clPatchDOAs (Vector of DoaT) : Directions to the clutter patches.
%     Default is -60 to 60 degrees with a step of 0.5 degrees
%  transAnt (DefAntT)           : Transmitting antenna definition.
%  platHeight (RealScalarT)     : Height of the radar platform [m].
%  platVel (RealScalarT)        : Velocity along the linear array
%     of the radar platform [m/s].
%
%OutPut:
%  clutterdef (ClutterDefT) : A definition of the clutter in a struct.
%
%--------
%Clutter of type 'MIT-LCE'
%-------------------------
%Input:
%  clTypeParam = {rangeBins, beta, windSpeed}:
%     rangeBins (Vector of RealScalarT) : vector of the rangebins where
%         the clutter should be copied into.
%     beta (RealScalarT) : shape factor for the exponential power
%         spectral density function describing the clutter. Follwing
%         values are suitable to corresponding windspeed:
%           ---------------------------------------------------------
%           | Wind Speed [m/s] | beta (typical) | beta (worst case) |
%           |------------------|----------------|-------------------|
%           |     0.5-3        |      12        |       -           |
%           |------------------|----------------|-------------------|
%           |     3-7          |       8        |       -           |
%           |------------------|----------------|-------------------|
%           |     7-13         |      5.7       |       5.2         |
%           |------------------|----------------|-------------------|
%           |     13-23        |      4.3       |       3.8         |
%           ---------------------------------------------------------
%     windSpeed (RealScalarT) : Wind speed in [m/s].
%
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in
%  [1] or by "help dbtdata".
%
%Examples:
%  clSrc1 = defclutter('MIT-LCE', {6000 4 50}, 1, d2r(-30):0.5:d2r(30));
%     This defines a clutter sourc of type 'MIT-LCE' at the range 6000 [m]
%     with a windspeed of 50 [mph] and a windfactor of 4. The clutter is
%     also of power 1 [W] and the directions to the clutterpatches is
%     given by the directions -30 to 30 [degrees] with a step of 0.5
%     [degrees].
%
%Software Quality:
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S., Rejdemyhr D. Athley F.: "DBT, A MATLAB Toolbox
%       for Radar Signal Processing. Reference Guide",
%       FOA-D--9x-00xxx-408--SE, To be published.
%
%  [2]: Billingsley B. Et. Al. Impact of experimental measured doppler
%       spectrum of ground clutter on MTI and STAP. Edinburgh.
%       IEE international conference RADAR 97. 14-16 October 1997.
%
%See also:
%  defnoise deftarget defjammer simradarsig

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
%  (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 981001 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:20:31 $ $Author: svabj $.
%  $Revision     : 1.0 $
%*************************************************************************

%-------------------------------------------------------------------------
%----          In-parameter control & default values handling         ----
%-------------------------------------------------------------------------


if (0)

arginNo=1;
if (nargin < arginNo)
  error('DBT-Error: To few input parameters.')
end
arginNo = arginNo +1;

% ****************** Add missing input parameters ******************

if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;


% ****************** Default values ******************

if isempty(method)
  method = 'cbf';
end%if


% ******************  ******************

else %if(0)

if nargin < 7               % Defaults to platform velocity 40 [m/s]
  platVel = 0;
end%if

if nargin < 6               % Defaults to platform height 3000 [m]
  platHeight = 2;
end%if

if nargin < 5               % Defaults to string that tells us that
  transAnt = 'recAnt';      % the transmitting antenna is to be the
end%if                      % same as receiving antenna. see simradarsig.m
if (isempty(transAnt))
    transAnt = 'recAnt';
end%if 

if nargin < 4 | isempty(clPatchDOAs)
  clPatchDOAs = d2r(-60):0.5:d2r(60);  % Directions to clutterpatches
end%if

if nargin < 3 | isempty(clPower)  % Defaults to the power 1 [W].
  clPower = 1;
end%if

if nargin < 2 | isempty(clType) | isempty(clTypeParam)
  if nargin == 1 & ~strcmp(clType,'MIT-LCE')
    error('DBT-Error: Missing clutter type specific parameters');
  end%if
  b =  12;                    % Defaults to light air condition
  windSpeed = 5;              % Defaults to a windspeed of 5 [mph]
  clRange = 1000;             % Defaults to the range 1000 [m]
  clTypeParam{1} = b;         % All these parameters are specific
  clTypeParam{2} = windSpeed; % to the clutter type 'MIT-LCE'.
  clTypeParam{3} = clRange;
end%if

end%if(0)

if isempty(platHeight)
  platHeight = 0;
end%if
if platHeight == 0
   dbtwarning('Platform height set to zero => no clutter simulated')
end

%-------------------------------------------------------------------------
%----               Setting up the output parameter                   ----
%-------------------------------------------------------------------------

clutterdef.dataType     = 'ClutterDefT';
clutterdef.Type         = clType;
clutterdef.TypeParam    = clTypeParam;
clutterdef.Power        = clPower;
clutterdef.DOAs         = clPatchDOAs;
clutterdef.TransAnt     = transAnt;
clutterdef.PlatHeight   = platHeight;
clutterdef.PlatVelocity = platVel;

%End Of File -------------------------------------------------------------

