function noisedef = defnoise(noiseType, noisePower, noiseCorrMx)

%DEFNOISE Defines a noise source in a structure of type NoiseDefT.
%
%Synopsis:
%  noisedef = defnoise(noiseType, noisePower, corrMx)
%
%Description:
%  Defines a struct variable containing all information needed by function
%  "simradarsig" in combination with function "simnoise" to simulate the
%  radarsignal from noise that are created in the receiving radarsystem's
%  channels. see simnoise for more detailed information.
%
%Input:
%  noiseType [D](StringT)      : Type of noise, available type are:
%     = 'Gaussian': Complex gaussian noise. (Default).
%  noisePower [D](RealScalarT) : Power of noise in receiving antenna
%     channels. Default is 1 [W].
%  noiseCorrMx [D](CxMatrixT)  : The correlations matrix describing the
%     correlation of noise between channels. Default is white noise.
%
%OutPut:
%  noisedef (NoiseDefT) : A definition of the noise in a struct.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in
%  [1] or by "help dbtdata".
%
%Examples:
%  noiseSrc1 = defnoise('Gaussian', 0.3, eye(noChannels));
%     This defines a noise source of the type 'Gaussian' which is a
%     complex gaussian noise source with the power 0.3. i.e the variance
%     of the complex noise is equal to 0.3. This noise source is also
%     uncorrelated between the antenna channels.
%
%Software Quality:
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S., Rejdemyhr D. Athley F.: "DBT, A MATLAB Toolbox
%       for Radar Signal Processing. Reference Guide",
%       FOA-D--9x-00xxx-408--SE, To be published.
%
%See also:
%  deftarget defclutter defjammer simradarsig

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
%  (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 981001 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:20:32 $ $Author: svabj $.
%  $Revision     : 1.0 $
%*************************************************************************

%-------------------------------------------------------------------------
%----         In-parameter control & default values handling          ----
%-------------------------------------------------------------------------

if nargin < 3
  noiseCorrMx = [];               % Defaults to white noise
end%if

if nargin < 2
  noisePower = 1;                 % Defaults to the power 1 [W].
end%if

if nargin < 1
  noiseType = 'Gaussian';         % Defaults to gaussian noise
end%if


%-------------------------------------------------------------------------
%----               Setting up the output parameter                   ----
%-------------------------------------------------------------------------

noisedef.dataType = 'NoiseDefT';
noisedef.Type     = noiseType;
noisedef.Power    = noisePower;
noisedef.CorrMx   = noiseCorrMx;


%End Of File -------------------------------------------------------------

