function targetdef = deftarget(trgType, trgPower, trgRange, trgDOA, trgVel)

%DEFTARGET Defines a target as a struct of type TargetDefT.
%
%Synopsis:
%  targetdef = deftarget
%  targetdef = deftarget(trgType)
%  targetdef = deftarget(trgType, trgPower)
%  targetdef = deftarget(trgType, trgPower, trgRange)
%  targetdef = deftarget(trgType, trgPower, trgRange, trgDOA)
%  targetdef = deftarget(trgType, trgPower, trgRange, trgDOA, trgVel)
%
%Description:
%  Defines a struct variable containing all information needed by function
%  "simradarsig" in combination with function "simtarget" to simulate the
%  radarsignal from a target.
%
%Input:
%  trgType [D](StringT): Type of target, available types are:
%                        = 'swerling0', 'swerling5', 'marcum' or 'constant'
%                             : (Default). See reference [2] .
%                        = 'swerling1' : Not implemented.
%                        = 'swerling2' : Not implemented.
%                        = 'swerling3' : Not implemented.
%                        = 'swerling4' : Not implemented.
%  trgPower [D](RealScalarT): Power (linear scale) of target in recieving
%                           antenna channels
%                           Default is 1 [W].
%  trgRange [D](RealScalarT): Range to target. Default is 1000 [m].
%  trgDOA [D](DoaT)         : Direction to target. Default is straight forward.
%  trgVel [D](RealScalarT)  : Radial velocity of target relative radar. 
%                             Default is 0 [m/s].
%
%OutPut:
%  targetdef (TargetDefT) : A definition of the target in a struct.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%  trg = deftarget('swerling0', 4.7, 6000, [d2r(25); 0], 2);
%     This defines a constant (swerling0) target with a power of 4.7 [W]
%     at the distance 6 [km] and the direction 25 [degrees] horisontally
%     from the center direction of the antenna and with the velocity
%     2 [m/s].
%
%Software Quality:
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S., Rejdemyhr D. Athley F.: "DBT, A MATLAB Toolbox
%       for Radar Signal Processing. Reference Guide",
%       FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Sume A.: "Kursmaterial i radarteori", FOA 1995, Internal report
%       FOA-D--95-00108-3.3--SE.
%
%See also:
%  defnoise defclutter defjammer defsources simradarsig

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
%  (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 981001 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:20:34 $ $Author: svabj $.
%  $Revision     : 1.0 $
%*************************************************************************

%------------------------------------------------------------------------
%----         In-parameter control & default values handling         ----
%------------------------------------------------------------------------

if nargin < 5 | isempty(trgVel)
  trgVel = 0;                       % Defaults to a stationary target
end%if

if nargin < 4 | isempty(trgDOA)
  trgDOA = [0; 0];                  % Defaults to straight forward
end%if

if nargin < 3 | isempty(trgRange)
  trgRange = 1000;                  % Defaults to 1 km distant
end%if

if nargin < 2  | isempty(trgPower)
  trgPower = 1;                     % Defaults to the power 1
end%if

if nargin < 1 | isempty(trgType)
  trgType = 'swerling0';            % Defaults to the target type swerling0
end%if


%------------------------------------------------------------------------
%----              Setting up the output parameter                   ----
%------------------------------------------------------------------------

targetdef.dataType  = 'TargetDefT';
targetdef.Type      = trgType;
targetdef.Power     = trgPower; % Might be replaced by RCS later on
targetdef.Range     = trgRange;
targetdef.DOA       = trgDOA;
targetdef.Velocity  = trgVel;

%End Of File -------------------------------------------------------------

