function waveform = defwave(wavelength, noRangeBins, noPulses, pModulation, sampleTime, noCPI)

%DEFWAVE Defines the time properties of a radar or radio signal.
%
%Synopsis:
%  waveform=defwave(wavelength,noRangeBins,noPulses,pModulation,...
%                   sampleTime, noCPI)
%
%Description:
%  Defines the time properties of a radar or radio signal, e.g. number of
%  time samples, sample time and waveform modulation.
%  The information is stored in a structure variable of the data type
%  WaveformT, which is used when simulating radar signals etc.
% 
%  The pulse repetition interval (PRI) is defined as "waveform.noRangeBins
%  * waveform.sampleTime". This means that these fields can not be
%  changed without also changing the PRI.
%
%Input:
%  wavelength (RealScalarT): Wavelength of the carrier [m].
%     OR
%  wavelength (RealVectorT): Wavelengths of the carrier [m]. This means
%    that different wavelengths are used in different coherent pulse
%    intervalls (CPI:s), i.e. length(wavelength) (must be) = noCPI.
%    wavelength(1) is used in the first CPI and so on.
%  noRangeBins (IntScalarT): The separation between different pulses or
%    PRIs expressed in number of range bins. This is the same as the
%    maximum unambiguous range. The number of range bins actually present
%    in the signal is less or equal to "noRangeBins". The parameter 
%    "noRangeBins" is used to calculate the PRI [s] as 
%      PRI = noRangeBins * sampleTime [s].
%  noPulses (IntScalarT)   : Number of pulses * PRI = CPI (coherent
%     processing interval).
%  pModulation (CxVectorT) : The coeffient of the pulse modulation. See
%     help "getmod" for more information.
%  sampleTime (RealScalarT): Sample period time in transmitter and
%     receiver. One sampleTime corresponds to one range bin.
%  noCPI [D](IntScalarT)   : Number of coherent processing intervals (CPI:s).
%
%Output:
%  waveform (WaveformT) : The record variable containing the combined data.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See also:
%  getmod.m, simsig.m

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 971024 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:20:35 $ $Author: svabj $.
%  $Revision     : 1.0 $
% ************************************************************************

if (nargin < 5)
  error('DBT-Error: To few input parameters to defwave.');
end%if


%-------------------------------------------------------------------------
%----                    Handle default values                        ----
%-------------------------------------------------------------------------

if (nargin < 6)
  noCPI = 1;
end%if


%-------------------------------------------------------------------------
%----                   Checking input variables                      ----
%-------------------------------------------------------------------------

%----  Alternating wavelengths between cpi:s  ----

if ((length(wavelength) > 1) & (length(wavelength) ~= noCPI))
  error('DBT-Error: Wrong combination of noCPI and vector of wavelength.')
end%if


%-------------------------------------------------------------------------
%----            Construct the waveform struct variable.              ----
%-------------------------------------------------------------------------

waveform.dataType = 'WaveformT';

waveform.version = 1;

waveform.wavelength = wavelength;

waveform.noRangeBins = noRangeBins;

waveform.noPulses = noPulses;

waveform.sampleTime = sampleTime;

waveform.pModulation = pModulation;

waveform.noCPI = noCPI;

% End Of File
