function  [doaHat, sHat, Q, doaHatAllIt, sigma2] = doapar1(method, sigIn, doaStart, focusDist, rangeIx, cpiIx, in7, in8, in9)

%DOAPAR1 Parametric estimation of DOA with different methods.
%
%--------
%Synopsis:
%  [doaHat,...]=doapar1(method,sigIn,doaStart,focusDist,rangeIx,cpiIx,...)
%
%Description:
%  Parametric estimation of DOA (Direction of Arrival) with different methods.
%
%Output and Input:
%  doaHat (DoaParamT): The estimated doa.
%  method (StringT): Estimation method. See below for sections about each
%    method.
%    = 'dml': Deterministic Maximum Likelihood.
%    = 'sml': Stochasic Maximum Likelihood.
%    = 'wsf': Weighted Subspace Fitting.
%    = 'ssf': Subspace Fitting.
%    = 'rmusic': Root Music.
%    = 'esprit': LS-Esprit.
%    = 'ptmf': Parametric Target Model Fitting.
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    spatial correlation matrix. (For PTMF only RxRadarSigT is possible).
%  focusDist (RealScalarT): Focusing distance, e.g. 'Inf' for far field.
%    Not used yet.
%  rangeIx (IntScalarT): Index of range bin. Only used when "sigIn" is
%    of type RxRadarSigT. Default = 1.
%  cpiIx (IntScalarT): Index for CPI. Only used when "sigIn" is
%    of type RxRadarSigT. Default = 1.
%
%--------
%DML (Deterministic Maximum Likelihood).
%Synopsis:
% doaHat = doapar1('dml', sigIn, doaStart)
% doaHat = doapar1('dml', sigIn, doaStart, [], rangeIx, [])
% doaHat = doapar1('dml', sigIn, doaStart, [], rangeIx, cpiIx)
% doaHat = doapar1('dml', sigIn, doaStart, focusDist, rangeIx, cpiIx)
% [doaHat, sHat, dummy1, dummy2, sigma2] = doapar1('dml', sigIn, doaStart,
%   focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  sHat (CxMatrixT): The complex envelope of the estimated source signals,
%    a complex matrix of size (noSrc x T), where noSrc is the number of
%    sources and T is the number of snapshots. Not tested.
%  sigma2 (RealScalarT): Estimated noise variance. Not tested.
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  doaStart (Vector of DoaT): Start values of DOA:s. These should be
%    close to the real values.
%
%Description:
%  Returns maximum likelihood estimates for the DOA:s for a deterministic
%  signal model, see [2].
%  This method needs Matlab Optimization Toolbox.
%
%Algoritm:
%  An optimization problem is solved using the function "leastsq" in the
%  Optimization Toolbox [6,7]. For details see the code in the function doapar1
%  and in the function "dmlfun" and reference [2] and [11] p. 111-112.
%
%--------
%SML (Stochasic Maximum Likelihood).
%Synopsis:
% doaHat = doapar1('sml', sigIn, doaStart)
% doaHat = doapar1('sml', sigIn, doaStart, [], rangeIx, [])
% doaHat = doapar1('sml', sigIn, doaStart, [], rangeIx, cpiIx)
% doaHat = doapar1('sml', sigIn, doaStart, focusDist, rangeIx, cpiIx)
% [doaHat, Rss, dummy1, dummy2, sigma2] = doapar1('sml', sigIn, doaStart,
%   focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  Rss (RxMatrixT): Estimated source signal correlation matrix.
%  sigma2 (RealScalarT): Estimated noise variance. Not tested.
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  doaStart (Vector of DoaT): Start values of DOA:s. These should be
%    close to the real values.
%
%Description:
%  Returns maximum likelihood estimates for the DOA:s for a stochastic
%  signal model, see [2].
%  This method needs Matlab Optimization Toolbox.
%
%Algoritm:
%  An optimization problem is solved using the function "fminu" in the
%  Optimization Toolbox [6,7]. For details see the code in the function doapar1
%  and in the function "smlfun" and reference [2].
%
%--------
%WSF (Weighted Subspace Fitting).
%Synopsis:
% doaHat = doapar1('wsf', sigIn, doaStart)
% doaHat = doapar1('wsf', sigIn, doaStart, [], rangeIx, [])
% doaHat = doapar1('wsf', sigIn, doaStart, [], rangeIx, cpiIx)
% doaHat = doapar1('wsf', sigIn, doaStart, focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  doaStart (Vector of DoaT): Start values of DOA:s. These should be
%    close to the real values.
%
%Description:
%  See reference [4] (This is probably the same description as in [2]).
%  This method needs Matlab Optimization Toolbox.
%
%Algoritm:
%  An optimization problem is solved using the function "leastsq" in the
%  Optimization Toolbox [6,7]. For details see the code in the function doapar1
%  and in the function "wsffun" and reference [2].
%
%--------
%SSF (Subspace Fitting).
%Synopsis:
% doaHat = doapar1('ssf', sigIn, doaStart)
% doaHat = doapar1('ssf', sigIn, doaStart, [], rangeIx, [])
% doaHat = doapar1('ssf', sigIn, doaStart, [], rangeIx, cpiIx)
% doaHat = doapar1('ssf', sigIn, doaStart, focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  doaStart (Vector of DoaT): Start values of DOA:s. These should be
%    close to the real values.
%
%Description:
%  See reference [4] (This is probably the same description as in [2]).
%  This method needs Matlab Optimization Toolbox.
%
%Algoritm:
%  An optimization problem is solved using the function "leastsq" in the
%  Optimization Toolbox [6,7]. For details see the code in the function doapar1
%  and in the function "ssffun" and reference [2].
%
%--------
%ROOT MUSIC.
%Synopsis:
% doaHat = doapar1('rmusic', sigIn, noSrc)
% doaHat = doapar1('rmusic', sigIn, noSrc, [], rangeIx, [])
% doaHat = doapar1('rmusic', sigIn, noSrc, [], rangeIx, cpiIx)
% doaHat = doapar1('rmusic', sigIn, noSrc, focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  noSrc (IntScalarT): The number of sources.
%
%Description:
%  Gets a RootMUSIC pseudo DOA. For a description, see reference [2].
%  This method can only be used for ULA:s ([2] p. 80-81).
%  Note, that Root Music and LS-Esprit do not calculate the steering
%  matrix and does not call the function spastemat. All necessary calibration
%  compensation must therefore be done beforehand on the signals.
%
%Algoritm:
%
%--------
%LS-ESPRIT.
%Synopsis:
% doaHat = doapar1('esprit', sigIn, noSrc)
% doaHat = doapar1('esprit', sigIn, noSrc, [], rangeIx, [])
% doaHat = doapar1('esprit', sigIn, noSrc, [], rangeIx, cpiIx)
% doaHat = doapar1('esprit', sigIn, noSrc, focusDist, rangeIx, cpiIx)
%
%Output and Input:
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    correlation matrix of it.
%  noSrc (IntScalarT): The number of sources.
%
%Description:
%  For a description, see reference [2].
%  This method can only be used for ULA:s ([2] p. 80-81).
%  Can only be used for ULA:s ([2] p. 80-81).
%  Note, that Root Music and LS-Esprit do not calculate the steering
%  matrix and does not call the function spastemat. All necessary calibration
%  compensation must therefore be done beforehand on the signals.
%
%Algoritm:
%
%--------
%PTMF (Parametric Target Model Fitting)
%Synopsis:
%  doaHat = doapar1('ptmf', sigIn, doaStart, [], [], [], stepLength)
%  doaHat = doapar1('ptmf', sigIn, doaStart, [], rangeIx, [], stepLength)
%  doaHat = doapar1('ptmf', sigIn, doaStart,[], rangeIx, cpiIx, stepLength)
%  [doaHat, sHat, Q, doaHatAllIt] = doapar1('ptmf', sigIn, doaStart,
%    focusDist, rangeIx, cpiIx, stepLength)
%
%Description:
%  Estimates the DOA with PTMF (Parametric Target Model Fitting) and
%  stochastic approximation. This is probably the same optimization problem
%  as with the 'dml' method but a different optimization method is used!?!
%
%Output and Input:
%  sHat (CxMatrixT): The complex envelope of the estimated source signals,
%    a complex matrix of size (noSrc x T), where noSrc is the number of
%    sources and T is the number of snapshots.
%  Q (RealVectorT.'): The error function (the squared modulus of the error,
%    see []) for all iterations. Size (1 x T), where T is the number of
%    snapshots.
%  doaHatAllIt (RealMatrixT): Estimated theta angle for all iterations.
%    The size is (noSrc x T), where noSrc is the number of sources and T
%    is the number of snapshots.
%  sigIn (RxRadarSigT): The input antenna signal.
%  doaStart (Vector of DoaT) :Start values of DOA:s. These should be
%    close to the real values.
%  stepLength (RealScalarT): Step length, e.g. 0.01. It is a balance between
%    stability an rapid convergence.
%
%Algoritm:
%  The algorithm is based on the one in [] by U. Nickel but there are
%  some changes:
%  * The notation is somewhat changed (to come closer to []).
%  * The step length 'stepLength' is fix and not a function of time.
%  * There is no limitation of the change in theta to 1/10 of the beamwidth.
%  * The change in theta is normalized with the power of the antenna signals.
%  * The pseudo invers is used instead of the "normal equations" due to better
%    numerical properties.
%
%Known Bugs:
%  A possible bug is that the derivative of the steering matrix is complex
%  conjugated.
%  Now a complex conjugation of the signals is done. This is maybe an
%  indication of a bug somewhere.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  The methods 'dml', 'sml', 'wsf', 'ssf', 'rmusic' and 'esprit' were used
%  in the work done by A. Heydarkhan [8].
%  The method 'ptmf' was used in the work done by M. Borgstrm and
%  S. Bjrklund [9].
%
%Known Bugs:
%  The last input parameters "ranges", "theta0" and "poserr" (DBT release 1.6)
%  of the function spastemat are not used.
%  The wavelength is acquired from the antenna definition. This is not
%  logical but works as the wavelength happens to be stored there.
%
%References:
%  [1]: Bjrklund S.:"DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Krim H., Viberg M.:"Two Decades of Array Signal Processing Research",
%    IEEE Signal Processing Magazine, July 1996, pp. 67-94.
%  [3]: Roy R., Kailath T.:"ESPRIT-Estimation of Signal Parameters via
%    Rotational Invariance Techniques", IEEE Transaction on Acoustics,
%    Speech and Signal Processing, Vol. 37, No. 7, p. 984-995, July 1989.
%  [4]: Viberg M.:"Sensor Array Signal Processing, Lecture notes", Chalmers
%    University of Technology, From courses held at Chalmers the spring 1996
%    and at FOA the autumn 1996.
%  [5]: Carlstedt O.: "Superupplsning vid radarspaning i strd signalmilj",
%    FOA  Rapport C 30740-8.4,3.3, december 1993.
%  [6]: Grace A.:"Optimization Toolbox For Use with MATLAB", The MathWorks,
%    Inc. 1992. (Matlab 4)
%  [7]: Branch M. A.,Grace A.:"MATLAB Optimization Toolbox, User's Guide",
%    The MathWorks, Inc. 1996. (Matlab 5)
%  [8]: Heydarkhan A.: "Model Based Direction of Arrival Estimation Methods
%    Applied to Experimental Antenna Data", Master's Thesis, Chalmers
%    University of Technology 1997. FOA-R--97-00631-408--SE, FOA November 1997.
%  [9]: Borgstrm M., Bjrklund S.: "Ngra modeller, modellbaserade metoder
%    och simuleringar fr att skatta antalet radarml med digitala
%    gruppantenner", FOA-D--97-00319-408--SE.
%  [10]: Nickel U.:"    ".
%  [11]: Haykin S., Litva J., Shepherd T.J.(Eds.):"Radar Array Processing",
%    Springer Verlag 1993, ISBN 3-540-55224-3, 0-387-55224-3.
%
%See Also:
%  sdoapar, compsim2, expsig1, stardoa, pdoapar, pdoait, perrfun, doaspc1,
%  sdoaspc

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 960924 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:20:41 $ $Author: svabj $.
%  $Revision: 1.45 $
% *****************************************************************************


global thSign
  % Defines the reference direction for the theta angle in DOA:s.
  % Defined in "defant".


% ----------------------------------------------------------------------- %
% Handle output parameters
% ----------------------------------------------------------------------- %

doaHat      = [];
sHat        = [];
Q           = [];
doaHatAllIt = [];
sigma2      = [];


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

if (nargin < 2)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************

if (nargin < 3)
  doaStart = [];
end
if (nargin < 4)
  focusDist = [];
end
if (nargin < 5)
  rangeIx = [];
end
if (nargin < 6)
  cpiIx = [];
end
if (nargin < 7)
  in7 = [];
end
if (nargin < 8)
  in8 = [];
end
if (nargin < 9)
  in9 = [];
end


% ****************** Default values ******************

if (isempty(doaStart))
  doaStart = d2r([-1 1]);  % What default values should it be?
end%if
%if (isempty(focusDist))
%  focusDist = [];
%end%if
if (isempty(rangeIx))
  rangeIx = 1;
end%if
if (isempty(cpiIx))
  cpiIx = 1;
end%if


% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT','RxCorrMatT')
chkdtype(doaStart, 'Vector of DoaT')
%chkdtype(focusDist, 'RealScalarT')
chkdtype(rangeIx, 'IndexT')
chkdtype(cpiIx, 'IndexT')

if (size(doaStart,1) > 2)
  error('DBT-Error: doaStart must be a row vector.')
end%if


new = 0;
if (new)

% ----------------------------------------------------------------------- %
% Code common to most methods.
% ----------------------------------------------------------------------- %

% **************** Special treatment for some methods. ****************

if (strcmp(method,'esprit') | strcmp(method,'rmusic'))
  % We need to know how many DOAs to return.
  noSrc = doaStart;
  noDoasOut = noSrc;
else
  noDoasOut = length(doaStart);
end%if


% ****************** Create output variable. ******************

doaHat = DoaParamT(zeros([length(smplPoints), length(rangeIx), 1, ...
  length(cpiIx)]), smplPoints, [],rangeIx, [], cpiIx);


% *********** Estimate a correlation matrix. ***********

if (strcmp(sigIn.dataType,'RxRadarSigT'))
  Rxx1  = ecorrm(sigIn,[],rangeIx, cpiIx);
    % Estimation of the correlation matrices for all ranges and cpis in 
    % "rangeIx" and "cpiIx".
elseif (strcmp(sigIn.dataType,'RxCorrMatT'))
  % This if-clause is not necessary because the same if-clause exists
  % in the function "ecorrm" but it makes the code clearer.
  Rxx1 = sigIn;
else
  error('DBT-Error: Illegal datatype of input parameter "inData".')
end%if


% *********** Pick out some more fields from input parameters. ***********

antenna  = sigIn.antenna;
noChan   = antenna.noElem; % Number of channels.

waveform = sigIn.waveform;
lambda   = waveform.wavelength;



else %(new)

% ----------------------------------------------------------------------- %
% Code common to most methods.
% ----------------------------------------------------------------------- %

% ****************** Pick out fields from input parameters. ******************

% global antenna % "antenna" is global variable instead of input parameter to
    % dmlfun. This is an optimization of memory and execution speed.
    % The "global antenna" command must be located before the assignment of
    % "antenna variable".

Rxx1	= ecorrm(sigIn, [], rangeIx, cpiIx);
Rxx     = Rxx1.corrMat;
  % Get a antenna signal correlation matrix.
antenna = sigIn.antenna;
K       = antenna.noElem;   % Number of channels.

waveform = sigIn.waveform;
lambda  = waveform.wavelength;


% ****************** Precalculations. ******************



% ****************** Create output variable. ******************

doaHat.dataType = 'DoaParamT';
doaHat.version = 2;

end%if (new)


if (new)


% ----------------------------------------------------------------------- %
% Loops for several range gates and cpis.
% ----------------------------------------------------------------------- %

% ****************** Loops. ******************

%for cpiLoop = 1:length(cpiIx)
%for rangeLoop = 1:length(rangeIx)

  % ----------------------------------------------------------------------- %
  % Code common to all methods.
  % ----------------------------------------------------------------------- %

  % ***************** Pick out fields from input parameters. *****************

  Rxx     = Rxx1.corrMat(:,:,rangeLoop, 1, cpiLoop);
    % Get a antenna signal correlation matrix. "Rxx" is a 2D matrix.

  Pspect = zeros(noDoas,1);


end%if (new)

  % ----------------------------------------------------------------------- %
  % Methods
  % ----------------------------------------------------------------------- %

  % ----------------------------------------------------------------------- %
  % Deterministic Maximum Likelihood.
  %
  % Needs Matlab Optimization Toolbox.
  % For details, see reference [11] p. 111-112 and [2].
  % Start: 96xxxx Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  if (strcmp(method,'dml'))
    %  --- Calculate DOA estimates ---
    options = foptions; % Default parameters used by the optimization routines.
    thetaHat = leastsq('dmlfun', doaStart', options, [], Rxx, antenna, lambda)';

    %  --- Calculate noise variance estimate ---
    m = size(Rxx,1);
    A = spastemat(antenna, thetaHat, lambda);
    Ac = inv(A'*A)*A';
    p = eye(m)-A*Ac;
    sigma2 = (1/m)*trace(p*Rxx);

    %  --- Calculate source signal estimates ---
    if (~chkdtyph(sigIn,'RxRadarSigT'))
      % The calulation of "sHat" is only possible if the input is a
      % radar signal, not a correlation matrix.
      X   = getm(sigIn.signals, ':',rangeIx,':',1,cpiIx).';
      sHat = Ac*X;
    end%if

    % doaHat.doa = setm(zerosm([size(doaStart), 1]), thetaHat ,':',':');


  % ----------------------------------------------------------------------- %
  % Stochastic Maximum Likelihood.
  %
  % Needs Matlab Optimization Toolbox.
  % Start: 97xxxx Amir Heydarkhan (amihey).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'sml'))
    options = foptions; % Default parameters used by the optimization routines.
    thetaHat = fminu('smlfun', doaStart', options, [], Rxx, antenna, lambda)';
    if (any(any(isnan(thetaHat))))
      dbtwarning('SML minimization returns NaN.')
    end%if
    [Vsml, Rss, sigma2] = smlfun(thetaHat.',Rxx, antenna, lambda);


  % ----------------------------------------------------------------------- %
  % Weighted Subspace Fitting.
  %
  % Needs Matlab Optimization Toolbox.
  % Start: 97xxxx Amir Heydarkhan (amihey).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'wsf'))
    options = foptions; % Default parameters used by the optimization routines.
    thetaHat = leastsq('wsffun', doaStart', options, [], Rxx, antenna, lambda)';


  % ----------------------------------------------------------------------- %
  % Subspace Fitting.
  %
  % Needs Matlab Optimization Toolbox.
  % Start        : 97xxxx Amir Heydarkhan (amihey).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'ssf'))
    options = foptions; % Default parameters used by the optimization routines.
    thetaHat = leastsq('ssffun', doaStart', options, [], Rxx, antenna, lambda)';


  % ----------------------------------------------------------------------- %
  % Root MUSIC.
  %
  % Note, this function does not calculate the steering matrix and does not
  % call the spastemat function. All necessary calibration compensation
  % must be done beforehand on the signals.
  % Start: 970718 Amir Heydarkhan (amihey).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'rmusic'))
    antType = antenna.antennaType;
    if (~strcmp(antType,'ULA'))
      error('DBT-Error: Root MUSIC is only implemented for antennaType ULA')
    end
    noSrc = doaStart;
      % Note the different use of the input parameter "noSrc".
    d       = antenna.distElem;
    M       = size(Rxx,1);
    [U,S,V]=svd(Rxx);         % Eigendecomp. of correlation matrix using SVD.
    En=U(:,(noSrc+1):M);    % Splitting in signal and noise subspace.
    EE = En*En';
    for k = 1:M,
      tmp = 0;
      for h = 1:k,
         tmp = tmp+EE(h,M-k+h);
      end
      p(k) = tmp;
      p(2*M-k) = conj(tmp);
    end
    Z  = sort(roots(p));
    [m,mm] = size(Z);
    for k=1:m,
      if abs(Z(k))<1,
      tmpZ(k) = Z(k);
      end
    end
    z = tmpZ(size(tmpZ,2)-noSrc+1:size(tmpZ,2));
    thetaHat = asin(thSign * lambda/(2*pi*d)*angle(z));


  % ----------------------------------------------------------------------- %
  % LS-ESPRIT.
  %
  % Note, this function does not calculate the steering matrix and does not
  % call the spastemat function. All necessary calibration compensation
  % must be done beforehand on the signals.
  % Start: 97xxxx Amir Heydarkhan (amihey).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'esprit'))
    antType = antenna.antennaType;
    if (~strcmp(antType,'ULA'))
      error('DBT-Error: LS-ESPRIT is only implemented for antennaType ULA')
    end
    noSrc = doaStart;
      % Note the different use of the input parameter "noSrc".
    d       = antenna.distElem;
    M       = size(Rxx,1);
    [U,S,V]=svd(Rxx);         % Eigendecomp. of correlation matrix using SVD.
    Un=U(:,(noSrc+1):M);    % Splitting in signal and noise subspace.
    Us = U(:,1:noSrc);
    U1 = Us(1:M-1,:);
    U2 = Us(2:M,:);
    Q = U2\U1;              %LS-ESPRIT
    fi = eig(Q);
    thetaHat = asin(-thSign * lambda/(2*pi*d)*angle(fi))';
      % Note the minus sign before "thSign" here but not for Root-Music.


  % ----------------------------------------------------------------------- %
  % Root-WSF.
  %
  % Note, this function does not calculate the steering matrix and does not
  % call the spastemat function. All necessary calibration compensation
  % must be done beforehand on the signals.
  % Literature reference: ???
  % Start: 99xxxx Svante Bjklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'rwsf'))
    dbterror('Implemention is not finnished.')
    antType = antenna.antennaType;
    if (~strcmp(antType,'ULA'))
      error('DBT-Error: Root-WSF is only implemented for antennaType ULA')
    end
    noSrc = doaStart;
      % Note the different use of the input parameter "noSrc".
    d       = antenna.distElem;
    M       = size(Rxx,1);
    [U,S,V]=svd(Rxx);         % Eigendecomp. of correlation matrix using SVD.
    Un=U(:,(noSrc+1):M);    % Splitting in signal and noise subspace.
    Us = U(:,1:noSrc);
    U1 = Us(1:M-1,:);
    U2 = Us(2:M,:);
    Q = U2\U1;              %LS-ESPRIT
    fi = eig(Q);
    thetaHat = asin(-thSign * lambda/(2*pi*d)*angle(fi))';
      % Note the minus sign before "thSign" here but not for Root-Music.



  % ----------------------------------------------------------------------- %
  % PTMF (Parametric Target Model Fitting).
  %
  % Start: 95xxxx Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'ptmf'))
    if (isempty(rangeIx))
      rangeIx = 1;
    end%if
    if (isempty(cpiIx))
      cpiIx = 1;
    end%if
    if (isempty(in7))
      in7 = 1e-5;
    end%if

    stepLength = in7;
    noInMedian = 15;
      % This number of iterations is used in the median calculation of the
      % doa estimates.
    chkdtype(sigIn, 'RxCompSigT','RxRadarSigT')
    X   = getm(sigIn.signals, ':',rangeIx,':',1,cpiIx).';
    %X = conj(X); %Remove this line!

    T   = size(X,2);      % Number of snapshots == number of iterations.
    if (T < noInMedian)
      error('DBT-Error: To few snapshots for median calculation.')
    end
    noSrc = size(doaStart,2);
    if (size(doaStart,1) > 1)
      error('DBT-Error: Only implemented for one DOA-angle (theta).')
    end
    % Suppose that the number of targets is known: noSrc
    % Note that the variables containing angles below (PTMFthstart, thetahat,
    % thetahatPre etc.) do not conform to the specification that different
    % angels should be different columns.

    PTMFthstart = doaStart(1,:)';
    thetahat = zeros(noSrc,T);
      % thetahat(m,n), m = target index, n = timeindex
    thetaStart = PTMFthstart.*ones(noSrc,1);
    bhat = zeros(noSrc,T);
    Q = zeros(1,T);
    for t = 1:T
      %if (rem(t,10)==0)
      %  disp(sprintf('t=%d',t));
      %end%if
      if (t==1)
        thetahatPre = thetaStart;
      else
        thetahatPre = thetahat(:,t-1);
      end%if
      AmatrisPre = spastemat(antenna, thetahatPre', lambda);
      bhat(:,t) = pinv(AmatrisPre)*X(:,t);
        % Estimation of the target signals bhat(t).
      xTildePre = X(:,t) - AmatrisPre*bhat(:,t);
        % Error vector calculated with thetahat(t-1).
      Am = dspastemat(antenna, thetahatPre', lambda)';
        % Matrix with the derivative of the steering vectors.
        % Should it be .' instead of ' ?
      Qtheta = - 2*real(conj(bhat(:,t)).*(Am*xTildePre));
        % The gradient of Q.
      % thetahat(:,t) = thetahatPre - stepLength.*Qtheta;
        % Update of thetahat without normalization.
      thetahat(:,t) = thetahatPre - stepLength.*Qtheta/(X(:,t)'*X(:,t));
        % Update of thetahat with normalization.
      zTildePost = X(:,t) - spastemat(antenna, thetahat(:,t)', lambda) ...
        * bhat(:,t);
        % Error vector calculated with thetahat(t).
      Q(1,t) = zTildePost' * zTildePost;
        % The error function Q.
    end %for t

    thetaOut = median((thetahat(:,(T-noInMedian+1):T))');
    thetaHat = thetaOut;  % Note, thetahat and thetaHat are two different
      % variables.
    sHat = bhat;
    doaHatAllIt = thetahat;  % All iterations.


  % ----------------------------------------------------------------------- %
  % The else branch.
  % ----------------------------------------------------------------------- %
  else
    error('DBT-Error: Selected method is not implemented.')
  end%if


if (new)


  % ----------------------------------------------------------------------- %
  % Return data.
  % ----------------------------------------------------------------------- %

  infoStr = sprintf('doaspc1: cpiLoop = %d(%d), rangeLoop = %d(%d)\r',...
    cpiLoop, length(cpiIx), rangeLoop,length(rangeIx));
  dbtinfo(infoStr,1);

  spect.specSmpl(:,rangeLoop, 1, cpiLoop) = Pspect;

%end%for rangeLoop
%end%for cpiLoop
dbtinfo('');	%New line.

%spect.doaPos = smplPoints;
%spect.ranIx = rangeIx;
%spect.cpiIx = cpiIx;


else (new)
  % ----------------------------------------------------------------------- %
  % Return data common to all methods.
  % ----------------------------------------------------------------------- %

  thetaHat = sort(thetaHat);
  doaHat.doa = thetaHat;

end%if (new)


%endfunction doapar1
