function  spect = doaspc1(method, sigIn, specType, smplPoints, focusDist, rangeIx, cpiIx, in8, in9, in10, timeIx, snapshotDim)


%DOASPC1 Estimates a DOA-spectrum with different methods.
%
%  spect = doaspc1(method, sigIn, [], smplPoints, focusDist, rangeIx,
%    cpiIx, ...)
%
%More detailed help text by commands "help2 doaspc1" or "helpwin2 "doaspc1".


%DOASPC1 Estimates a DOA-spectrum with different methods.
%
%--------
%Synopsis:
%  spect = doaspc1(method, sigIn, [], smplPoints, focusDist, rangeIx,
%    cpiIx, ...)
%
%Description:
%  Estimates a DOA spectrum ((Direction of Arrival)) with different methods.
%
%  If the input parameters "rangeIx" or "cpiIx" are vectors, several DOA
%  spectra will be estimated, one for each specified range bin and CPI.
%
%  The input can either be a radar signal (RxCorrMatT) or a spatial
%  correlation matrix (RxCorrMatT). In the former case a correlation
%  matrix is estimated for all range bins and CPIs by calling
%     Rxx1  = ecorrm(sigIn,[],rangeIx, cpiIx);
%  This means that the samples used for the estimation are taken from all
%  pulses (see function "eccorrm").
%
%Output and Input:
%  spect (DoaSpecT): Output DOA-spectrum. It is a quadratic measure of the
%    presence of sources in different directions. It can be either a power
%    spectrum (e.g. cbf) or a pseudo spectrum (e.g. music).
%    All methods use the antenna signal correlation matrix, which is
%    estimated by the function "ecorrm" if it is not supplied as input.
%  method (StringT): Estimation method. See below for sections about each
%    method.
%    = 'cbf': Conventional beamforming.
%    = 'acbf': Adaptive conventional beamforming. This is beamforming with
%      adaptive sidelobe cancelation (ASLC). Not implemented.
%    = 'capon': Capons beamformer.
%    = 'capon2': An other implementation of Capons beamformer.
%    = 'music : MUSIC.
%    = 'music2': An other implementation of MUSIC.
%    = 'minnorm': Minimum norm.
%    = 'minnorm2': An other implementation of Minimum norm (freath).
%    = 'eigvec': Eigenvector method.
%    = 'pisa': Pisarenko Harmonic Decomposition.
%  sigIn (RxRadarSigT or RxCorrMatT): The input antenna signal or a
%    spatial correlation matrix.
%  smplPoints [D](Vector of DoaT): Calculate the spectrum at these angles.
%  focusDist [D](RealScalarT): Focus distance of the antenna [m]
%    (default = Inf). Not implemented.
%  rangeIx [D](IndexT): Indices of range bin (default = 1). Only used
%    when "sigIn" is of type RxRadarSigT. Then default is all range bins.
%  cpiIx [D](IndexT): CPI indices (default = 1). Only used
%    when "sigIn" is of type RxRadarSigT. Then default is all CPIs.
%  timeIx : Not Implemented.
%  snapshotDim [D](IntScalarT): Use snapshots from this dimension to
%    estimate the correlation matrix.  Not Implemented.
%
%--------
%Conventional Beamforming.
%Synopsis:
%  spect = doaspc1('cbf', sigIn)
%  spect = doaspc1('cbf', sigIn, [], smplPoints)
%  spect = doaspc1('cbf', sigIn, [], smplPoints, [], rangeIx, cpiIx)
%  spect = doaspc1('cbf', sigIn, [], [], [], [], [], taperType, taperParam)
%  spect = doaspc1('cbf', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    taperType, taperParam)
%
%Output and Input:
%   taperType (StringT): Tapering type.
%     = 'norm': Normalised beamforming ([2] p. 72-73).
%     = Otherwise the parameter "taperType" of function "gettap1".
%   taperParam (?): The third input parameter of function "gettap1".
%
%Description:
%  The conventional beamforming. It can be done either with any of the
%  taperings that are available with te function gettap1 or by a kind of
%  normalization.
%
%Algoritm:
%  taperType = 'norm':
%    P = (a'*Rxx*a)/(a'*a),
%    where P=P(doa) is the output spectrum as a function of direction,
%    a=a(doa) is the steering vector calculated by the spastemat function,
%    Rxx is an estimate of the input antenna signal correlation matrix.
%  taperType = Otherwise:
%     w = a .* taper
%     P = w'*Rxx*w,
%     where taper is a tapering from the function gettap1. Other variables
%     are as above.
%
%--------
%Adaptive Conventional Beamforming.
%Synopsis:
%  spect = doaspc1('acbf', sigIn, [], [], [], [], [], [], [], iNCorrMat)
%  spect = doaspc1('acbf', sigIn, [], [], [], [], [], taperType, taperParam,
%    iNCorrMat)
%  spect = doaspc1('acbf', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    taperType, taperParam, iNCorrMat)
%
%Output and Input:
%  taperType  (StringT): Tapering type.
%    = The parameter "taperType" of function "gettap1".
%  taperParam (?): The third input parameter of function "gettap1".
%  iNCorrMat (RxCorrMatT): The input antenna signal interference-plus-noise
%    correlation matrix. This should be estimated on a radar signal without
%    targets.
%
%Description:
%  Not implemented.
%  Adaptive (conventional) beamforming. Reference [4] p. 57-58 contains a
%  description of STAP which is a generalization of adaptive beamformning.
%  Possibility to choose a desired weight vector ([4] p. 57-58).
%
%Algoritm:
%
%Examples:
%  ant = defant('isotropULA',[noElem,D,lambda]);
%  sigJam = compsim2(ant, 1, T, jamModel, [thetaJam, phiJam, SNRJam, ...
%    alphaJam, dalphaJam, dist, ones(size(theta,1))], 'rndn', eye(noElem));
%  iNCorrMat = ecorrm(sigJam);
%  sigTgt = compsim2(ant, 1, T, tgtModel, [thetaTgt, phiTgt, SNRTgt, ...
%    alphaTgt, dalphaTgt, distTgt, ones(size(theta,1))], 'rndn', eye(noElem));
%  splot2(doaspc1('acbf', sigTgt, [], 'taylor', 40, iNCorrMat))
%
%--------
%Capons Beamformer.
%Synopsis:
%  spect = doaspc1('capon', sigIn)
%  spect = doaspc1('capon', sigIn, [], smplPoints)
%  spect = doaspc1('capon', sigIn, [], smplPoints, [], rangeIx, cpiIx)
%
%Description:
%  This is another adaptive conventional beamformer (see 'acbf') but there
%  is no possibility to choose a desired weight vector.
%  This method is the solution to a minimization that attempts to minimize
%  signals from other direction other than the current look direction, in
%  which a constant gain is maintained, [2] p73-74.
%  This method is also known as Minimum Variance Distorionless Response.
%
%Algoritm:
%  1/(a'*inv(Rxx)*a),
%  where P=P(doa) is the output spectrum as a function of direction,
%  a=a(doa) is the steering vector calculated by the spastemat function,
%  Rxx is an estimate of the input antenna signal correlation matrix.
%
%--------
%MUSIC.
%Synopsis:
%  spect = doaspc1('music', sigIn, [], [], [], [], [], noSrc)
%  spect = doaspc1('music', sigIn, [], smplPoints, [], [], [], noSrc)
%  spect = doaspc1('music', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    noSrc)
%
%Output and Input:
%   noSrc [D](IntScalarT): Hypotesis for the number of targets or signals.
%
%Description:
%  Multiple Signal Classification. Probably the most known model based DOA
%  spectrum estimation method. See for example [2] p. 74-75.
%
%Algoritm:
%  See [3].
%
%--------
%Minimum  Norm.
%Synopsis:
%  spect = doaspc1('minnorm', sigIn, [], [], [], [], [], noSrc)
%  spect = doaspc1('minnorm', sigIn, [], smplPoints, [], [], [], noSrc)
%  spect = doaspc1('minnorm', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    noSrc)
%
%Output and Input:
%   noSrc (IntScalarT): Hypotesis for the number of targets or signals.
%
%Description:
%  A modification of the MUSIC method. See [2] p. 75.
%Algoritm:
%
%--------
%Minimum  Norm 2.
%Synopsis:
%  spect = doaspc1('minnorm2', sigIn, [], [], [], [], [], noSrc)
%  spect = doaspc1('minnorm2', sigIn, [], smplPoints, [], [], [], noSrc)
%  spect = doaspc1('minnorm2', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    noSrc)
%
%Output and Input:
%   noSrc (IntScalarT): Hypotesis for the number of targets or signals.
%
%Description:
%  An other implementation of Minimum norm (by Fredrik Athley).
%
%Algoritm:
%  See [3].
%
%--------
%Eigenvector Method.
%Synopsis:
%  spect = doaspc1('eigvec', sigIn, [], [], [], [], [], noSrc)
%  spect = doaspc1('eigvec', sigIn, [], smplPoints, [], [], [], noSrc)
%  spect = doaspc1('eigvec', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    noSrc)
%
%Output and Input:
%   noSrc (IntScalarT): Hypotesis for the number of targets or signals.
%
%Description:
%  See [6] p. 9 and [7] p. 384.
%
%Algoritm:
%
%--------
%Pisarenko Harmonic Decomposition.
%Synopsis:
%  spect = doaspc1('pisa', sigIn, [], [], [], [], [], noSrc)
%  spect = doaspc1('pisa', sigIn, [], smplPoints, [], [], [], noSrc)
%  spect = doaspc1('pisa', sigIn, [], smplPoints, [], rangeIx, cpiIx,
%    noSrc)
%
%Output and Input:
%   noSrc (IntScalarT): Hypotesis for the number of targets or signals.
%
%Description:
%  See [6] p. 10. and [7] p. 371.
%  This method does not seem to work.
%
%Algoritm:
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%  ant = defant('isotropULA',[noElem,D,lambda]);
%  sig = compsim2(ant, 1, T, tgtModel, [theta, phi, SNR, alpha, dalpha, ...
%    dist, ones(size(theta,1))], 'rndn', eye(noElem));
%  splot2(sdoaspc('music',sig,[],MMu))
%  figure;
%  splot2(doaspc1('cbf', sig, [], d2r(-30:0.1:30), [], 3, 2, 'taylor', 40));
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  The methods 'cbf', 'capon', 'music' and 'minnorm' were used in the work
%  done by A. Heydarkhan [5].
%
%Known Bugs:
%  The wavelength is acquired from the antenna definition. This is not
%  logical but works as the wavelength happens to be stored there.
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Krim H., Viberg M.:"Two Decades of Array Signal Processing Research",
%    IEEE Signal Processing Magazine, July 1996, pp. 67-94.
%  [3]: Fredrik Athley, Ericsson Microwave Systems, Mlndal, Sweden.
%  [4]: Ward J.: "Space-Time Adaptive Processing for Airborne Radar",
%    Technical report 1015, MIT Lincoln Laboratory 1994.
%  [5]: Heydarkhan A.: "Model Based Direction of Arrival Estimation Methods
%    Applied to Experimental Antenna Data", Master's Thesis, Chalmers
%    University of Technology 1997. FOA-R--97-00631-408--SE, FOA November 1997.
%  [6]: Pettersson L.: "Inverkan av fel p ngra spektralmetoder fr 
%    riktningsbestmning", FOA, Under preparation.
%  [7]: Johnson D.H., Dudgeon D.E.: "Array Signal Processing, Concepts
%    and Techniques", Prentice Hall 1993, ISBN 0-13-048513-6.
%
%See Also:
%  sdoaspc, splot2, doaspc2sig, doapeak, doapar1, sdoapar1, ecorrm



%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 960924 Svante Bjrklund (svabj).
%  Latest change: $Date: 2001/08/31 14:31:44 $ $Author: svabj $.
%  $Revision: 1.54 $
% *****************************************************************************

%Remaining Help Text
%  spect = doaspc1(method, sigIn, specType, smplPoints, focusDist, rangeIx,
%    cpiIx, ...)
%  specType StringT   : Type of spectrum, = 'smpl','func' or []. Not implemented.
%
%Comments.
%  The order of the "method" and the "signal" input parameters should be changed to make it possible to let "method" have a default value.


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

if (nargin < 2)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************

if (nargin < 3)
  specType = [];
end
if (nargin < 4)
  smplPoints = [];
end
if (nargin < 5)
  focusDist = [];
end
if (nargin < 6)
  rangeIx = [];
end
if (nargin < 7)
  cpiIx = [];
end
if (nargin < 8)
  in8 = [];
end
if (nargin < 9)
  in9 = [];
end
if (nargin < 10)
  in10 = [];
end


% ****************** Default values ******************

if isempty(specType)
  specType = 'smpl';
end%if
if isempty(smplPoints)
  smplPoints = d2r(-90:0.5:90);
end
%if isempty(focusDist)
%  focusDist = [];
%end%if
if isempty(rangeIx)
  if (strcmp(sigIn.dataType,'RxCompSigT') | ...
    strcmp(sigIn.dataType,'RxRadarSigT'))
    sigSizeIn = sigsize(sigIn);
    rangeIx = (1:sigSizeIn(2));
  else
    rangeIx = 1;
  end%if
end%if
if isempty(cpiIx)
  if (strcmp(sigIn.dataType,'RxCompSigT') | ...
    strcmp(sigIn.dataType,'RxRadarSigT'))
    sigSizeIn = sigsize(sigIn);
    cpiIx = (1:sigSizeIn(5));
  else
    cpiIx = 1;
  end%if
end%if


% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT','RxCorrMatT')
chkdtype(specType, 'StringT')
chkdtype(smplPoints, 'Vector of DoaT')
%chkdtype(focusDist, 'RealScalarT')
%chkdtype(rangeIx, 'IndexT')
%chkdtype(cpiIx, 'IndexT')

if ~(strcmp(specType,'smpl'))
  error('DBT-Error: Only sampled spectrum type is implemented.')
end%if

% ----------------------------------------------------------------------- %
% Code common to all methods.
% ----------------------------------------------------------------------- %

% ****************** Create output variable. ******************

spect = DoaSpecT(zeros([length(smplPoints), length(rangeIx), 1, ...
  length(cpiIx)]), smplPoints, [],rangeIx, [], cpiIx);


% *********** Estimate a correlation matrix. ***********

if (strcmp(sigIn.dataType,'RxRadarSigT'))
  Rxx1  = ecorrm(sigIn,[],rangeIx, cpiIx);
    % Estimation of the correlation matrices for all ranges and cpis in 
    % "rangeIx" and "cpiIx".
elseif (strcmp(sigIn.dataType,'RxCorrMatT'))
  % This if-clause is not necessary because the same if-clause exists
  % in the function "ecorrm" but it makes the code clearer.
  Rxx1 = sigIn;
else
  error('DBT-Error: Illegal datatype of input parameter "inData".')
end%if


% *********** Pick out some more fields from input parameters. ***********

antenna  = sigIn.antenna;
noChan   = antenna.noElem; % Number of channels.

waveform = sigIn.waveform;
lambda   = waveform.wavelength;

% ****************** Precalculations. ******************

noDoas = size(smplPoints,2);  % Number of points to calculate the spectrum in.
pointSubDoa = 0;
A = spastemat(antenna,smplPoints,lambda,focusDist,pointSubDoa);
A = weightcols(A,1./vecnorm2(A).');
  % Normalizing the steering vectors to unity norm.

if (0) % This code could be used if there were a data type 'SteerMatT'. Then one could choose to input either sampling points or a precalculated steering matrix.
if (~chkdtyph(smplPoints,'Vector of DoaT'))
  A = spastemat(antenna,smplPoints,lambda,focusDist,pointSubDoa,0);
elseif (noChan == size(smplPoints,1)) % if (~chkdtyph(smplPoints,'SteerMatT'))
  A = smplPoints;
  %A = smplPoints.steermat;
  %smplPoints = smplPoints.smplPoints;
else
  error('DBT-Error: smplPoints is neither a Vector of DoaT nor a steering matrix for this antenna.')
end%if
end%if (0)


% ----------------------------------------------------------------------- %
% Loops for several range gates and cpis.
% ----------------------------------------------------------------------- %

% ****************** Loops. ******************

for cpiLoop = 1:length(cpiIx)
for rangeLoop = 1:length(rangeIx)

  % ----------------------------------------------------------------------- %
  % Code common to all methods.
  % ----------------------------------------------------------------------- %

  % ***************** Pick out fields from input parameters. *****************

  Rxx     = Rxx1.corrMat(:,:,rangeLoop, 1, cpiLoop);
    % Get a antenna signal correlation matrix. "Rxx" is a 2D matrix.

  Pspect = zeros(noDoas,1);


  % ----------------------------------------------------------------------- %
  % Methods
  % ----------------------------------------------------------------------- %

  % ----------------------------------------------------------------------- %
  % Min-norm
  %
  % Start: 9609xx Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  if (strcmp(method,'minnorm'))
    if (isempty(in8))
      in8 = 5;  % This should be a large enough number.
    end
    noSrc = in8;

    [U,S,V]=svd(Rxx);
    En=U(:,(noSrc+1):noChan);
    EnEn = En*En';
    p = EnEn(:,1);

    for n =1:noDoas
      a= A(:,n);
      Pspect(n) = real((a'*a)/(eps+ a'*p*p'*a));
        % eps to avoid divide by zero
    end%for
    spect.spectrumType = 'pseudoSpectrum';


  % ----------------------------------------------------------------------- %
  % Min-norm 2
  %
  % An other implementation of Min norm.
  % The result should be exactly the same as in 'minnorm' but the execution
  %   time somewhat larger.
  % Start: 97xxxx Fredrik Ahtley (freath).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'minnorm2'))
    if (isempty(in8))
      in8 = 5;  % This should be a large enough number.
    end
    noSrc = in8;

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.
    En=U(:,(noSrc+1):noChan);          % Splitting in signal and noise subspace.
    for n =1:noDoas
      a = A(:,n);
      q = En*En';
      Pspect(n,1) = (a'*a)/(eps+ abs(a'*q(:,1))^2);;
        % eps to avoid divide by zero
    end%for
    Pspect = real(Pspect);  % Remove imaginary part (due to finite precision).
    spect.spectrumType = 'pseudoSpectrum';


  % ----------------------------------------------------------------------- %
  % Conventional beamforming
  %
  % Start: 9xxxxx Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'cbf'))
    taperType = in8;
    taperParam = in9;
    if (strcmp(taperType,'norm'))
      for n =1:noDoas
        a = A(:,n);
        Pspect(n) = (a'*Rxx*a)/(a'*a);
      end%for
    else
      taperVar = gettap1(antenna, taperType, taperParam);
      for n =1:noDoas
        w = A(:,n) .* taperVar;
        Pspect(n) = w'*Rxx*w;
      end%for
    end%if

    Pspect = real(Pspect);  % Remove imaginary part (due to finite precision).
    spect.spectrumType = 'powerSpectrum';


  % ----------------------------------------------------------------------- %
  % Capon
  %
  % Start: 9xxxxx Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'capon'))
    Pspect = 1./(diag(A'*inv(Rxx)*A));
      % In this calculation, the correlation a(doa1)'*Rxx*a(doa2) is calculated
      % for all doa1 and doa2. Since the cross correlations are not used
      % this is an unnecessary and time consuming calculation.
    Pspect = real(Pspect);   % Remove imaginary part (due to finite precision).
    spect.spectrumType = 'powerSpectrum';
    %if ((rangeLoop == 100) | (rangeLoop == 200))
    %  keyboard
    %end%if


  % ----------------------------------------------------------------------- %
  % MUSIC
  %
  % Start: 97xxxx Fredrik Ahtley (freath).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'music'))
    if (isempty(in8))
      in8 = 5;  % This should be a large enough number.
    end
    noSrc = in8;

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.
    En=U(:,(noSrc+1):noChan);  % Splitting in signal and noise subspace.
    Pspect = sum((abs(A)).^2) ./ ( eps+sum( (abs(En.'*conj(A))).^2 ) ); 
      % eps to avoid divide by zero
    Pspect = Pspect.';
      % Should it be .' ? The complex conjugate does not mattes since Pspect
      % is real.
    spect.spectrumType = 'pseudoSpectrum';


  % ----------------------------------------------------------------------- %
  % MUSIC 2
  % 
  % See [6] p. 10. and [7] p. 384.
  %
  % Start: 990531 Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'music2'))
    if (isempty(in8))
      in8 = 11;  % This should be a large enough number.
    end
    noSrc = in8;

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.
    eigVects=U(:,(noSrc+1):noChan);  % Splitting in signal and noise subspace.
    eigValWeights = ones(noChan-noSrc,1);
    Pspect = calcSpect(A, eigVects, eigValWeights);
    spect.spectrumType = 'powerSpectrum';



  % ----------------------------------------------------------------------- %
  % Capon 2
  %
  % See [6] p. 8-9 and [7] p. 382-.
  %
  % Start: 990531 Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'capon2'))

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.
    eigVects = U;
    eigValWeights = 1 ./ diag(S);
    Pspect = calcSpect(A, eigVects, eigValWeights);
    spect.spectrumType = 'powerSpectrum';



  % ----------------------------------------------------------------------- %
  % Eigenvector method
  % 
  % See [6] p. 9 and [7] p. 384.
  %
  % Start: 990531 Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'eigvec'))
    if (isempty(in8))
      in8 = 5;  % This should be a large enough number.
    end
    noSrc = in8;

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.
    eigVects=U(:,(noSrc+1):noChan);  % Splitting in signal and noise subspace.
    eigValWeights = 1 ./ diag(S);
    eigValWeights = eigValWeights( (noSrc+1):noChan );
    Pspect = calcSpect(A, eigVects, eigValWeights);
    spect.spectrumType = 'powerSpectrum';



  % ----------------------------------------------------------------------- %
  % Pisarenko Harmonic Decomposition
  % 
  % See [6] p. 10. and [7] p. 371.
  % This method does not seem to work.
  %
  % Start: 990531 Svante Bjrklund (svabj).
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'pisa'))
    %if (isempty(in8))
    %  in8 = 9;  % This should be a large enough number.
    %end
    %noSrc = in8;

    [U,S,V]=svd(Rxx);   % Eigendecomp. of correlation matrix using SVD.

    eigVects=U(:,noChan);  % Splitting in signal and noise subspace.
    %eigVects=U(:,(noSrc+1):noChan);  % Splitting in signal and noise subspace.
    %eigVects=U(:,11);  % Splitting in signal and noise subspace.

    eigValWeights = 1;
    %eigValWeights = ones(noChan-noSrc,1);
    %eigValWeights = ones(1,1);

    Pspect = calcSpect(A, eigVects, eigValWeights);
    spect.spectrumType = 'powerSpectrum';



  % ----------------------------------------------------------------------- %
  % Maximum Entropy Method / Yule-Walker AR Method / linear
  %
  % Start: 980927 Svante Bjrklund (svabj).
  %
  % Can only be used with isotropical ULA.
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'pmem') | strcmp(method,'pyulear'))
    dbterror('The method "pmem" or "pyulear" is not implemented.')
    if (isempty(in8))
      in8 = 5;  % This should be a large enough number.
    end
    noSrc = in8;

    [Pspect, freq] = pyulear(Rxx,noSrc,[],'corr');
    smplPoints = 1;  % asin(freq)*???
    spect.spectrumType = 'pseudoSpectrum';



  % ----------------------------------------------------------------------- %
  %
  % ----------------------------------------------------------------------- %
  elseif (strcmp(method,'dummy'))


  % ----------------------------------------------------------------------- %
  % The else branch.
  % ----------------------------------------------------------------------- %
  else
    error('DBT-Error: Selected method is not implemented.')
  end%if


  % ----------------------------------------------------------------------- %
  % Return data.
  % ----------------------------------------------------------------------- %

  infoStr = sprintf('doaspc1: cpiLoop = %d(%d), rangeLoop = %d(%d)\r',...
    cpiLoop, length(cpiIx), rangeLoop,length(rangeIx));
  dbtinfo(infoStr,1);

  spect.specSmpl(:,rangeLoop, 1, cpiLoop) = Pspect;

end%for rangeLoop
end%for cpiLoop
dbtinfo('');	%New line.

%spect.doaPos = smplPoints;
%spect.ranIx = rangeIx;
%spect.cpiIx = cpiIx;

%endfunction doaspc1


function Pspect = calcSpect(A, eigVects, eigValWeights)
  mellan = (abs( eigVects.'*conj(A) )).^2;
  Pspect = 1 ./ (eps+sum(weightrows(mellan, eigValWeights))); 
  Pspect = Pspect.';
%endfunc 
