function sigOut = dopplerfilt(sigIn, freqChoice, taperCoeff)

%DOPPLERFILT Conventional Doppler filtering.
%
%--------
%Synopsis:
%  sigOut = dopplerfilt(sigIn)
%  sigOut = dopplerfilt(sigIn, freqChoice)
%  sigOut = dopplerfilt(sigIn, freqChoice, taperCoeff)
%
%Description:
%  Conventional (linear) Doppler filtering.
%
%Output and Input:
%  sigOut (RxRadarSigT): Output radar signal.
%
%  sigIn (RxRadarSigT): Input radar signal.
%  freqChoice [D](RealVectorT): The Doppler frequency in the Doppler channels.
%  subArrPoint [D](DoaT): Subarray pointing direction. (Default = 0).
%  taperCoeff [D](CxVectorT): The tapering to use.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  This function is not tested yet.
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  pulfilt


%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 980913 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:20:47 $ $Author: svabj $.
%  $Revision: 1.2 $
% *****************************************************************************


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

if (nargin < 1)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************

arginNo=2;
if (nargin < arginNo)
  freqChoice = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  taperCoeff = [];
end
arginNo = arginNo +1;


% ****************** Default values ******************

if isempty(freqChoice)
  freqChoice = [];
end%if
if isempty(taperCoeff)
  taperCoeff = [];
end%if


% ************* Pick out some more fields from input parameters. *************


% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT')
chkdtype(freqChoice.', 'RealVectorT','EmptyT')
chkdtype(taperCoeff, 'CxVectorT','EmptyT')


% ----------------------------------------------------------------------- %
% Doppler filt.
% ----------------------------------------------------------------------- %


sigOut = pulfilt(sigIn, 'acbf', {sigIn.antenna, ...
  sigIn.waveform}, [], freqChoice, taperCoeff);
