function sigOut = msigmat2(sigIn, noPulses)

%MSIGMAT2 Splits the time dimension of the signal into pulse and range dimensions.
%
%--------
%Synopsis:
%  function sigOut = msigmat2(sigIn, noPulses)
%
%Description:
%  Splits the time dimension of the signal into two dimensions where the
%  first represents different pulses for the same ranges and the second
%  represents different ranges for the same pulses. This is necessary when
%  a radar signal is delivered as one long time signal.
% 
%  The function "sigsplitdim" is a more general function that splits
%  dimensions of radar signals and is recomended to use instead of this
%  function "msigmat2".
%
%Output and Input:
%  sigOut (RxCompSigT): Output signal where the split is done.
%  sigIn (RxRadarSigT): Input signal where the range dimension is 1.
%  noPulses (IntScalarT): The number of pulses (= the number of PRI:s).
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  This function has been tested by test 103 in the test script "tdbt.m".
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  sigsplitdim

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%  Start        : 961105 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:21:10 $ $Author: svabj $.
%  $Revision: 1.13 $
% *****************************************************************************

chkdtype(sigIn, 'RxCompSigT','RxRadarSigT')
signalField = sigIn.signals;
chkdtype(noPulses, 'IntScalarT')

ssf = sizem(signalField);
if (rem(ssf(1,1),noPulses) ~= 0)
  error('DBT-Error: The total  number of snapshots is not a multiple of the PRI length.')
end
if (ssf(1,2) ~= 1)
  error('DBT-Error: There are already more than one distance.')
end

nssf = [noPulses, fix(ssf(1,1)/noPulses), ssf(1,3), ssf(1,4), ssf(1,5)];
sigOut = sigIn;
sigOut.signals = reshapem(signalField, nssf);
