function spect = pantpat3(antenna, mainPointDoa, subPointDoa, antError, taperType, taperParam,  iNCorrMat, smplPoints, lambda, sigType, sigPart, minValuedB, scaleType, normType,  plotType, plotOpt, extra1, extra2, extraParam)

%PANTPAT3 Plots a simulated (mechanical) antenna pattern.
%
%--------
%Synopsis:
%  pantpat3(antenna)
%  pantpat3(antenna, mainPointDoa)
%  pantpat3(antenna, {beamWeights})
%
%  spect = pantpat3(antenna, mainPointDoa, subPointDoa, antError, taperType, 
%    taperParam,  iNCorrMat, smplPoints, lambda, sigType, sigPart, minValuedB, 
%    scaleType, normType,  plotType, plotOpt, extra1, extra2)
%
%  spect = pantpat3(antenna, {beamWeights}, subPointDoa, antError, taperType,
%    taperParam,  iNCorrMat, smplPoints, lambda, sigType, sigPart, minValuedB, 
%    scaleType, normType,  plotType, plotOpt, extra1, extra2)
%
%Description:
%  Plots one or more simulated (mechanical) antenna patterns. Antenna
%  patterns using signals (simulated or measured) can not be done with this
%  function.
%
%  There are two different ways to specify the beamforming weights to use:
%  1) Specifying one or more pointing directions of the main antenna using the
%  input parameter "mainPointDoa", in which case the steering vectors for
%  these directions are taken as the weights.
%  2) Supplying the weights directly as the input parameter "beamWeights".
%  Note, that in this case the weights must be a matrix surounded by { }, 
%  i.e.  the only element of a cell array. The reason for the cell array
%  is simply to distinguish between beam pointing directions and beam weights. 
%
%  In boths ways, tapering of main antenna can be choosen and adaptive side
%  lobe cancellation can be used.
%
%
%  There are two possible antenna patterns for an array antennna, either with
%  mechanical angle scan or electronical angle scan. The antenna patterns are
%  generated by the same formulas:
%
%    w(thetaEl) = inv(iNRxx) * (a(thetaEl).*taper)
%    P(thetaMec, thetaEl) = abs(a(thetaMek)' * w(thetaEl))^2
%
%  where "w" is the weight vector to use in the beamforming,
%  "iNRxx" is the input antenna signal interference-plus-noise correlation
%  matrix and a() is the steering vector (a model of the antenna response).
%
%  Depending on which of thetaMec and thetaEl that is beeing varied while
%  the other is held constant, the mechanical or the electrical pattern is
%  generated.
%
%  The function pantpat3 generates the mechanical pattern. Model errors are
%  taken in account when calculating a(thetaMek) but not when calculating
%  a(thetaEl).
%
%Output and Input:
%  spect [D](DoaSpecT): Output DOA-spectrum. It is a quadratic measure of
%    the sensity of sources from different directions or more specific the
%    squared modulus of the antenna patterns P above. The scale is linear.
%    There can be several antennas patterns, in which case different columns
%    in "spect.specSmpl" means different patterns.
%  antenna (AntDefT): Antenna definition.
%  mainPointDoa [D](DoaT): Pointing direction of main antenna or array.
%    Default is zero. If this input parameter is used, the beamforming
%    weights are calculated using the spatial steering vector directed
%    in the direction of "mainPointDoa". Tapering (input paremeters
%    "taperType" and "taperParam") and adaptive interference supression
%    (input paremeters "iNCorrMat") are then applied.
%  beamWeights [D](CxMatrixT): Beamforming weights. If this input parameter
%    is used, the beamforming weights are given by this input parameter as
%    a (noElems x noBeams) complex matrix. Note, the matrix must be
%    surounded by { }.The beamforming is done via (signal'*w ). Note,
%    the hermitian transpose.
%    (Should it be w'*signal instead?)
%  subPointDoa [D](Vector of DoaT): Pointing direction of subarrays if
%    the function "spastemat" allows this for the used antenna type. Default
%    is zero.
%  antError [D](AntDefT): Antenna definition with errors. This is a model of
%    the real signal.
%  taperType [D](StringT): Tapering type = the parameter "taperType" of
%    function "gettap1".
%  taperParam [D](?): The third input parameter of function "gettap1".
%  iNCorrMat (RxCorrMatT): The input antenna signal interference-plus-noise
%    correlation matrix. This should be estimated on a radar signal without
%    targets. Use this if you want antenna pattern adapted to supress
%    jamming or other interference.
%  smplPoints [D](Vector of DoaT): Calculate the antenna diagram at these
%    angles.
%  lambda (RealScalarT): Wavelength [m].
%  minValuedB [D](RealScalarT): Lowest value in dB to plot for a quantity
%    relative the maximum value.
%  scaleType [D](): Not used.
%  normType [D]: Type of normalization. Only used for "plotType" 'lin' & 'sin'.
%    = 'max': Normalize the maximum of the signal to 1 (or 0 dB).
%    = 'none' : No normalization is done.
%  plotType [D](StringT):
%    = 'lin': Rectangular plot with linear axis for the direction (default).
%    = 'sin': Rectangular plot with sin(direction) for the direction axis.
%      Not implemented.
%    = 'polar': Polar plot using Matlabs function "polar".
%    = 'lppolar': Polar plot using Lars Petterssons function "lppolar"
%    = 'noplot' : No plot, only calculation of antenna pattern.
%  plotOpt [D](): This input parameter is sent to the "S" input parameter of
%    the functions "plot" and "polar". Various line types, plot symbols and
%    colors are defined in this way. See the "S" input parameter of
%    the functions "plot" for more information.
%  extra1 (RealVectorT): The elements of this input parameter is sent
%    to the input parameters "rmax", "rmin", "rticks", "fimax", "fimin"
%    and "fiticks" of the function "lpplot".
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%  These examples are the same as in the file dbtex15.m.
%  Example 1:
%    This example shows an antenna pattern without and with adaptive
%    sidelobe cancellation.
%    figure
%    theta  = d2r([40 60])';
%    lambda = 1;
%    ant = defant('isotropULA',[12, 0.45*lambda, lambda]);   % Define antenna.
%    sigJam = compsim2(ant, 1, 24, 'const', [theta,zeros(size(theta)),...
%      [10 5]', d2r([0 16])', d2r([34 -13])', Inf*ones(size(theta)), ...
%      eye(size(theta,1))], 'rndn', eye(12));
%        % Generate simulated jammer signals
%    Rjam = ecorrm(sigJam);
%    hold on
%    pantpat3(ant, d2r(10), [],[], 'taylor', 30)
%      % Without adaptive sidelobe cancellation.
%   pantpat3(ant,  d2r(10), [],[], 'taylor', 30, Rjam, [], [],[],[],[], ...
%      [],[],[],'r')
%      % With adaptive sidelobe cancellation.
%    legend('Without SLC', 'With SLC')
%
%  Example 2:
%    This example shows how the pattern of subarrays and elements can
%    be plotted.
%    lambda = 0.1;
%    ant = defant('aimtEx');
%    figure, hold on
%    pantpat3(ant, [], [], [], [], [], [], [], lambda)
%    subarr = ant.element;
%    pantpat3(subarr, [], [], [], [], [], [], [], lambda, [], [], [],[], [], ...
%      [], 'r')
%    elem = subarr.element;
%    pantpat3(elem, [], [], [], [], [], [], [], lambda, [], [], [],[], [], ...
%      [], 'b')
%    legend('Total', 'Subarrays', 'Elements')
%
%  Example 3:
%    This example shows two different polar plots. A user defined tapering
%    is used.
%    tap = cos(([-5.5:1:5.5].')./(5.5)*(pi/2)).^2;
%      % A cos^2 tapering
%    lambda = 1;
%    ant = defant('isotropULA',[12, 0.5*lambda, lambda]);
%    figure
%    s1 = pantpat3(ant, d2r(30), 0, [], 'user', tap, [], [], [], ...
%      [],[],-70,[],[],'polar');
%    figure
%    s1 = pantpat3(ant, d2r(30), 0, [], 'user', tap, [], [], [], ...
%      [],[],-70,[],[],'lppolar',[],[0,-60, 6,pi/2,-pi/2,6]);
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%
%Known Bugs:
%  Should the "beamSpaceTrans" field of the top level in the antenna
%  definition be used as tapering? Now that field is not used automatically
%  but can be used manually via the "taperType" and "taperParam" input
%  parameters of this function.
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  defant, polar, lppolar

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 980108 Svante Bjrklund (svabj).
%  Latest change: $Date: 2001/08/31 14:33:35 $ $Author: svabj $.
%  $Revision: 1.30 $
% *****************************************************************************




% NOTE: The following help text is currently not used.
%
%function spect = pantpat3(antenna, mainPointDoa, subPointDoa, antError, taperType, taperParam,  iNCorrMat, smplPoints, focusDist, sigType, sigPart, minValuedB, scaleType, normType,  plotType, plotOpt, extra1)

%  focusDist [D](RealScalarT): Focus distance of the antenna [m]
%    (default = Inf). Not implemented.
%

%  pantpat3(antenna, mainPointDoa, subPointDoa)
%  pantpat3(antenna, [], [], [], taperType, taperParam)
%  pantpat3(antenna, [], [], [], [], [],  iNCorrMat)
%  pantpat3(antenna, [], [], [], [], [],  [], smplPoints)
%  pantpat3(antenna, [], [], [], [], [],  [], [], lambda)
%  pantpat3(antenna, [], [], [], [], [],  [], [], [], [], [], [], [], [],
%    plotType)
%  pantpat3(antenna, [], [], [], [], [],  [], [], [], [], [], minValuedB,
%    [], [], plotType)
%  pantpat3(antenna, [], [], [], [], [],  [], [], [], [], [], [], [], [],
%    plotType, plotOpt)
%  pantpat3(antenna, mainPointDoa, subPointDoa, antError, taperType,
%    taperParam,  iNCorrMat, smplPoints, lambda, sigType, sigPart,
%    minValuedB, scaleType, normType,  plotType, plotOpt, extra1)

%
%  spect = pantpat3(antenna, mainPointDoa, subPointDoa, antError, taperType,
%    taperParam,  iNCorrMat, smplPoints, focusDist, sigType, sigPart, scaleType, normType, minValue, plotType, dispOpt)
%
%, maxValue, xAxisType)
%
%  sigType [D](StringT): Signal type.
%    = 'sig': The signal itself (default).
%    = 'fft': FFT of the signal.
%  sigPart [D](StringT): What part of the signal to plot.
%    = 'power': (default).
%    = 'abs': Absolute value.
%    = 'angle': Phase angle with Matlab:s "angle()" function.
%    = 'uwangle': Phase angle with Matlab:s "unwrap(angle())" function.
%       This gives a continous phase angle.
%    = 'real': Real part.
%    = 'imag': Imaginary part.
%  scaleType [D](StringT) : Scale Type for vertical axis.
%    = 'lin': Linear scale.
%    = 'dB': dB scale.
%  normType: Type of normalization. Not implemented.
%  maxValue: Hsta niv som ska plottas. Behvs denna fr antenndiagram? Vi har ju inga modellbaserade metoder.
%  xAxisType = 'lin', 'sin' (ej fr plolr plot.)

%Possible extensions:
%  "axis" for the polar plots. Especially the rmax value.
%  Written angle values for lpplot.
%  Written angle values -180 .. 180 degrees instead of 0 .. 360 degrees.
%  Normalisation to antenna gain. Maybe possible to choose zero dB level.


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

if (nargin < 1)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************

arginNo=2;
if (nargin < arginNo)
  mainPointDoa = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  subPointDoa = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  antError = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  taperType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  taperParam = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  iNCorrMat = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  smplPoints = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  lambda = [];
end
arginNo = arginNo +1;

if (nargin < arginNo)
  sigType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  sigPart = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  minValuedB = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  scaleType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  normType = [];
end
arginNo = arginNo +1;

if (nargin < arginNo)
  plotType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  plotOpt = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  extra1 = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  extra2 = [];
end
arginNo = arginNo +1;


% *************** Evalute the list extraParam of keywords  ***************
% These the value of this keywords are assigned to the local variables with
% the same name as the keywords. The use of keywords is to make calls to
% this function simpler.

if (nargin >= arginNo)
  lenExtra = length(extraParam);
  if (mod(lenExtra,2) ~= 0), error('DBT-Error: Value missing.'),end%if
  for n = 1:2:length(extraParam)
    eval([extraParam{n} '= extraParam{n+1};' ]);
  end%for n
end
arginNo = arginNo +1;



% *************** Pick out some fields from input parameters. ***************

noElem = antenna.noElem;


% ****************** Default values ******************

if isempty(lambda)
  %lambda = 1;
  dbterror('Input parameter "lambda" must be specified.')
end%if
if isempty(mainPointDoa)
  mainPointDoa = 0;
end%if
if isempty(subPointDoa)
  subPointDoa = [];
end%if
if isempty(antError)
  antError = antenna;	% No errors. Use the ideal antenna model.
end%if
if isempty(taperType)
  taperType = [];
end%if
if isempty(taperParam)
  taperParam = [];
end%if
if isempty(iNCorrMat)
  iNRxx = eye(noElem);
else
  iNRxx = iNCorrMat.corrMat;
    % Get the interference plus noise antenna signal correlation matrix.
end%if
if isempty(smplPoints)
  antennaType = antenna.antennaType;
  if (strcmp(antennaType,'ULA'))
    beamWidth = lambda/(noElem*antenna.distElem);
    smplPoints = -pi/2:(beamWidth/20):pi/2;
  else
    smplPoints = asin(-1:0.005:1);
  end
end%if
if isempty(sigType)
  sigType = 'sig';
end%if
if isempty(sigPart)
  sigPart = 'abs';
end%if
if isempty(scaleType)
  scaleType = 'dB';
end%if
if isempty(normType)
  normType = 'max';
end%if
if isempty(minValuedB)
  minValuedB = -100;
end%if

if isempty(plotType)
  plotType = 'lin';
end%if
if isempty(plotOpt)
  plotOpt = [];
end%if

focusDist = Inf;


% ****************** Error check input parameters ******************

%chkdtype(mainPointDoa, 'DoaT')
%chkdtype(subPointDoa, 'Vector of DoaT')
%chkdtype(taperType, 'StringT')
%chkdtype(iNCorrMat, 'RxCorrMatT')
chkdtype(smplPoints, 'Vector of DoaT')
chkdtype(lambda, 'RealScalarT')

if (strcmp(antenna.name,'expAnt'))
  disp('Warning: The antenna pattern for this antenna type is not plotted using measured data as it should be. A model is used instead.')
end%if


% ----------------------------------------------------------------------- %
% Calculate the antenna pattern.
% ----------------------------------------------------------------------- %

taperVar = gettap1(antenna, taperType, taperParam);
if (iscell(mainPointDoa))
  % If Supplying the weights directly as the input parameter "beamWeights".
  aModel = mainPointDoa{1};
else
  % Specifying one or pointing directions of the main antenna using the
  % input parameter "mainPointDoa".
  aModel   = spastemat(antenna, mainPointDoa, lambda, focusDist, subPointDoa);
             % The antenna response according to the used signal model for
	     % the choosen point direction.
end%if
sd  = weightrows(aModel,taperVar);
             % Desired response including tapering.

w = inv(iNRxx) * sd;
             % Beamforming weights including side lobe cancellation.
aSig = spastemat(antError,smplPoints,lambda,focusDist,subPointDoa);
             % A more real antenna response (model + model errors) for a
	     % signal inpinging from directions "smplPoints". This is a
	     % "mechanical scan".
antPatOrg = aSig'*w;
             % Create the antenna diagram.
	     % Shouldn't it be w'*aSig ?

%aModel = aModel./aModel(1,1);
%aSig = aSig./aSig(1,1);

% ----------------------------------------------------------------------- %
% Prepare the antenna pattern for plotting.
% ----------------------------------------------------------------------- %

antPatPlot = antPatOrg;
  % "antPatPlot" is for plotting, while (the squared modulus of) "antPatOrg"
  % is delivered as the output parameter "spect".

if (0)
  % Not used yet.
  maxValueB = Inf;
  %antPatPlot = sigprep(antPatPlot,sigType, sigPart, minValuedB, maxValueB, ...
  %  scaleType, normType);
  %  % This function does not exist yet.
end%if

% ************** Set values below "minValueLin" to "minValueLin" *************

  % antPatPlot (CxMatrix): One or more antenna patterns in the columns in
  %   linear scale.
  antPatPlot = abs(antPatPlot);
  maxAbsAntPatPlot = max(antPatPlot);
    % For matrices, MAX(X) is a row vector containing the maximum element
    % from each column.
  minValueLin = 10^(minValuedB/20);
  antPatPlot = weightcols(antPatPlot,1./maxAbsAntPatPlot);
    % Normalizing the patterns so that maximum is equal to 1.
  for n=1:size(antPatPlot,2)
    indexVector = (antPatPlot(:,n) < minValueLin);
    antPatPlot(indexVector,n) = ones(sum(indexVector),1)*minValueLin;
  end%for n

  %antPatPlot = 20*log10(antPatPlot);


% ************** Normalize  *************

switch plotType
case {'lin', 'sin'}
  %antPatPlot = weightcols(antPatPlot,1./max(antPatPlot));
  %  % Already done above.
  if (strcmp(normType,'none'))
    antPatPlot = weightcols(antPatPlot,maxAbsAntPatPlot);
    % Undo the normalization done above.
  end%if
case 'polar'
  antPatPlot = weightcols(antPatPlot,1./min(antPatPlot));
case 'lppolar'
  % Do nothing;
case 'noplot'
  antPatPlot = weightcols(antPatPlot,maxAbsAntPatPlot);
  % Undo the normalization done above.
otherwise
  dbterror('Unknown plotType')
end%switch


% ************** Convert to deciBel *************

antPatPlot = 20*log10(antPatPlot+eps);     % "+eps" to avoid log of zero.


% ----------------------------------------------------------------------- %
% Plotting of the antenna pattern.
% ----------------------------------------------------------------------- %

if strcmp(plotType,'lin')  
    % 'lin' here does not mean that the y-axis has a linear scale but
    % the x-axis has a linear scale.
  if isempty(plotOpt)
    plot(r2d(smplPoints), antPatPlot)
  else
    plot(r2d(smplPoints), antPatPlot, plotOpt)
  end%if
  grid on;
  xlabel('azimuth angle [degrees]');
  ylabel('[dB]')

elseif strcmp(plotType,'sin')
  % Rectangular plot with sin(direction) for the direction axis.
  if isempty(plotOpt)
    plot(sin(smplPoints), antPatPlot)
  else
    plot(sin(smplPoints), antPatPlot, plotOpt)
  end%if
  grid on;
  xlabel('azimuth angle [sin(degrees)]');
  ylabel('[dB]')

elseif strcmp(plotType,'polar')
  if isempty(plotOpt)
    polar(smplPoints, antPatPlot.')
  else
    polar(smplPoints, antPatPlot.',plotOpt)
  end%if

elseif strcmp(plotType,'lppolar')
  lppolar(smplPoints, antPatPlot.', extra1(1), extra1(2),extra1(3), ...
    extra1(4), extra1(5), extra1(6))

elseif strcmp(plotType,'uv')
  % Plotting of a 2D antenna pattern in uv-coordinates.
  % There should be several plot types for 2D antenna patterns.

elseif strcmp(plotType,'noplot')
  % Do nothing

else
  error('DBT-Error: Unknown plot type.')
end%if

if ~strcmp(plotType,'noplot')
  title('Antenna pattern')
end%

% ----------------------------------------------------------------------- %
% Create output variable.
% ----------------------------------------------------------------------- %
if (nargout > 0)
  spect.dataType = 'DoaSpecT';
  spect.version = 2;
  spect.spectrumType = 'Antenna Pattern';
  spect.specSmpl = abs(antPatOrg).^2;
  spect.doaPos = smplPoints;
end%if
