function pptmfq1(signals, noTgt, snapshot, smplPoints)

%PPTMFQ1 Plots the error function Q for the PTMF method.
%
%--------
%Synopsis:
%  function pptmfq(signals, noTgt, snapshot, smplPoints)
%
%Output and Input:
%  signals  (RxRadarSigT) : The 'true' signals.
%  noTgt  (RealScalarT) : No. of targets. Have to be supplied.
%  snapshot (RealScalarT) : Plot for this snapshot.
%  smplPoints (Vector of DoaT)  : Plot for these doas. Not implemented yet.
%
%Description
%  Plots the error function Q for the standard antenna signal model:
%   x(t) = A(Theta)*b(t) + n(t)
%  The prediction is xhat(t) = A(Thetahat)*bhat, bhat(t)=pinv(A(Thetahat))*x(t)
%  The error function Q = (x(t)-xhat(t))'*(x(t)-xhat(t))
%  x(t) is the signal supplied in the 'signals' input parameter. 'snapshot'
%  chooses the time t. Theta = [theta1, theta2, ...] angles to different
%  targets.
%
%Known Bugs:
%  This function (DBT release 1.11) seems to give the negative values
%  of the angles.
%
%See Also:
%  doapar1, perrfun, dbtex4



% *****************************************************************************
%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%
%  Start        : 970317 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:21:23 $ $Author: svabj $.
%  $Revision: 1.13 $
% *****************************************************************************

% ----------------------------------------------------------------------- %
% Pick out fields from input parameters.
% ----------------------------------------------------------------------- %

chkdtype(signals, 'RxCompSigT','RxRadarSigT')
X   = getm(signals.signals, ':',1,':',1,1)';
T   = size(X,2);    % Number of snapshots == number of iterations.

ant   = signals.antenna;

chkdtype(ant, 'AntDefT')
K   = ant.noElem;
D   = ant.distElem;
%lambda  = ant.lambda;
waveform = signals.waveform;
lambda  = waveform.wavelength;


if (nargin >= 4)
  error('DBT-Error: smplPoints is not implemented yet.')
end
if (nargin < 4)
  smplPoints = [];
end
if isempty(smplPoints)
  smplPoints = d2r(-90:0.5:90);
end


if (noTgt == 1)
  maxth2 = 1; % Maximum number of angle steps for the second angle.
elseif (noTgt == 2)
  maxth2 = 30;  % Maximum number of angle steps for the second angle.
else
  error('DBT-Error: Only possible to plot Q for one or two angles.')
end%if


%-------------------------------------------------------------------------- %
% Calculate the Q-function.
% ------------------------------------------------------------------------- %

% Qniva2.m Ritar graf p felfunktionen Q fr tv mlvinklar.

maxth1 = 30;      % Maximum number of angle steps for the first angle.

th1steg = d2r(1);   % Vinkelavstnd mellan vinkelstegen.
th2steg = d2r(1);


if ((maxth1/2)*th1steg > pi) | ((maxth2/2)*th2steg > pi/2)
  error('DBT-Error: Angle outside +- 90 degrees');
end%if
t=snapshot;           % Choose snapshot to plot Q for.
Q = zeros(maxth1,maxth2);

for th1 = 1:maxth1
    for th2 = 1:maxth2
  vinkel1 = [(th1-maxth1/2)*th1steg];
  vinkel2 = [(th2-maxth2/2)*th2steg];
  vinkel  = [vinkel1; vinkel2];

  %Amatris = styrMatris(D/lambda, vinkel, N);
  %bbhat = pinv(Amatris)*Z(:,t);
  %zTildePost = Z(:,t) - Amatris*bbhat;

  Amatris = spastemat(ant, vinkel', lambda);
  bhat(:,t) = pinv(Amatris)*X(:,t);  % Estimation of the target
             %  signals bhat(t).
  zTildePost = X(:,t) - Amatris*bhat(:,t); % Felvektor.
  Q(th1,th2) = zTildePost' * zTildePost;
    end%for th2
end%for th1

%-------------------------------------------------------------------------- %
% Plot the Q-function.
% ------------------------------------------------------------------------- %

if (noTgt == 1)

  figure;
  plot(r2d(((1:maxth1)-maxth1/2)*th1steg), max(10*log10(Q),-50))
  title('Error function Q [dB]')
  xlabel('theta1 [degrees]')
  grid

  figure;
  plot(r2d(((1:maxth1)-maxth1/2)*th1steg), Q)
  title('Error function Q [linear]')
  xlabel('theta1 [degrees]')
  grid

elseif (noTgt == 2)

  figure;
  xaxel = r2d(((1:maxth2)-maxth2/2)*th2steg);
  yaxel = r2d(((1:maxth1)-maxth1/2)*th1steg);
  c3 = contour(xaxel, yaxel, 10*log10(Q),[-20:1:20]);
  %clabel(c3)
  title('Error function Q [dB]')

  figure;
  c4 = contour(xaxel, yaxel, Q);
  %clabel(c4)
  title('Error function Q [linear]')

  xlabel('theta2 [degrees]')
  ylabel('theta1 [degrees]')
  grid

  figure;
  meshc(xaxel, yaxel, max(max(Q))-Q)
  title('Error function Q turned upside down')
  xlabel('theta2 [degrees]')
  ylabel('theta1 [degrees]')
  grid
  axis('square')

end%if
