function [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, timeFilter, carrierFreq, bandwidth, noTime, noTrials, propSpeed, srcType, srcSignals, srcPowers, srcDoas, srcRanges, srcCorr, notUsed, noiseType, noisePower, noiseCorr, orthSig, corrMethod, testParam)

%SIMWIDEBANDKERNEL Simulates wide-band radio signals received by array antennas.
%
%--------
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    timeFilter, carrierFreq, bandwidth, noTime, noTrials, propSpeed,
%    srcType, srcSignals, srcPowers, srcDoas, srcRanges, srcCorr, [], 
%    noiseType, noisePower, noiseCorr, orthSig, corrMethod)
%
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    timeFilter, carrierFreq, {lowOffFreq, highOffFreq}, noTime, noTrials, 
%    propSpeed, srcType, srcSignals, srcPowers, srcDoas, srcRanges,
%    {srcCorrSrc, srcCorrTime}, [], noiseType, noisePower, 
%    {noiseCorrChan, noiseCorrTime}, orthSig, corrMethod)
%
%Description:
%  Simulator for wide-band radio signals received by array antennas.
%
%Output and Input:
%  sigOut (RxRadarSigT): Simulated radio signal. As the sample time in
%    the waveform, sampleTime =  1/(freqHigh-freqLow) is chosen, which
%    is equal to the reciprocal of the absolute bandwidth.
%    As the wavelength, carrierWavelength = propSpeed / carrierFreq is 
%    chosen.
%  sourceSigTime (CxMatrixT): Complex envelope of the simulated source
%    signals in the time domain in a (noSrc, noTime) complex matrix.
%  noiseSigTime (CxMatrixT): Complex envelope of the simulated noise
%    signals in the time domain in a (???, ???) complex matrix.
%
%  antenna (AntDefT): Antenna definition for the receiver.
%  carrierFreq [D](RealScalarT): Carrier frequency fRF [Hz]. This can be
%    specified to zero for baseband signals.
%  bandwidth [D](RealScalarT): Absolute bandwidth B [Hz]. It will be
%    centered around the carrier frequency fRF. The frequency region will
%    be (fRF - B/2) <= f <= (fRF - B/2). The condition (B < 2*fRF) must be 
%    satisfied. How does this correspond with "baseband signals" in the
%    help text of "carrierFreq"?
%  lowOffFreq [D](RealScalarT): Low frequency offset fBL [Hz]. The condition 
%    (fRF - fBL) > 0 must be satisfied. How does this correspond with 
%    "baseband signals" in the help text of "carrierFreq"?
%  highOffFreq [D](RealScalarT): High frequency offset fBH [Hz]. The frequency 
%    region will be (fRF + fBL) <= f <= (fRF + fBH).
%  noTime [D](IntScalarT): Number of time samples to simulate of the signals.
%  noTrials [D](IntScalarT): Not implemented.
%  propSpeed [D](RealScalarT): Propagation speed of the waves [m/s].
%    Default = speed of light in free space.
%  srcType (StringT): Type of source signals. See below for sections about
%    each type.
%    = 'det': Deterministic Signals.
%    = 'randn': Normally distributed random amplitudes.
%    = 'rand': Uniformly distributed random amplitudes. Not implemented.
%    = 'nosrc': No source signals are simulated and added.
%  srcSignals [D](CxMatrixT): Complex envelope of the source time signals
%    in a (noTime, noSrc)-matrix. noSrc = number of sources.
%  srcPowers [D](RealVectorT): Source signal powers in a (noSrc x 1) vector.
%    Where are the powers measured?
%  srcDoas [D](Vector of DoaT): Source signal direction-of-arrival.
%  srcRanges [D](): Source signal ranges. Not implemented.
%  srcCorr [D](): Not implemented.
%  srcCorrSrc [D](CxMatrixT): Correlation matrix for the correlation between
%    different signalsources.
%  srcCorrTime [D](CxMatrixT): Not implemented.
%  noiseType [D](StringT): Type of noise signals. See below for sections
%    about each type.
%    = 'randn': Normally distributed random amplitudes.
%    = 'rand': Uniformly distributed random amplitudes. Not implemented.
%    = 'nonoise': No noise is simulated and added.
%  noisePower [D](RealVectorT):
%  noiseCorr [D](): Not implemented.
%  noiseCorrChan [D](CxMatrixT): Correlation matrix for the correlation of
%    the noise between different antenna channels. Not implemented.
%  noiseCorrTime [D](CxMatrixT): Not implemented.
%  orthSig [D](BoolT): 1= orthogonalization of the signals. 0 = no
%    orthogonalization (default). Implementation not finnished.
%  corrMethod [D](BoolT): Implementation not finnished.
%  testParam (): For debugging only.
%
%--------
%Signal Source Simulation:
%--------
%Deterministic Signal Sources.
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, bandwidth, noTime, [], propSpeed,
%    'det', srcSignals, srcPowers, srcDoas, srcRanges, [], [], 
%    noiseType, noisePower, noiseCorr, orthSig, corrMethod)
%
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, {lowOffFreq, highOffFreq}, noTime, [], 
%    propSpeed, 'det', srcSignals, srcPowers, srcDoas, srcRanges,
%    [], [], noiseType, noisePower, 
%    {noiseCorrChan, noiseCorrTime}, orthSig, corrMethod)
%
%Output and Input:
%
%Description:
%  As the source signals is the contents of the input parameter "srcSignals" 
%  used. These signals are considered to be deterministic. The number of
%  time samples to simulate is given by the length of the signals in the
%  input parameter "srcSignals".
%
%--------
%Normally Distributed Random Signal Sources.
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, bandwidth, noTime, [], propSpeed,
%    'randn', [], srcPowers, srcDoas, srcRanges, srcCorr, [], 
%    noiseType, noisePower, noiseCorr, orthSig, corrMethod)
%
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, {lowOffFreq, highOffFreq}, noTime, [], 
%    propSpeed, 'randn', srcSignals, srcPowers, srcDoas, srcRanges,
%    {srcCorrSrc, srcCorrTime}, [], noiseType, noisePower, 
%    {noiseCorrChan, noiseCorrTime}, orthSig, corrMethod)
%
%Output and Input:
%
%Description:
%  The source signals are simulated as complex normally (gaussian)
%  distributed random signals.
%
%--------
%No Source Signals.
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, bandwidth, noTime, [], propSpeed,
%    'nosrc', [], [], [], [], [], [], 
%    noiseType, noisePower, noiseCorr, orthSig, corrMethod)
%
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(antenna, 
%    [], carrierFreq, {lowOffFreq, highOffFreq}, noTime, [], 
%    propSpeed, 'nosrc', [], [], [], [],
%    [], [], noiseType, noisePower, 
%    {noiseCorrChan, noiseCorrTime}, orthSig, corrMethod)
%
%Output and Input:
%
%Description:
%  No source signals are simulated and added.
%
%--------
%Noise Simulation:
%--------
%Normally Distributed Noise.
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(...,
%    'randn', noisePower, noiseCorr,...)
%
%Output and Input:
%
%Description:
%  The noise is simulated as a complex normally (gaussian)
%  distributed random signal.
%
%--------
%No Noise.
%Synopsis:
%  [sigOut, sourceSigTime, noiseSigTime] = simwidebandkernel(...,
%    'nonoise', noisePower, noiseCorr,...)
%
%Output and Input:
%
%Description:
%  No noise is simulated and added.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Data Types:
%  AntDefT: Antenna definition. See [1] and help text of function "defant".
%  BoolT: An integer scalar. False = 0, True = 1.
%  CxMatrixT: 2D complex matrix.
%  IntScalarT: Integer scalar. Complex integers are not allowed.
%  RealScalarT: Real scalar.
%  RealVectorT: Real vector. Vectors are (n x 1)-matrices.
%  RxRadarSigT: : Stores a radio or radar signal and related information.
%    See [1] and help text of function "RxRadarSigT".
%  StringT: String.
%
%Examples:
%  See the example programs "wideex1", "wideex2" and "wideex3".
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  This function is not tested yet.
%
%Known Bugs:
%  Should it be possible to also specify the distance to the signals sources?
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Bjrklund S.: "A MATLAB implementation of a Simulator for Wideband
%    Radio Signals Received by Array Antennas", FOA-D--99-00435-504--SE,
%    March 1999.
%
%See Also:
%  compsim4, simradarsig, wideex1, wideex2, wideex3


%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 981211 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:39:35 $ $Author: svabj $.
%  $Revision: 1.13 $
% *****************************************************************************

% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

% ****************** Add missing input parameters ******************
% These staments can be removed without affecting the operation of this
% function.

arginNo=1;
if (nargin < arginNo)
  error('DBT-Error: To few input parameters.')
end
arginNo = arginNo +1;

if (nargin < arginNo)
  timeFilter = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  carrierFreq = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  bandwidth = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noTime = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noTrials = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  propSpeed = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcSignals = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcPowers = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcDoas = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcRanges = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  srcCorr = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  notUsed = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noiseType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noisePower = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noiseCorr = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  orthSig = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  corrMethod = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  testParam = [];
end
arginNo = arginNo +1;


if (0)
% *************** Evalute the list extraParam of keywords  ***************
% These the value of this keywords are assigned to the local variables with
% the same name as the keywords. The use of keywords is to make calls to
% this function simpler.

if (nargin >= arginNo)
  lenExtra = length(extraParam);
  if (mod(lenExtra,2) ~= 0), error('DBT-Error: Value missing.'),end%if
  for n = 1:2:length(extraParam)
    eval([extraParam{n} '= extraParam{n+1};' ]);
  end%for n
end
arginNo = arginNo +1;
end%if (0)


% ****************** Default values ******************
% These staments can be removed without affecting the operation of this
% function.

if isempty(antenna)
  antenna = [];
end%if
if isempty(timeFilter)
  timeFilter = [];
end%if
if isempty(carrierFreq)
  carrierFreq = 0;
end%if
if isempty(bandwidth)
  bandwidth = 0;
end%if
if isempty(noTime)
  noTime = 128;
end%if
if isempty(noTrials)
  noTrials = 1;
end%if
if isempty(propSpeed)
  propSpeed = speedoflight;
end%if
if isempty(srcType)
  srcType = 'randn';
end%if
if isempty(srcSignals)
  if (strcmp(srcType,'det'))
    srcSignals = ones(1,noTime);
  else
    srcSignals = [];
  end%if
end%if
if isempty(srcPowers)
  srcPowers = 1;
end%if
if isempty(srcDoas)
  srcDoas = d2r([0;0]);
end%if
if isempty(srcRanges)
  srcRanges = 1000;
end%if
if isempty(notUsed)
  notUsed = [];
end%if
if isempty(srcCorr)
  srcCorr = {[]};
end%if
if isempty(noiseType)
  noiseType = 'nonoise';
end%if
if isempty(noisePower)
  noisePower = 1;
end%if
if isempty(noiseCorr)
  noiseCorr = {[]};
end%if
if isempty(orthSig)
  orthSig = 0;
end%if
if isempty(corrMethod)
  corrMethod = 0;
end%if
if isempty(corrMethod)
  testParam = [];
end%if


% ************* Pick out some fields from input parameters. *************

noSrc = size(srcDoas,2);	% Number of sources signals.
noChan = antenna.noElem	;	% Number of antenna channels.

%if (iscell(bandwidth))
%  freqLow = carrierFreq + bandwidth{1};
%  freqHigh = carrierFreq + bandwidth{2};
%else
%  freqLow = carrierFreq - bandwidth/2;
%  freqHigh = carrierFreq + bandwidth/2;
%end%if
%freq = linspace(freqLow, freqHigh, noFreq+1);
%freq = freq(1:noFreq);
%  This code is now handled by the function "genfreqaxis".

srcCorrSrc = srcCorr{1};	
  % The correlation between different signal sources.

noiseCorrSrc = noiseCorr{1};	
  % The correlation of the noise between different antenna channels.


% ****************** Error check input parameters ******************
% These tests can be removed without affecting the operation of this function.

chkdtype(srcType, 'StringT')
chkdtype(antenna, 'AntDefT')
chkdtype(carrierFreq, 'RealScalarT')
chkdtype(bandwidth, 'RealScalarT', 'CellArrayT')
chkdtype(noTime, 'IntScalarT')
chkdtype(noTrials, 'IntScalarT')
chkdtype(srcSignals, 'CxMatrixT')
chkdtype(srcDoas, 'Vector of DoaT')
chkdtype(srcPowers, 'RealVectorT')
chkdtype(srcCorr, 'CxMatrixT')
chkdtype(orthSig, 'IntScalarT')
chkdtype(propSpeed, 'RealVectorT')
chkdtype(corrMethod, 'IntScalarT')
chkdtype(noiseType, 'StringT')
chkdtype(noisePower, 'RealVectorT')
chkdtype(noiseCorr, 'CxMatrixT')


% ----------------------------------------------------------------------- %
% Simulation of source signals.
% ----------------------------------------------------------------------- %
    % sourceSigTime (CxMatrixT): Complex envelope of the simulated source
    %   signals in the time domain in a (noSrc, noTime) complex matrix.


    % ****************** Deterministic Signals. ******************

    if (strcmp(srcType,'det'))
      sourceSigTime = srcSignals;
        % Use the signals given by the input parameter "srcSignals".
      noTime = size(sourceSigTime,2);
        % Over-ride the setting of the input parameter "noTime".


    % ****************** Gaussian stochastic signals. ******************

    elseif (strcmp(srcType,'randn'))
      %[U,S,V] = svd(srcCorrSrc);
      %B=(1/sqrt(2))*U*sqrt(S);
        % Compute the square root of the source covariance matrix with SVD.
      %sourceSigTime = B*(randn(noSrc,noTime) + j*randn(noSrc,noTime))

      sourceSigTime = (randn(noSrc,noTime) + j*randn(noSrc,noTime)) / sqrt(2);
        % The variance for the real and imaginary parts are 1.
        % The total variance of the signal is therefore 1+1=2.
        % The signal power is equal to the variance when, as here,
        % the mean value is 0. This is the reason for the normalisation
        % (of the amplitude) with the square root of the power


    % ************* Uniformly distributed stochastic signals. *************

    elseif (strcmp(srcType,'rand'))
      % NOTE: This implementation is not finnished. The power is not yet 
      % normalized.
      sourceSigTime = (rand(noSrc,noTime) + j*rand(noSrc,noTime));
    

    % ****************** No source signals. ******************

    elseif (strcmp(srcType,'nosrc'))
      sourceSigTime = 0;


    % ****************** The else branch. ******************

    else
      error('DBT-Error: Specified signal type is not implemented.')
    end%if



% ----------------------------------------------------------------------- %
% Simulation of noise signals.
% ----------------------------------------------------------------------- %

  % ****************** Gaussian stochastic signals. ******************

  if (strcmp(noiseType,'randn') | strcmp(noiseType,'Gaussian'))
    noiseSigTime = (randn(noChan,noTime) + j*randn(noChan,noTime)) ...
      * 1/sqrt(2) * sqrt(noisePower);
    % First, generate i.i.d random variables with variance (power) two.
    % The total power of the simulatred noise from "randn"
    % is 1 + 1 (real + imaginary) = 2.
    % The factor 1/sqrt(2) normalizes the simulated noise to 1.
    % The last factor, sqrt(noisePower), gives the noise the desired power.


  % ****************** No noise signals. ******************

  elseif (strcmp(noiseType,'nonoise'))
    noiseSigTime=0;


  % ****************** The else branch. ******************

  else
    error('DBT-Error: The noise model does not exist.')
  end%if


% ----------------------------------------------------------------------- %
% Orthogonalization of the source and noise signals.
%
% NOTE: The implementation of these operations is not finnished.
% ----------------------------------------------------------------------- %

% ************** Orthogonalization of the source signals. **************

% Should also the noise signal be orthogonalized simultaneously?

if (orthSig)
  Rrand=sourceSigTime*sourceSigTime'/noTime;
  sourceSigTime=sqrtm(inv(Rrand))*sourceSigTime;
    % ortonormala vektorer.
    % The signal vectors (row vectors) are now orthogonal to eachother.
    % CHECK THIS !?! AS FA?
end%if


% ************** Orthogonalization of the noise signals. **************

  % Maybe it is better to do the orthogonalization together with the
  % source signals so that source and noise signal become orthogonal to
  % eachother. LP did it that way. Or is it a good idea to be able to choose?

  if (orthSig)
    noiseSigTime = noiseSigTime;
  end%if

  if (~isempty(noiseCorrSrc) )
    % Color the noise spatially with the spatial noise correlation matrix.
    [U,SIG,V] = svd(noiseCorrSrc);
    noiseSigTime = U*sqrt(SIG)*noiseSigTime;
      % Compute the square root of the source covariance matrix with SVD.
      % Color the i.i.d random variables with the square root of the
      % noise covariance matrix.
  end%if



% ----------------------------------------------------------------------- %
% Correlate the source signals and adjust the power level.
%
% ----------------------------------------------------------------------- %

% ************** Scale the source signals to the right power. **************

estPower = mean(abs(sourceSigTime).^2 ,2);
  % estPower (RaelVectorT) (noSrc x 1): Estimate the power in the
  % source signals.
scaleFactors = sqrt(srcPowers) ./ estPower;
sourceSigTime = weightrows(sourceSigTime, scaleFactors);
  %WEIGHTROWS: A DBT function that weights the rows of a matrix. The same
  %  weight is applied to all columns of the same row. This is equal to
  %  Y = diag(weights)*X .


% ************** Correlate the source signals. **************

if (~isempty(srcCorrSrc))
  if (corrMethod == 0)
    % Lars Pettersson's version.
    sourceSigTime=sqrtm(srcCorrSrc)*sourceSigTime;
  else
    [U,S,V] = svd(srcCorrSrc);
    B=(1/sqrt(2))*U*sqrt(S);
       % Compute the square root of the source covariance matrix with SVD.
       % Is there any advantage with this method?
    sourceSigTime = B*sourceSigTime;
  end%if
end%if


% ----------------------------------------------------------------------- %
% Correlate the noise signals and adjust the power level.
%
% ----------------------------------------------------------------------- %

% ************** Scale the noise signals to the right power. **************



% ************** Correlate the noise signals. **************




% ----------------------------------------------------------------------- %
% Time shift, according to element positions, and scale the source signals.
%
% This is the influence of the array antenna on the source sign.als.
% Start: 981211 Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %


focusDist = []; 	% Focusing distance for the spatial steering matrix.
                        %   [] = default value. See the help text for
                        %   the function "spastemat".
subPointDoa = [];	% Pointing directions for possible subarrays.
                        %   [] = default value. See the help text for
                        %   the function "spastemat".
noFreq = noTime;	% Number of frequency points for the signals.
chanSigFreq = zeros(noChan,noTime);	% Signals X in the antenna channels.

  % sourceSigTime (CxMatrixT): Complex source time signals (the complex
  %   envelopes) S in a (noTime, noSrc)-matrix.
  % chanSigFreq (CxMatrixT): Complex signals (the complex envelopes) X of
  %   the antenna channels in a (noOfChannels, noFreq)-matrix.

sourceSigFreq = fftshift1(fft(sourceSigTime.')).'; % To the frequency domain.
  % FFTSHIFT1: A DBT function that shifts DC component to center of spectrum
  %   for several 1D spektra or shifts the DC component from the center.
  % sourceSigFreq (CxMatrixT): Complex source time signals (the complex
  %   envelopes) S in a (noSrc, noFreq)-matrix.

[freq, freqLow, freqHigh] = genfreqaxis(carrierFreq, bandwidth, noFreq);
lambda = propSpeed./freq;

for freqLoop = 1:noFreq
  A = spastemat(antenna,srcDoas,lambda(freqLoop),focusDist,subPointDoa);
    % A DBT function that calculates the spatial steering matrix.
  chanSigFreq(:,freqLoop) = A * sourceSigFreq(:,freqLoop);
end%for freqLoop

chanSigTime = ifft(fftshift1(chanSigFreq.','inv')).'; % Back to time domain.
  % chanSigFreq (CxMatrixT): Complex signals (the complex envelopes) X of
  %   the antenna channels in a (noOfChannels, noTime)-matrix.



% ----------------------------------------------------------------------- %
% Add noise and source signals.
% ----------------------------------------------------------------------- %

%estSignalPower = mean(abs(chanSigTime).^2 ,2).'	% For debugging.
%estNoisePower = mean(abs(noiseSigTime).^2 ,2).'	% For debugging.
%estSNRdB = 10*log10(estSignalPower./estNoisePower)	% For debugging.

chanSigTime = chanSigTime + noiseSigTime;


% ----------------------------------------------------------------------- %
% Create output signal variable.
%
% This section creates a standard DBT signal of data type RxRadarSigT.
% ----------------------------------------------------------------------- %

carrierWavelength = propSpeed / carrierFreq;
noRangeBins = 1;
pModulation = [];	% Should this parameter be assigned "sourceSigTime" ?
sampleTime =  1/(freqHigh-freqLow);
  %  sampleTime (RealScalarT): Stored in the output signal "sigOut" for
  %    possible use in signal processing routines.
noCPI = 1;

signals = zerosm([noTime, noRangeBins, noChan, 1, noCPI]);
signals = setm(signals, chanSigTime.', ':',1,':',1,1);
  % SETM is a RECVAR function deliverad with DBT. It can be replaced with
  %   standard Matlab 5 multidimensional array indexing.
waveform = defwave(carrierWavelength, noRangeBins, noTime, pModulation, ...
  sampleTime, noCPI);
  % DEFWAVE is a DBT function, with the help of which the time properties
  %   of a radar or radio signal can be specified.
sigOut = RxRadarSigT(signals,{antenna, waveform});


%endfunction simwidebandkernel



%End Of File -------------------------------------------------------------


