function [ranBins, ranBinLen] = ran2ranbin(ranges, sampleTime, ...
                                           propSpeed, noRanBins)

%RAN2RANBIN: Converts ranges to rangebins & calculates length of a rangebin
%
%Synopsis:
%  ranBins = ran2ranbin(ranges, sampleTime,
%                       propSpeed, noRanBins)
%
%  [ranBins, ranBinLen] = ran2ranbin(ranges, sampleTime,
%                                    propSpeed, noRanBins)
%
%Description:
%  Converts ranges (in meter) to appropriate rangebin according to
%  propagation speed of light in current medium and sampletime.
%  If the result exceeds the maximum noRangeBins it then starts over
%  at rangebin #1 again. If the max no of rangebins is omitted then
%  there is no wrap back to rangebin #1. The function also calculates
%  the range resolution, i.e. the length (in meters) of each rangebin.
%
%Input:
%  ranges (RealVectorT)     : Distance between target and radar [m].
%  noRanBins (IntScalarT)   : Maximum number of rangebins.
%  sampleTime (RealScalarT) : Time between rangebins [s].
%  propSpeed (RealScalarT)  : Propagation speed of the beam [m/s].
%
%Output:
%  ranBins (IntVecT)      : The rangebins corresponding to input ranges.
%  ranBinLen (ReaScalatT) : Length of the calculated range bins [m].
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in
%  [1] or by "help dbtdata".
%
%Example:
%
%Software Quality:
%  Tested extensively.
%
%Known Bugs:
%
%References:
%  [2]: Kingsley S., Quegan S.: "Understanding Radar Systems",
%       McGraw-Hill 1992.
%
%See Also:
%  getranbinlen

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
%  (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 980511 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:21:29 $ $Author: svabj $.
%  $Revision     : 1.0 $
%*************************************************************************

if (nargin < 4)
  noRanBins = realmax;
end%if

ranBinLen = sampleTime * propSpeed/2;

unModRange = ceil(ranges / ranBinLen);

%if sum(find(unModRange > noRanBins)) > 0
%  infoStr = sprintf('\nFollowing range was ambiguous: %1.0f [m]. Proceeding...', ...
%                     ranges(find(unModRange > noRanBins)));
%  dbtinfo(infoStr);
%end%if


ranBins = mod(unModRange,noRanBins);
ranBins(find(ranBins == 0)) = noRanBins;



%End Of File
