function sigOut = setspatrans(sigIn, spaTrans)

%SETSPATRANS Set spatial transformation matrix in a radar signal.
%
%--------
%Synopsis:
%  sigOut = setspatrans(sigIn, spaTrans)
%
%Description:
%  Set spatial transformation matrix in a radar signal. This must be done
%  after linear filtering in the spatial dimension, i.e. beamforming, to
%  make further beamforming or signal processing in the spatial dimension
%  possible.
%
%Output and Input:
%  sigOut (RxRadarSigT): Output radar signal.
%  sigIn (RxRadarSigT): Input radar signal.
%  spaTrans (CxMatrixT): Transformation matrix used for beamforming, i.e.
%    filtering in the spatial dimension.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1].
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.
%   Known bugs.)
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 990116 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:21:39 $ $Author: svabj $.
%  $Revision: 1.5 $
% *****************************************************************************


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

if (nargin < 2)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************


% ****************** Default values ******************


% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT','RxCorrMatT')
chkdtype(spaTrans, 'CxMatrixT')


% ----------------------------------------------------------------------- %
% Do the work.
% ----------------------------------------------------------------------- %

sigOut = sigIn;

if (isfield(sigOut,'trans.spaTrans'))
  % This check is necessary in case the field "trans.spaTrans" is missing in
  % the input parameter "sigIn". In this first if-branch the field exists.

  sigOut.trans.spaTrans = sigOut.trans.spaTrans * spaTrans;
else
  % In this else-branc, the field "trans.spaTrans" is missing in the
  % input parameter "sigIn".

  sigOut.trans.spaTrans = spaTrans;
end%if

sigOut.antenna = defant('beamform',sigIn.antenna, spaTrans);
  % This extension of the antenna definition is required for the function
  % "stemat" to work after beamforming.


%End Of File
