function [Xout, xValues, yValues, xstr, ystr, handles, contMat] = sigplot2(signal, pulseIx, rangeIx, thetaIx, extraIx, cpiIx, sigType, sigPart, scaleType, normType, plotType, lineSpec, pulAxis, ranAxis, spaAxis, extraAxis, cpiAxis, normMinLevel, normMaxLevel, plotMinLevel, plotMaxLevel, sigMinVal, sigMaxVal, extraPlotParam, guiFlag)

%SIGPLOT2 Plots a 1D- or 2D-slice of a radar signal.
%
%--------
%Synopsis:
%  sigplot2(signal, pulseIx, rangeIx, thetaIx)
%  sigplot2(signal, pulseIx, rangeIx, thetaIx, [], [], sigType, sigPart,
%    scaleType)
%  sigplot2(signal, pulseIx, rangeIx, thetaIx, [], cpiIx, sigType, sigPart,
%    scaleType, normType, plotType, lineSpec)
%  sigplot2(signal, pulseIx, rangeIx, thetaIx, [], cpiIx, sigType,
%    sigPart, scaleType, normType, plotType, lineSpec, pulAxis, ranAxis,
%    spaAxis)
%  sigplot2(signal, pulseIx, rangeIx, thetaIx, [], cpiIx, sigType,
%    sigPart, scaleType, normType, plotType, lineSpec, pulAxis, ranAxis,
%    spaAxis, [],[], normMinLevel, normMaxLevel, plotMinLevel, plotMaxLevel,
%    sigMinVal, sigMaxVal, extraPlotParam)
%
%  sigplot2(signal, pulseIx, rangeIx, thetaIx, keywordList)
%
%  Xout = sigplot2(signal,...)
%  [Xout, xValues, yValues] = sigplot2(signal, ...)
%  [Xout, xValues, yValues, xstr, ystr] = sigplot2(signal, ...)
%  [Xout, xValues, yValues, xstr, ystr, handles, contMat] 
%     = sigplot2(signal, ...)
%
%Description:
%  Plots a 2D-slice of a radar signal.
%  The notation ':' can be used to denote all indices in one dimension, e.g.
%  sigplot2(sig,':',':',1,1,1)
%  The order of operations is:
%  1) Get the part of the signal with the right indices.
%  2) Create the signal type indicated by the input parameter "sigType".
%  3) Get the part of the signal indicated by the input parameter "sigPart".
%  4) Normalize the signal in the way indicated by "normType".
%  5) Convert the signal to the scale type indicated by "scaleType".
%  6) Remove values (set to NaN) below "plotMinLevel" and above "plotMaxLevel".
%  7) "sigMinVal" specifies the signal value to use at the minimum level
%    (given by plotMinLevel) to plot. Default value is NaN.
%  8) Plot the signal with the plot type indicated by "plotType".
%
%Output and Input:
%  Xout (CxMatrixT): The processed signal matrix after step 1)-5) of the
%   "description". This is what is plotted by this function. This matrix
%    is returned for the case the user want to plot it himself/herself.
%  xValues (): Values of x-axis.
%  yValues (): Values of y-axis.
%  xstr (StringT): Label of x-axis.
%  ystr (StringT): Label of y-axis.
%  handles (?): One or more handle(s) of the just created graphics object(s).
%    Not applicable for plotType 'stairs'.
%  contMat (?): Contour matrix returned by the Matlab functions "countour"
%    etc. "contMat" is called only "C" in the help text of "contour".
%    "contMat" can be used as input to the Matlab function "clabel" to add 
%    height labels to a two-dimensional contour plot.
%
%  signal (RxRadarSigT): Input radar signal.
%  pulseIx (IndexT): Index for pulses or doppler channels.
%  rangeIx [D](IndexT): Index for range bins.
%  thetaIx [D](IndexT): Index for theta channels or beams.
%
%  keywordList [D](CellArrayT): A cell array with input parameter name
%    (keyword) and value pairs, see the example below. All input parameters
%    of this function following "thetaIx" can be used. The use of keywords
%    is to make calls simpler to this function with many possible input
%    parameters.
%
%  extraIx [D](IndexT): Extra index.
%  cpiIx [D](IndexT): Index for CPIs.
%  sigType [D](StringT): Signal type.
%    = 'sig': The signal itself (default).
%    = 'fft': FFT of the signal.
%  sigPart [D](StringT): What part of the signal to plot.
%    = 'abs': Absolute value (default).
%    = 'angle': Phase angle with Matlab:s "angle()" function.
%    = 'uwangle': Phase angle with Matlab:s "unwrap(angle())" function.
%       This gives a continous phase angle.
%    = 'real': Real part.
%    = 'imag': Imaginary part.
%  scaleType [D](StringT): Scale Type for vertical axis.
%    = 'lin': Linear scale.
%    = 'dB': dB scale (20*log10(X)). This is a power measure.
%    = 'log': 10*log10(X).
%    = 'radians': Does nothing. Suitable to display phase angles in radians.
%      The same effect can be done with 'lin'.
%    = 'degrees': Divide by 180 and multiplicate by pi. Suitable to display
%      phase angles in degrees instead of radians.
%    = 'pirad': Divide by pi. Suitable to display phase angles in multiples
%      of pi radians
%  normType [D](StringT): Type of normalization.
%    = 'min': Normalize the minimum of the signal to "normMinLevel",see below.
%    = 'max': Normalize the maximum of the signal to "normMaxLevel",see below.
%    = 'none' : No normalization is done.
%  plotType [D](StringT): Plot type.
%    3D-plots:
%    = 'surfl': Iluminated surface plot (Default).
%    = 'surf': See online MATLAB Function Reference.
%    = 'surfc': See online MATLAB Function Reference.
%    = 'mesh': See online MATLAB Function Reference.
%    = 'meshc': See online MATLAB Function Reference.
%    = 'meshz': See online MATLAB Function Reference.
%    = 'pcolor': See online MATLAB Function Reference.
%    = 'contour': See online MATLAB Function Reference. "extraPlotParam" can
%      be used as the input parameters "n" or "v" to "contour".
%    = 'contourf': See online MATLAB Function Reference. "extraPlotParam" can
%      be used as the input parameters "n" or "v" to "contourf".
%    = 'contour3': See online MATLAB Function Reference. "extraPlotParam" can
%      be used as the input parameters "n" or "v" to "contour3".
%    = 'plot': Like plotting a matrix with MATLABs "plot". Several lines are 
%      plotted on each other in 2D.
%    For 2D-plots:
%    = 'bar': Vertical bar plot.
%    = 'barh': Horizontal bar plot.
%    = 'bar3': 3D bar plot.
%    = 'plot': Using MATLAB's "plot" function.
%    = 'stem': Using MATLAB's "stem" function.
%    = 'stairs': Using MATLAB's "stairs" function.
%    All plots:
%    = 'noplot' : No plot but returning the processed signal to an
%      output parameter "Xout".
%  lineSpec [D](StringT) : Specifies a line style, marker symbol, and color
%    for the plot when the undelying Matlab plot function allows that. (see
%   LineSpec in Matlab HelpDesk for more information).
%  pulAxis [D](RealVectorT): Values on the pulse or doppler channel axis.
%  ranAxis [D](RealVectorT): Values on range axis.
%  spaAxis [D](RealVectorT): Values on antenna channel, beam or DOA axis.
%  normMinLevel [D](RealScalarT): Minimum value in a linear amplitude scale.
%    Note: This can be a very small value but not zero.
%  normMaxLevel [D](RealScalarT): Maximum value in a linear amplitude scale.
%  plotMinLevel [D](RealScalarT): Minimum signal level to plot.
%  plotMaxLevel [D](RealScalarT): Maximum signal level to plot.
%  sigMinVal [D] (RealScalarT): Signal value to use at the minimum level
%    (given by plotMinLevel) to plot. Default value is NaN.
%  extraPlotParam [D](?): Extra input parameters to MATLAB's plotting
%    function (as determined by "plotType"").
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in ...
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  method.
%
%Examples:
%  Example 1:
%    ant = defant('expAnt');
%    expFileName = 'filename'
%    sig = expsig1(ant,[expFileName, '.dbs']);
%    figure, sigplot2(sig,5,1:20,6,[],[],[],'angle');
%    figure
%    [Xout,xValues,yValues,xstr,ystr]=sigplot2(sig,':',10,':',[],[],[],'angle');
%    surfl(xValues, yValues, Xout);
%    colormap copper;
%    view([-37 50])
%    xlabel([xstr ]);
%    ylabel([ystr ])
%
%  Example 2:
%    ant = defant('expAnt');
%    expFileName = 'filename'
%    sig = expsig1(ant,[expFileName, '.dbs']);
%    sigplot2(sig,5,1:20,6,{'sigPart','angle',  'scaleType','pirad'});
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%
%Known Bugs:
%  It is unclear if the levels are correct when using scaleType = 'dB'. (in
%  conjuction with "dbf" and tapering)
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  splot2, doaspc2sig, correig2sig

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
% Start        : 970725 Svante Bjrklund (svabj).
% Latest change: $Date: 2000/10/16 15:21:42 $ $Author: svabj $.
% $Revision: 1.55 $
% *****************************************************************************
%Function Calls:
%  sigplot2win -> sigplot2winGUIcb -> sigplot2
%  sigplot2win_s -> sigplot2win -> sigplot2winGUIcb -> sigplot2
%  "A -> B" means that A calls B.


if (nargin < 2)
  error('DBT-Error: To few input parameters.')
end

% ****************** Add missing input parameters ******************

if (nargin < 3)
  rangeIx = [];
end
if (nargin < 4)
  thetaIx = [];
end
if (nargin < 5)
  extraIx = [];
end
if (nargin < 6)
  cpiIx = [];
end
if (nargin < 7)
  sigType = [];
end
if (nargin < 8)
  sigPart = [];
end
if (nargin < 9)
  scaleType = [];
end
if (nargin < 10)
  normType = [];
end
if (nargin < 11)
  plotType = [];
end
if (nargin < 12)
  lineSpec = [];
end
if (nargin < 13)
  pulAxis = [];
end
if (nargin < 14)
  ranAxis = [];
end
if (nargin < 15)
  spaAxis = [];
end
if (nargin < 16)
  extraAxis = [];
end
if (nargin < 17)
  cpiAxis = [];
end
if (nargin < 18)
  normMinLevel = [];
end
if (nargin < 19)
  normMaxLevel = [];
end
if (nargin < 20)
  plotMinLevel = [];
end
if (nargin < 21)
  plotMaxLevel = [];
end
if (nargin < 22)
  sigMinVal = [];
end

% A jump to 24 because the input parameter "sigMaxVal" is not used.

if (nargin < 24)
  extraPlotParam = [];
end
if (nargin < 25)
  guiFlag = [];
end



% *************** Evalute the list "extraIx" of keywords  ***************
% These the value of this keywords are assigned to the local variables with
% the same name as the keywords. The use of keywords is to make calls to
% this function simpler.

if (iscell(extraIx))
  % Now extraIx == keywordList is interpreted as a cell array with input
  % parameter name (keyword) and value pairs.
  keywordList = extraIx;
  extraIx = [];
  lenExtra = length(keywordList);
  if (mod(lenExtra,2) ~= 0), error('DBT-Error: Value missing.'),end%if
  for n = 1:2:lenExtra
    eval([keywordList{n} '= keywordList{n+1};' ]);
  end%for n
end

if ((exist('sigMaxVal') == 1) & (~isempty(sigMaxVal))) %Is a variable in the workspace.
  dbtwarning('The input parameter sigMaxVal has changed name to plotMaxLevel.')
  %error('DBT-Error: The input parameter sigMaxVal has changed name to plotMaxLevel.')
end%if
if ((exist('sigMinLevel') == 1)) %Is a varibale in the workspace.
  error('DBT-Error: The input parameter sigMinLevel has changed name to sigMinVal.')
end%if


% ****************** Default values ******************

if isempty(rangeIx)
  rangeIx = 1;
end%if
if isempty(thetaIx)
  thetaIx = 1;
end%if
if isempty(extraIx)
  extraIx = 1;
end%if
if isempty(cpiIx)
  cpiIx = 1;
end%if
if isempty(sigType)
  sigType = 'sig';
end%if
if isempty(sigPart)
  sigPart = 'abs';
end%if
if isempty(scaleType)
  scaleType = 'lin';
end%if
if isempty(normType)
  normType = 'none';
end%if
if isempty(plotType)
  plotType = 'surfl';
end%if
if isempty(lineSpec)
  lineSpec = '';
end%if
if isempty(pulAxis)
  pulAxis = [];
end%if
if isempty(ranAxis)
  ranAxis = [];
end%if
if isempty(spaAxis)
  spaAxis = [];
end%if
if isempty(extraAxis)
  extraAxis = [];
end%if
if isempty(cpiAxis)
  cpiAxis = [];
end%if
if isempty(normMinLevel)
  normMinLevel = 1e-4;
end%if
if isempty(normMaxLevel)
  normMaxLevel = 1;
end%if
if isempty(sigMinVal)
  sigMinVal = NaN;
end%if

% The input parameter "sigMaxVal" is not used.

if isempty(extraPlotParam)
  extraPlotParam = [];
end%if
if isempty(guiFlag)
  guiFlag = 0;
end%if

% Default values for plotMinLevel and plotMaxLevel is determined during plot
% by Matlab.



% ****************** Find which indices that are vectors ******************

if (length(pulseIx) > 1)
  pulseIxHlp = ':';
else
  pulseIxHlp = pulseIx;
end%if

if (length(rangeIx) > 1)
  rangeIxHlp = ':';
else
  rangeIxHlp = rangeIx;
end%if

if (length(thetaIx) > 1)
  thetaIxHlp = ':';
else
  thetaIxHlp = thetaIx;
end%if

if (length(extraIx) > 1)
  extraIxHlp = ':';
else
  extraIxHlp = extraIx;
end%if

if (length(cpiIx) > 1)
  cpiIxHlp = ':';
else
  cpiIxHlp = cpiIx;
end%if

index = [pulseIxHlp; rangeIxHlp; thetaIxHlp; extraIxHlp; cpiIxHlp];

stop  = 0;
yindex = 999;
n = 1;
while ((n <= 5) & (~stop))
  if ((index(n,1) == ':') | (size(index(n,:),2) > 1))
    % The first vector is found.
    yindex = n;
    stop = 1;
  end;
  n = n +1;
end%while
if (yindex == 999)
  dbterror('At least one index must be a vector.')
end%

stop  = 0;
xindex = 999;
  % This value will remain if a second vector is not found.
  % If a second vector is not found, xindex is not used.
  % The statement above is necessary to avoid an error message.
while ((n <= 5) & (~stop))
  if ((index(n,1) == ':') | (size(index(n,:),2) > 1))
    % The second vector is found.
    xindex = n;
    stop = 1;
  end;
  n = n +1;
end%while

while (n <= 5)
  if ((index(n,1) == ':') | (size(index(n,:),2) > 1))
    dbterror('Maximum two indices can be vectors.')
  end;
  n = n +1;
end%while




% ************** Get indices for plotting axes  **************

% When the index is ':' replace it by a vector with the correspondig indices.

%maxNoIndex = 6;

XX = signal.signals;

sizeXX = sizem(XX);

%sizeXX = [sizeXX ones(1,maxNoIndex-size(sizeXX,2))];

if (isstr(pulseIx) & (pulseIx == ':'))
  pulseIx = 1:sizeXX(1,1);
end
if (isstr(rangeIx) & (rangeIx == ':'))
  rangeIx = 1:sizeXX(1,2);
end
if (isstr(thetaIx) & (thetaIx == ':'))
  thetaIx = 1:sizeXX(1,3);
end
if (isstr(extraIx) & (extraIx == ':'))
  extraIx = 1:sizeXX(1,4);
end
if (isstr(cpiIx) & (cpiIx == ':'))
  cpiIx = 1:sizeXX(1,5);
end


% ****************** Possibly plot in a GUI ******************

if (guiFlag)
  disp('sigplot2: guiFlag')
  cfh = sigplot2win(signal);

  set(findobj(cfh, 'Tag','EditTextStartIndexPulse'),'String', ...
    num2str(pulseIx(1)));
  set(findobj(cfh,'tag','EditTextLengthPulse'),'String',...
    num2str(pulseIx(length(pulseIx)) - pulseIx(1) + 1));

  set(findobj(cfh, 'Tag','EditTextStartIndexRange'),'String', ...
    num2str(rangeIx(1)));
  set(findobj(cfh,'tag','EditTextLengthRange'),'String',...
    num2str(rangeIx(length(rangeIx)) - rangeIx(1) + 1));

  set(findobj(cfh, 'Tag','EditTextStartIndexSpace'),'String', ...
    num2str(thetaIx(1)));
  set(findobj(cfh,'tag','EditTextLengthSpace'),'String',...
    num2str(thetaIx(length(thetaIx)) - thetaIx(1) + 1));

  set(findobj(cfh, 'Tag','EditTextStartIndexExtra'),'String', ...
    num2str(extraIx(1)));
  set(findobj(cfh,'tag','EditTextLengthExtra'),'String',...
    num2str(extraIx(length(extraIx)) - extraIx(1) + 1));

  set(findobj(cfh, 'Tag','EditTextStartIndexCPI'),'String', ...
    num2str(cpiIx(1)));
  set(findobj(cfh,'tag','EditTextLengthCPI'),'String',...
    num2str(cpiIx(length(cpiIx)) - cpiIx(1) + 1));

  %set(findobj(cfh, 'Tag','EditTextStartIndexTrial'),'String', ...
  %  num2str(trialIx(1)));
  %set(findobj(cfh,'tag','EditTextLengthTrial'),'String',...
  %  num2str(trialIx(length(trialIx)) - trialIx(1) + 1));

  set(findobj(cfh,'Tag','EditTextAxisValuesPulse'),'String',...
    ['[',num2str(pulAxis(:).'),']']);
  set(findobj(cfh,'Tag','EditTextAxisValuesRange'),'String',...
    ['[',num2str(ranAxis(:).'),']']);
  set(findobj(cfh,'Tag','EditTextAxisValuesSpace'),'String',...
    ['[',num2str(spaAxis(:).'),']']);
  set(findobj(cfh,'Tag','EditTextAxisValuesExtra'),'String',...
    ['[',num2str(extraAxis(:).'),']']);
  set(findobj(cfh,'Tag','EditTextAxisValuesCPI'),'String',...
    ['[',num2str(cpiAxis(:).'),']']);
  %set(findobj(cfh,'Tag','EditTextAxisValuesTrial'),'String',...
  %  ['[',num2str(trialAxis(:).'),']']);

  scaleTypeHandle = findobj(cfh,'Tag','PopupMenuScaleType');
  altTxt = get(scaleTypeHandle,'String')
  altIx = find(strcmp(altTxt,scaleType))
  set(scaleTypeHandle,'Value',altIx);

else % (guiFlag)



% ************** Get the right part (indices) of the signal **************

X  = getm(XX,pulseIx,rangeIx,thetaIx,extraIx,cpiIx);

%if (yindex > xindex)
%  X = X.';
%end

titleText = '[';


% ****************** Labels and values of the axes ******************

pulseStr = 'Pulses or doppler ch.';
rangeStr = 'Range bins';
thetaStr = 'Theta channels or beams';
phiStr   = 'Phi channels or beams';
trialStr = 'Trials';

xErrStr  = 'Wrong x-axis index.';
yErrStr  = 'Wrong y-axis index.';

if ~isempty(pulAxis)
  pulAxisValues = pulAxis;
else
  pulAxisValues = pulseIx;
end%if
if ~isempty(ranAxis)
  ranAxisValues = ranAxis;
else
  ranAxisValues = rangeIx;
end%if
if ~isempty(spaAxis)
  spaAxisValues = spaAxis(1,:);
else
  spaAxisValues = thetaIx;
end%if



if (xindex == 1)
  xstr = pulseStr;
  xValues = pulAxisValues;
elseif (xindex == 2)
  xstr = rangeStr;
  xValues = ranAxisValues;
elseif (xindex == 3)
  xstr = thetaStr;
  xValues = spaAxisValues;
elseif (xindex == 4)
  xstr = phiStr;
  xValues = extraIx;
elseif (xindex == 5)
  xstr = trialStr;
  xValues = cpiIx;
elseif (xindex == 999)
  xstr = 'xstr is not used.';
else
  error(xErrStr)
end%if

if (yindex == 1)
  ystr = pulseStr;
  yValues = pulAxisValues;
elseif (yindex == 2)
  ystr = rangeStr;
  yValues = ranAxisValues;
elseif (yindex == 3)
  ystr = thetaStr;
  yValues = spaAxisValues;
elseif (yindex == 4)
  ystr = phiStr;
  yValues = extraIx;
elseif (yindex == 5)
  ystr = trialStr;
  yValues = cpiIx;
else
  error(yErrStr)
end%if

%if (length(pulseIx) = 1)
%  display pulseIx somewhere in the graph.
%end%if
%Do the same with the other indices.


% ****************** Signal types ******************

if (strcmp(sigType,'sig'))
  % Do nothing.
elseif (strcmp(sigType,'fft'))
  fprintf('sigplot2: size(X) %d',size(X))
  fprintf('sigplot2: Should it be fftshift1 instead of fftshift?')
  X = fftshift(fft(X));
  % Should it be fftshift1 ?
else
  error('DBT-Error: Invalid sigType.')
end%if
titleText = [titleText,sigType,','];

% ****************** Signal parts ******************

if (strcmp(sigPart,'abs'))
  X = abs(X);
elseif (strcmp(sigPart,'angle'))
  X = angle(X);
elseif (strcmp(sigPart,'uwangle'))
  X = unwrap(angle(X));
elseif (strcmp(sigPart,'real'))
  X = real(X);
elseif (strcmp(sigPart,'imag'))
  X = imag(X);
else
  error('DBT-Error: Invalid sigPart.')
end%if
titleText = [titleText,sigPart,','];


% ****************** Normalization ******************

if (strcmp(normType,'none'))
  % Do nothing.
elseif (strcmp(normType,'max'))
  X = normMaxLevel * X./max(max(X));
elseif (strcmp(normType,'min'))
  X = normMinLevel * X./min(min(X));
elseif (strcmp(normType,'minmax'))
  % Not implemented.
else
  error('DBT-Error: Invalid normType.')
end%if


% ****************** Scale types ******************

if (strcmp(scaleType,'lin'))
  % Do nothing.
elseif (strcmp(scaleType,'dB'))
  X = 20*log10(X+eps);			% "+eps" to avoid log of zero.
elseif (strcmp(scaleType,'log'))
  X = 10*log10(X+eps);			% "+eps" to avoid log of zero.
elseif (strcmp(scaleType,'radians'))
  % Do nothing.
elseif (strcmp(scaleType,'degrees'))
  X = r2d(X);
elseif (strcmp(scaleType,'pirad'))
  X = X/pi;
else
  error('DBT-Error: Invalid scaleType.')
end%if
titleText = [titleText,scaleType,','];


% ****************** Min and max values to plot ******************

if ~isempty(plotMaxLevel)
  X(X > plotMaxLevel) = NaN;
end%if
if ~isempty(plotMinLevel)
  X(X < plotMinLevel) = sigMinVal;
end%if


% ****************** Plot ******************

[viewAz, viewEl] = view;
cax = newplot;		%Returns current axes. Retores view to 2D.
holdState = ishold;
  % The correct way to create or relate to a figure and axes according to [Using MATLAB Graphics. Version 5] p. 8-34 -- 8-36.
%cfi = get(cax,'Parent')	% Returns current figure (parent of cax).
%set(cfi,'DefaultColormap',pink)

%viewAngles = [-37 50];
if ~strcmp(plotType,'noplot')
  if (min(size(X)) >=3)				% **** Plot a 2D-signal. ****
    if strcmp(plotType,'surfl')
      h = surfl(xValues, yValues, X);
      %view(viewAngles)
      noDimGraph = 3;
    elseif strcmp(plotType,'surf')
      h = surf(xValues, yValues, X);
      %view(viewAngles)
      noDimGraph = 3;
    elseif strcmp(plotType,'surfc')
      h = surfc(xValues, yValues, X);
      %view(viewAngles)
       noDimGraph = 3;
   elseif strcmp(plotType,'mesh')
      h = mesh(xValues, yValues, X);
      %view(viewAngles)
       noDimGraph = 3;
   elseif strcmp(plotType,'meshc')
      h = meshc(xValues, yValues, X);
      %view(viewAngles)
      noDimGraph = 3;
    elseif strcmp(plotType,'meshz')
      h = meshz(xValues, yValues, X);
      %view(viewAngles)
      noDimGraph = 3;
    elseif strcmp(plotType,'pcolor')
      h = pcolor(xValues, yValues, X);
      noDimGraph = 2;
    elseif strcmp(plotType,'contour')
      %[contMat,h] = contour(xValues, yValues, X);
      [contMat,h] = contourhlp(plotType, xValues, yValues, X, ...
        extraPlotParam, lineSpec);
      noDimGraph = 2;
   elseif strcmp(plotType,'contourf')
      %[contMat,h] = contourf(xValues, yValues, X);
      [contMat,h] = contourhlp(plotType, xValues, yValues, X, ...
        extraPlotParam, lineSpec);
      noDimGraph = 2;
   elseif strcmp(plotType,'contour3')
      %[contMat,h] = contour3(xValues, yValues, X);
      [contMat,h] = contourhlp(plotType, xValues, yValues, X, ...
        extraPlotParam, lineSpec);
      %view(viewAngles)
      noDimGraph = 3;
   elseif strcmp(plotType,'plot')
      % Like plotting a matrix with MATLABs "plot". Several lines are 
      % plotted on each other in 2D.
      % On the x-axis there is either ranges or pulses.
      % Each curve corresponds to a certain antenna channel, extraIndex,
      % CPI or trial.
      h = plot(yValues,X,lineSpec);
      %tmpStr = xstr;
      xstr = ystr;
      %ystr = tmpStr;
      ystr = '';
      noDimGraph = 2;
    else
      dbterror('This signal must be plotted with a 3D-plot.')
    end%if

    if (0)
    curColorMap = colormap;
    colormap('default')
    defaultColorMap = colormap;
    if (all(all(curColorMap == defaultColorMap)))
      colormap(pink);
    else
      colormap(curColorMap);
    end%if
    end%if (0)

    shading flat
    xlabel([xstr ]);
    ylabel([ystr ])

  elseif ((size(X,1) == 1) | (size(X,2) == 1))   % **** Plot a 1D-signal. ****
    if strcmp(plotType,'bar')
      h = bar(X,lineSpec);
    elseif strcmp(plotType,'barh')
      h = barh(X,lineSpec);
      %xlabelTxt = ylabelTxt;
      %ylabelTxt = '';
    elseif strcmp(plotType,'bar3')
      h = bar3(X,lineSpec);
      %xlabelTxt = '';
      %ylabelTxt = '';
    elseif strcmp(plotType,'plot')
      h = plot(yValues,X,lineSpec);
    elseif strcmp(plotType,'stem')
      h = stem(X,lineSpec);
    elseif strcmp(plotType,'stairs')
      stairs(X,lineSpec);
      h = [];
    else
      h = plot(yValues, X,lineSpec);
    end%if
    xlabel([ystr ]);
    noDimGraph = 2;
  else
    error(['DBT-Error: The signal X must be at least 3-by-3 or an ', ...
      'one-dimensional vector.'])
  end%if
  
   
  titleText = [titleText,']'];

  if (0)
  %get(get(gca,'Title'),'String')
  if (isempty(get(get(gca,'Title'),'String')))
    %disp('Settting title')
    % Only set title if there already is no title.
    %set(gca,'Title',text('String','New Title'))
    title(titleText)
  end%if
  end%if (0)

  %set(cax,'DefaultTitle',titleText)

  if (~holdState)
    if (noDimGraph == 2)
      view(2)
    elseif (noDimGraph == 3)
      if ((viewAz == 0) & (viewEl == 90))
        % Currently 2D view. change the view to 3D. 
        view(3)
      else
        % Already 3D view. Restore to the view before the plot command.
        view([viewAz, viewEl]);
      end%if
    else
      dbterror('Internal error: Unknown view dimension.')
    end%if
    %title(titleText)
    %colormap(pink)
  end%if (~holdState)

end%if plotType


% ****************** Output parameters ******************

if (nargout > 0)
  Xout = X;
  handles = h;
end%if (nargout)

end%if (guiFlag)

%endfunction sigplot2




% ----------------------------------------------------------------------- %
% function contourhlp
%
% Start: 9xxxxx Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %

function [contMat,h] = contourhlp(plotType, xValues, yValues, X, plotParam, lineSpec)
  if (isempty(plotParam))
    if (isempty(lineSpec))
      [contMat,h] = eval([plotType, '(xValues, yValues, X)']);
    else
      [contMat,h] = eval([plotType, '(xValues, yValues, X, lineSpec)']);
    end%if lineSpec
  else
    if (isempty(lineSpec))
      [contMat,h] = eval([plotType, '(xValues, yValues, X, plotParam)']);
    else
      [contMat,h] = eval([plotType,'(xValues,yValues,X,plotParam,lineSpec)']);
    end%if lineSpec
  end%if plotParam
%endfunction contourhlp
