function clutterSig3D = simclutter2(clutter, transmitGain, steeringMatrix, ...
                                   lambda, sampleTime, noPulses, noRangeBins,...
                                   propSpeed, pri, h, platVel);

%SIMCLUTTER is the core simulating engine for simulating clutter sources.
%
%Synopsis:
%  clutterSig3D = simclutter(clutter, transmitGain, steeringMatrix, ...
%                            lambda, sampleTime, noPulses, noRangeBins,...
%                            propSpeed, pri);
%
%Description:
%  Simulates radarsignals and stores the data in a 3D matrix
%  ( Number Pulses x Number Of RangeBins x Number Of Channels)
%
%  Clutter is defined by a power spectral density function (PSD) which
%  describes the power of the clutter at a certain frequency. To simulate
%  clutter for a specific direction and range you simply simulate complex
%  gaussian noise and multiply this noise with the PSD and then performe
%  an inverse FFT. These steps simulates a clutter signal in the pulse
%  domain. There are some considerations though. The PSD has to be sampled
%  and it is thus important not to undersample this function. If a
%  specific timeresolution is wanted (after the inverse FFT) a correct
%  frequency span must be chosen. I.e. if the wanted timeresolution is
%  equal to pri then the total frequency span that the PSD is to be
%  sampled over must be 1/pri! Since the PSD describes how the clutter
%  source are moving the PSD must be sampled symetrically round the
%  frequency zero. In the direction domain consideration to the
%  transmitting antenna patern has to be taken. The ground is divided into
%  small so called clutterpatches from which clutter is radiated. The
%  power of the clutter at each clutterpatch is determined by the
%  transmitting antenna pattern and the area of the patch and also by the
%  distance to the patch. So far no considerations to patcharea nor
%  the distance to the patch is taken. the input variable power is used to
%  determine the power of the strongest clutterpatch. The clutterpatch
%  that illuminates most is the one where the transmitting antenna pattern
%  has its maximum. The width of the clutterpatches are determined by the
%  stepsize in the variable "clutter.DOAs", i.e. the directions to the
%  patches. This stepsize should not be much larger then a third of the
%  beamwith of the transmitting antenna, since this is the sampling angle
%  of the transmitting antenna pattern.
%
%  This function is called upon by simradarsig and should be
%  hidden from the everyday user.
%
%Input:
%  clutter (ClutterDefT)      : Definition of clutter parameters.
%  transmitGain (RealVectorT) : Sampled gain(theta) from transmitting antenna
%  steeringMatrix (CxMatrixT) : Steering matrix calculated for all directions
%                               to clutterpatches.
%  lambda (RealScalarT)       : Wavelength [m]
%  sampleTime (RealScalarT)   : Sample period time in transmitter and receiver.
%                               One sampleTime corresponds to one range bin.
%  noPulses (IntScalarT)      : Number of pulese to simulate for.
%  noRangeBins (IntScalarT)   : Number of rangebins for this radarsystem.
%  propSpeed (RealScalarT)    : Speed of light in current medium [m/s].
%  pri (RealScalarT)          : Pulse repetition interval [s].
%  h (RealScalarT)            : Height above ground of radar platform
%  platVel (RealScalarT)      : Speed of Radar platform along array

%
%OutPut:
%  clutterSig3D (CxMatrixT) : 3D matrix with the simulated clutter data.
%                             pulses x rangebins x channels.
%
%--------
%Clutter of type 'MIT-LCE'
%-------------------------
%Description:
%  MIT-LCE is the result of some measurements done by MIT Lincoln
%  Laboratory [3]. These measurements indicate that clutter is
%  exponentially distributed in contrast to the commonly used gaussian
%  shape. The exponential power spectral density of clutter is discribed
%  with some shape parameters depending on wind conditions. See function
%  "defclutter" for a description of the input parameters.
%
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in
%  [1] or by "help dbtdata".
%
%Examples:
%
%Software Quality:
%
%Known Bugs:
%  1) Pulsemodulation is not performed.
%
%  2) No considerations for range distribution is taken. The simulated
%     signal is same for all wanted ranges.
%
%References:
%  [1]: Bjrklund S., Rejdemyhr D. Athley F.: "DBT, A MATLAB Toolbox
%       for Radar Signal Processing. Reference Guide",
%       FOA-D--9x-00xxx-408--SE, To be published.
%
%  [2]  Rejdemyhr D.: "En studie av klotters inverkan p hgupplsning i
%       riktning", FOA-D--98-00402-408--SE, Linkping, 1998.
%
%  [3]: Billingsley B. Et. Al. Impact of experimental measured doppler
%       spectrum of ground clutter on MTI and STAP. Edinburgh.
%       IEE international conference RADAR 97. 14-16 October 1997.
%
%  [4]: Kingsley S., Quegan S.: "Understanding Radar Systems",
%       McGraw-Hill 1992.
%
%  [5]: Horner R.A., Johnson, C.R.: "Topics in matrix Analysis",
%       Cambridge University Press 1991.
%
%See also:
%   defclutter, simradarsig, simtarget, simnoise, simjammer.

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
%  (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Initiation    : 980701 David Rejdemyhr (davrej).
%  Latest change : $Date: 2000/10/16 15:21:53 $ $Author: svabj $.
%  $Revision     : 1.0 $
%*************************************************************************

%-------------------------------------------------------------------------
%----                 Handle incoming parameters.                     ----
%-------------------------------------------------------------------------

  noClutterPatches = length(clutter.DOAs);


%-------------------------------------------------------------------------
%----                     Initialize variables.                       ----
%-------------------------------------------------------------------------

  clutterSig3D = zeros(noPulses, noRangeBins, size(steeringMatrix,1));


%-------------------------------------------------------------------------
%----                  Initialize random generator.                   ----
%-------------------------------------------------------------------------

  randn('seed',sum(100*clock));
  rand('seed',sum(100*clock));


%-------------------------------------------------------------------------
%----                   Simulate clutter signals.                     ----
%-------------------------------------------------------------------------


  ranStart = ran2ranbin(h, sampleTime, propSpeed, noRangeBins);
  lenOfRanBins = sampleTime * propSpeed/2;
  rans = ((ranStart:noRangeBins)-ranStart) * lenOfRanBins + lenOfRanBins/2;
  smpls=zeros(1,noRangeBins);
  smpls(1:ranStart-1) = 0;
  smpls(ranStart:noRangeBins) = atan(rans/h);

  specElev = pantpat3(clutter.TransAnt, [], [], [], [], [], [], smpls, lambda, [], [], ...
                  [], [], [], 'noplot');

  transGainElev = specElev.specSmpl;
  transGainElev=ones(size(transGainElev));

  %transmitGain=ones(size(transmitGain))/length(transmitGain);


  %-----------------------------------------------------------------------
  %----                Create empty 3D clutter matrix                 ----
  %-----------------------------------------------------------------------

  preClutterSig3D = zeros(noPulses, noRangeBins, noClutterPatches);


  %-----------------------------------------------------------------------
  %----    Calculate suitable number of points to calculate for.      ----
  %-----------------------------------------------------------------------

  fMax = 1/pri;        % Required to get wanted resolution in time-domain.
  noPoints = 512;

  while ((fMax/noPoints) > 5)   % Req. to get appropriate delta frequency
    noPoints = noPoints * 2;
  end%while


  %-----------------------------------------------------------------------
  %----          Create a snapshot for a specific rangebin            ----
  %-----------------------------------------------------------------------

  deltaf = fMax/(noPoints-1); % Makes IFFT much faster if even 2-potens.
  f = (-1/(2*pri): deltaf: 1/(2*pri)).';


  if strcmp(clutter.Type,'MIT-LCE'),
    %---------------------------------------------------------------------
    %----           Get clutter type specific parameters.             ----
    %---------------------------------------------------------------------

    ranges = clutter.TypeParam{1};

    rRT90 = 6377397; %[m]
    hiRan = sqrt(2*4/3*rRT90*h);
    ranges = h:lenOfRanBins:hiRan;

    b =  clutter.TypeParam{2};
    windSpeed = clutter.TypeParam{3}*3600/1609.269392; % [m/s] -> [mph]


    %---------------------------------------------------------------------
    %----         Calculate clutter type specific parameter           ----
    %---------------------------------------------------------------------

    r = 10^(0.1*(-15.5*log10(windSpeed) - 12.1*log10(propSpeed/lambda) + 63.2));


    %---------------------------------------------------------------------
    %---- Calculate power spectral density function for this clutter  ----
    %---- type.                                                       ----
    %---------------------------------------------------------------------

    psdClutter = 1/(1+r)*lambda*b/4*exp(-lambda*b/2*abs(f));
%psdClutter = ones(size(psdClutter))/length(psdClutter);


  else
    dbtwarning('Desired noise type was unknown to simulator, proceeding with no noise.');
  end%if

  %-----------------------------------------------------------------------
  %----  Below are algoritms that are common for all clutter types.   ----
  %-----------------------------------------------------------------------

  %-----------------------------------------------------------------------
  %----      Calculate clutter matrix with 2D transfer function       ----
  %-----------------------------------------------------------------------

  powerMatrix = psdClutter * transmitGain.';

  preClutterPSD = powerMatrix .* ...
                  (randn(noPoints, noClutterPatches) + ...
                  j*randn(noPoints, noClutterPatches));



  %-----------------------------------------------------------------------
  %----     We now have a (frequency x direction) cluttermatrix.      ----
  %----  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -  -   ----
  %----   Shift the signal so that the period is [0,pi] instead of    ----
  %----   [-pi/2, pi/2] in the discrete frequency domain.             ----
  %-----------------------------------------------------------------------

  preClutterPSD = preClutterPSD([noPoints/2+1:noPoints 1:noPoints/2],:);


  %-----------------------------------------------------------------------
  %----     Inverse 1D FFT to create a (time x direction) matrix.     ----
  %-----------------------------------------------------------------------

  clutterSnapShot = ifft(preClutterPSD, noPoints, 1);% * noPoints;


  %-----------------------------------------------------------------------
  %----  Normalization of the matrix...                               ----
  %----  Setting the maximum variance equal to 1, i.e. the power      ----
  %----  of the clutter signal will not exceed 1                      ----
  %-----------------------------------------------------------------------

  clutterSnapShot = clutterSnapShot / max(std(clutterSnapShot));


  %-----------------------------------------------------------------------
  %----        Apply desired maximum power of the clutter             ----
  %-----------------------------------------------------------------------

  clutterSnapShot = sqrt(clutter.Power) * clutterSnapShot;


  %-----------------------------------------------------------------------
  %----         Apply this 2D matrix to desired rangebins.            ----
  %-----------------------------------------------------------------------

  ranBin = ran2ranbin([ranges(1) ranges], sampleTime, propSpeed, noRangeBins);
  ranBins = (ranBin(1):max(ranBin));
  for ranIx = ranBins
    preClutterSig3D(:, ranIx, :) = transGainElev(ranIx) * ...
                                   clutterSnapShot(1:noPulses,:);
  end%for


  %-----------------------------------------------------------------------
  %----   Calculate platform motion matrix                                                                                           ----
  %-----------------------------------------------------------------------

  platVelMx = 2*2*pi/lambda*platVel*sin(smpls.') ...
              * sin(clutter.DOAs)*pri;


  %platVelMx(:,1:8)=0.5;

  %platVelMx(:,9:16)=-0.5;





  %mesh(clutter.DOAs,smpls,platVelMx)
  %xlabel('Direction')
  %ylabel('range')
  %figure
  %platVel3D = zeros(noPulses, noRangeBins, noClutterPatches);

  for pulseIx = 1:noPulses

    platVel3D(pulseIx,:,:) = exp(j*platVelMx * (pulseIx-1));
  end %for
%  sum(sum(sum(platVel3D)))
  preClutterSig3D = preClutterSig3D .* platVel3D;
%  plot(abs(squeeze(platVel3D(:,40,1))))


  %-----------------------------------------------------------------------
  %---- Take the step into beamspace with help of the steering matrix ----
  %-----------------------------------------------------------------------

  if noPulses < length(ranBins)
    for pulseIx = 1:noPulses
      clutterSig3D(pulseIx,:,:) = (steeringMatrix * ...
                                  squeeze(preClutterSig3D(pulseIx,:,:)).').';
    end %for
  else
    for ranIx = ranBins
      clutterSig3D(:,ranIx,:) = (steeringMatrix * ...
                                squeeze(preClutterSig3D(:,ranIx,:)).').';
    end %for
  end%if


%End Of File -------------------------------------------------------------
