function spect = spantpat3(antenna, extraParam)

%SPANTPAT3 Plots the (mechanical) antenna pattern. Simple call using a keyword list.
%
%--------
%Synopsis:
%  spantpat3(antenna)
%  spantpat3(antenna, extraParam)
%
%Description:
%  Simple calling version of the function "pantpat3" using a keyword list.
%  Plots the (mechanical) antenna pattern. This function has the same
%  functionality as "pantpat3" but it is not necessay to list all unused
%  input parameters. Instead a the second input parameter "extraParam" which
%  is a cell array (not prior to Matlab 4) with input parameter name and
%  value pairs is used.
%  See help text for "pantpat3" for more information.
%
%Output and Input:
%  spect [D](DoaSpecT): Output DOA-spectrum. It is a quadratic measure of the
%    sensity of sources from different directions.
%  antenna (AntDefT): Antenna definition.
%  extraParam [D](CellArrayT): A cell array with input parameter name
%    (keyword) and value pairs, see the example below. All input parameters
%    of "pantpat3" following "antenna" can be used.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%  These examples are the same as in the file dbtex15.m.
%  Example 1:
%    This example shows an antenna pattern without and with adaptive
%    sidelobe cancellation.
%    figure
%    theta  = d2r([40 60])';
%    lambda = 1;
%    ant = defant('isotropULA',[12, 0.45*lambda]);   % Define antenna.
%    sigJam = compsim4(ant, lambda, 24, 'const', [theta,zeros(size(theta)),...
%      [10 5]', d2r([0 16])', d2r([34 -13])', Inf*ones(size(theta)), ...
%      eye(size(theta,1))], 'rndn', eye(12));
%        % Generate simulated jammer signals
%    Rjam = ecorrm(sigJam);
%    hold on
%    pantpat3(ant, d2r(10),[],[],'taylor',30,[],[],[],[],[],[],[],[],[],'--')
%      % Without adaptive sidelobe cancellation.
%    spantpat3(ant,  'mainPointDoa',d2r(10),   'taperType','taylor',
%     'taperParam',30,   'iNCorrMat',Rjam,   'plotOpt','-.')
%      % With adaptive sidelobe cancellation.
%    legend('Without SLC', 'With SLC')
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  pantpat3, defant, polar, polar

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 980108 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:22:00 $ $Author: svabj $.
%  $Revision: 1.8 $
% *****************************************************************************

if (nargin < 2)
  spectTmp = pantpat3(antenna);
else
  spectTmp = pantpat3(antenna,[],[],[],[],[],[],[],[],[],[],[],[], ...
    [],[],[],[],[], extraParam);
end%if

if (nargout > 0)
  spect = spectTmp;
end%if

