function [sigOut, basisMat, doaFreqPosOut] = spapulfilt(sigIn, method, model, modelParam, doaFreqChoice, taperCoeff, interCorrMat, steVecFlag, extraParam, orthBeamFlag, normType, normLevel, sampleDim, interChoice, modCorrmParam)

%SPAPULFILT Space-pulse linear filtering (slow-time STAP).
%
%--------
%Synopsis:
%  sigOut = spapulfilt(sigIn)
%
%  [sigOut, basisMat, {beamPosOut, freqPosOut}] = spapulfilt(sigIn, method, 
%    model, modelParam, {beamChoice, freqChoice}, taperCoeff, interCorrMat, 
%    steVecFlag, extraParam, orthBeamFlag, normType, normLevel,
%    sampleDim, interChoice, modCorrmParam)
%
%  [sigOut,basisMat,{beamPosOut,freqPosOut}] = spapulfilt(sigIn, keywordList)
%
%Description:
%  Space-pulse linear filtering (slow-time STAP).
%
%Output and Input:
%  sigOut (RxRadarSigT): Output radar signal.
%  basisMat (CxMatrixT): After the filtering, the signal consists of the 
%    coefficients in the basis, which is made up of the columns of this
%    output matrix. "basisMat" is the memory of what is done with the signal 
%    during the filtering. "basisMat" is stored in the radar signal after
%    the linear filtering, which is necessary to make further processing
%    in the selected dimension possible.
%  beamPosOut (Vector of DoaT): The directions of the created main beams.
%  freqPosOut (RealVectorT): Frequencies of the created Doppler channels.
%
%  sigIn (RxRadarSigT): Input radar signal.
%
%  keywordList [D](CellArrayT): A cell array with input parameter name
%    (keyword) and value pairs, see the example below. All input parameters
%    of this function following "sigIn" can be used. The use of keywords
%    is to make calls simpler to this function with many possible input
%    parameters.
%
%  method (StringT): Filter method. See help text of function "calcfiltinit"
%    for sections about each method.
%    = 'cbf': Conventional beamforming (nonadaptive).
%    = 'acbf': Adaptive beamforming to a desired antenna pattern. This is 
%      beamforming with adaptive sidelobe cancelation (ASLC).
%    = 'arefsig': Adaptive beamforming to a reference time signal.
%      Not implemented
%    = 'capon': Capon's beamformer.
%    = 'fft': Discrete fourier transform.
%    = 'ifft': Inverse discrete fourier transform.
%    = 'music : MUSIC.
%    = 'minnorm': Minimum norm. Not implemented.
%  antenna [D](AntDefT): Antenna model to use. If this parameters is an
%    empty matrix, the antenna model in the input signal "sigIn" is used
%    instead.
%  waveform [D](WaveformT): Waveform (time properties) model to use. 
%    If this parameters is an empty matrix, the waveform model in the input
%    signal "sigIn" is used instead.
%  beamChoice [D](Vector of DoaT): Directions of one or several main beams 
%    [rad].
%  freqChoice [D](RealVectorT.'): Frequencies of one or several doppler 
%    channels [Hz].
%  taperCoeff [D](CxVectorT): Tapering coefficients to use.
%  interCorrMat [D](CxMatrixT): Interference signal correlation matrix. 
%    This is used to adaptively suppress the interference.
%  steVecFlag [D](BoolT): Specifies whether the steering vector (= 1, default) 
%    or filtering weights (= 0) will be used in the filtering.
%  extraParam [D](): The use and data type depend on filter method. See help 
%    text of function "calcfiltinit" for sections about each method.
%  orthBeamFlag [D](BoolT): 1 = orthogonalize the beams. 0 = Do not do it
%    (default).
%  normType [D](StringT): Type of normalization. Not implemented.
%  normLevel [D](RealScalarT): Normalization level. Not implemented.
%  interChoice [D](CellArray): Specifies which dimension in the 
%    (multidimensional) input signal to use when several snapshots are used
%    by the selected method, e.g. 'capon'. Default is the antenna channel
%    dimension.
%    Not implemented.
%  modCorrmParam [D](CellArray):
%
%--------
%Conventional Beamforming.
%Synopsis:
%
%Output and Input:
%
%Description:
%
%Algoritm:
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  The most parts of this function has been tested  by several runs of the 
%  example file "dbtex26.m". 
%  The use of the input parameters "modelParam" and "extraParam" is not tested.
%
%Known Bugs:
%  *) The default values of input parameters "{beamChoice, freqChoice}" are
%  calculated to a wrong values.
%  *) For Capon, only the non-adapted steering matrix is stored as the basis
%  matrix. See the function "calcfiltinit" for more information.
%  *) When 'fft' is chosen, the field "doaPos" in the ouput signal
%  will not have the right values.
%  *) The methods 'fft' and 'ifft' can not be used.
%  *) There is lacking a special treatment, as with 'fft', of the method
%  'dataeigvec because the number of output channels cannot be chosen by 
%  "doaFreqChoice". The number of output channels must be equal to the
%  number of input channels.
%  *) The space-pulse (slow-time) transformation matrix is not set in the
%  output radar signal. This means that further processing in the dimensions
%  in question is not possible.

%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  calcfiltinit, dfb, gettap1, spafilt, ranfilt

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 991027 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:22:01 $ $Author: svabj $.
%  $Revision: 1.5 $
% *****************************************************************************

% The comment "NFF" stands for "New filter function" and gives tips of
% necessary changes when using this function as a template for new filter 
% functions (in other dimensions).

only1DDim = 7;	
  % Must not be in the range 1:6 because these are used in RxRadarSigT.
  % "7" is a free "dimension".
sigDim1 = 3;	
sigDim2 = 1;
  % Choosing sigDim1 = 3 and sigDim2 = 1 means that each signal vector
  % will consist of  all ( = the first dimension, = slow-time or pulses) 
  % column vectors of the third dimension, i.e. space or antenna channels,
  % stacked on top of eachother. For example =
  % [allAntennaChannelsFromPulse1;  allAntennaChannelsFromPulse2; ...] .


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

arginNo=1;
if (nargin < arginNo)
  error('DBT-Error: To few input parameters.')
end
arginNo = arginNo +1;

% ****************** Add missing input parameters ******************

arginNo=2;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  model = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  modelParam = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  doaFreqChoice = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  taperCoeff = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  interCorrMat = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  steVecFlag = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  extraParam = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  orthBeamFlag = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  normType = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  normLevel = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  sampleDim = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  interChoice = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  modCorrmParam = [];
end
arginNo = arginNo +1;


% *************** Evalute the list of keywords  ***************
% If the input parameter "method" is a cell array, it is interpreted as a
% list of input parameter name (keyword) and value pairs.
% The values of these keywords are assigned to the local variables with
% the same name as the keywords. The use of keywords is to make calls to
% this function simpler.

if (iscell(method))
  keywordList = method;
  lenExtra = length(keywordList);
  if (mod(lenExtra,2) ~= 0), error('DBT-Error: Keyword value missing.'),end%if
  for n = 1:2:length(keywordList)
    eval([keywordList{n} '= keywordList{n+1};' ]);
  end%for n
end%if
arginNo = arginNo +1;


% ****************** Get signals sizes. ******************
sigSizeIn = sigsize(sigIn);
sigSizeIn(7) = 1;
  % To make possible to use the same code for 1D- and 2D-filtering.
  % When 1D-filtering assign sigDim2 = 7. When 2D-filtering assign
  % sampleDim = 7, since then we only want to filt one sample at a time
  % because of memory usage and because the program cannot handle
  % several samples when 2D-filtering.
  % The constant "only1DDim" is equal to 7 and should be used when testing
  % the value of sigDim1, sigDim2 and sampleDim.

noTrials  = sigSizeIn(6);
noCpis    = sigSizeIn(5);
noExtras  = sigSizeIn(4);
noBeams   = sigSizeIn(3);
noRanges  = sigSizeIn(2);
noPulses  = sigSizeIn(1);


% ****************** Default values ******************

if isempty(method)
  method = 'cbf';
end%if
if isempty(model)
  model = {sigIn.antenna, sigIn.waveform};
end%if

antenna = model{1};
waveform = model{2};
PRI = waveform.noRangeBins * waveform.sampleTime;
PRF = 1/PRI;
  % NFF

tempModelParam = {{[] []} {[] []}};
  % NFF: Change to suitable default values.
if isempty(modelParam)
  modelParam = tempModelParam;
else
  tempModelParam(1:length(modelParam)) = modelParam;
  modelParam = tempModelParam;
end%if

spaModelParam = modelParam{1};
pulModelParam = modelParam{2};

subArrPoint = spaModelParam{1};
focusDist = spaModelParam{2};
if isempty(subArrPoint)
  subArrPoint = [0 0].';
end%if
if isempty(focusDist)
  focusDist = Inf;
end%if
  % NFF

if isempty(doaFreqChoice)
  %beamChoice = d2r(-80:2:80);
  beamChoice = linspace(d2r(-90),d2r(90), antenna.noElem+1);
  beamChoice = beamChoice(:,1:(size(beamChoice,2)-1));
  %beamChoice = beamChoice(:,1:4)
  beamChoice = [beamChoice; zeros(1,length(beamChoice))];

  freqChoice = genfreqaxis(0,PRF,noPulses).';

  doaFreqChoice = {beamChoice,freqChoice};
    % NFF: Change to a suitable default value for "doaFreqChoice".
end%if

if isempty(taperCoeff)
  taperCoeff = [];
end%if
if isempty(interCorrMat)
  interCorrMat = [];
end%if
if isempty(steVecFlag)
  steVecFlag = 1;
end%if
if isempty(extraParam)
  extraParam = [];
end%if
if isempty(orthBeamFlag)
  orthBeamFlag = 0;
end%if
if isempty(normType)
  normType = [];
end%if
if isempty(normLevel)
  normLevel = [];
end%if
if isempty(sampleDim)
  sampleDim = only1DDim;
    % "only1DDim" = Use only one sample at a time.
    % NFF: Change to a suitable dimension.
end%if

% ************* Pick out some more fields from input parameters. *************

if (length(waveform.wavelength) > 1)
  error('DBT-Error: Can only handle one wavelength.')
end%if
lambda = waveform.wavelength;
  % NFF


% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT')
chkdtype(method, 'StringT')
chkdtype(model, 'CellArrayT')
chkdtype(spaModelParam, 'CellArrayT')
chkdtype(pulModelParam, 'CellArrayT')
chkdtype(subArrPoint, 'Vector of DoaT')
chkdtype(taperCoeff, 'CxVectorT')
chkdtype(interCorrMat, 'CxMatrixT', 'EmptyT')
%if (sampleDim ~= only1DDim)
%  error('DBT-Error: "sampleDim" cannot be chosen for this function.')
%end%if
  % NFF: Check any special input parameters.



% ----------------------------------------------------------------------- %
% Preparations before calculations.
% ----------------------------------------------------------------------- %


% *************** Adjustments for different methods. ***************

%In: method, extraParam, sigSizeIn, sigDim1, sigDim2, PRF, waveform, doaFreqChoice, interCorrMat 
%Out: steMatIn, beamPosOut, freqPosOut, noBeamsOut, noFreqsOut, interCorrMat

if (strcmp(method,'fft') | strcmp(method,'ifft'))
  % Special case when the number of output channels are not determined by 
  % "doaFreqChoice".
  dbterror('FFT or IFFT can not be used with this function.')
  % The following code is not finnished.
  if isempty(extraParam)
    extraParam = sigSizeIn(sigDim1) * sigSizeIn(sigDim2);
      %sigSizeIn(sigDim2);
  end%if
  extraParam = extraParam;

  noBeamsOut = extraParam(1);
  noFreqsOut = extraParam(2);
  extraParam = extraParam(1) * extraParam(2);
   % Is this the correct value for "extraParam"?
   % For the FFT method in the functions "calcfiltinit" and "calcfiltproc"
   % the parameter "extraParam" means the number of output channels or the
   % size of the output signal vector.

  steMatIn = [];
  interCorrMat = [];

  beamPosOut  = [];
    % This should be the DOAs of the beams created by FFT. NFF: 
  freqPosOut = genfreqaxis(0,PRF,noFreqsOut).';
    % NFF: 
else
  % The case when the number of frequencies are determined by 
  % the size of "doaFreqChoice".
  spaParams = {lambda, focusDist, subArrPoint};
  pulParams = {};
  steMatIn = spapulstemat(model, doaFreqChoice, {spaParams,pulParams});
    % NFF: Call the right function with the right parameters.
  noBeamsOut = size(doaFreqChoice{1},2);
  noFreqsOut = size(doaFreqChoice{2},2);
    % Shall it be the number of columns?
  beamPosOut  = doaFreqChoice{1};
  freqPosOut = doaFreqChoice{2};
end%if


% ****************** Create output variable. ******************

sigOut = sigIn;		% Copy all fields to output signal.
sigSizeOut = [noFreqsOut,noRanges, noBeamsOut, noExtras, noCpis,noTrials, 1];
  % The last "1" is to make possible to use the same code for 1D- and 
  % 2D-filtering. See the comment above about "sigSizeIn".
  % NFF: Move "noFreqsOut" and "noBeamsOut" to the right subscripts.


% ----------------------------------------------------------------------- %
% Do the filtering.
% ----------------------------------------------------------------------- %

calcInitParams = {taperCoeff, interCorrMat, orthBeamFlag, extraParam, ...
  normType, normLevel};
  % This parameter contains all input parameters 
  % after "steMat" to function "calcfiltinit". See the help text of the 
  % function "calcfiltinit" for more information. Most of these parameters
  % are also used by the function "calcfiltproc".

[signalsOut, basisMat, freqPos] = calcfiltsig(sigIn.signals, 'spapulfilt',...
  [sigDim1, sigDim2], sampleDim, sigSizeIn, sigSizeOut, steMatIn,...
  method, calcInitParams, interChoice, modCorrmParam);
  % Filter the whole radar signal. A signal vector sample or snapshot
  % consists of the dimensions "sigDim1" and "sigDim2". All samples from
  % the dimension "sampleDim" are filtered simultaneously.

% ----------------------------------------------------------------------- %
% Output variables.
% ----------------------------------------------------------------------- %

sigOut.signals = signalsOut;
sigOut = setspapultrans(sigOut, basisMat);
  % NFF: Call the right function.

% For Capon, only the non-adapted steering matrix is stored. See the function
% "calcfiltinit".

sigOut.doaPos = beamPosOut;
sigOut.freqPos = freqPosOut;
  % NFF: Assign the right field in the output function.
doaFreqPosOut = {beamPosOut, freqPosOut};

%endfunction spapulfilt
