function A = spastemat(antenna, doas, lambda, focusDist, pointSubDoa, steVecFlag)

%SPASTEMAT Calculation of spatial steering matrix.
%
%--------
%Synopsis:
%  A = spastemat(antenna, doas, lambda, focusDist, pointSubDoa)
%
%Description:
%  Calculates the spatial steering matrix for specified angles for a particular
%  focus distance. This function is normally not called by the end user but
%  by the extension programmer that extends the toolbox DBT with new signal
%  processing methods etc.
%
%  Possible antenna types are currently Isotropical ULA ('isotropULA'),
%  AIMT Application Example ('aimtEx'), AIMT Experimental Antenna ('expAnt'),
%  Uniform Linear Array ('ULA'), Non-Uniform Linear Array ('LA') and
%  Antenna elements (one channel antennas) with isotropical or arbitrary
%  antenna patterns. See the function "defant" for more information.
%
%Output and Input:
%  A (CxMatrix): The calculated spatial steering matrix. Size (noChan, noDoa),
%    where noChan is the number of antenna channels and noDoa is the number of
%    directions.
%  antenna (AntDefT): The definition of the receiving antenna to use.
%  doas (Vector of DoaT): Directions to calculate the steering vectors at.
%  lambda (RealScalarT): Wavelength of the carrier [m].
%  focusDist [D](RealVectorT): Focusing distance [m].
%  pointSubDoa [D](DoaT): Pointing direction of subarrays. The same direction
%    is used for all subarrays.
%  steVecFlag [D](StringT): Whether steering vectors (default) or beamforming 
%    weights are to be calculated. There is a difference only when correcting 
%    for calibration on the steering vectors and beamforming weights. See []
%    p. 15-18 for more information.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%  The 'expAnt' antenna type was used in the work done by A. Heydarkhan [5].
%
%
%Known Bugs:
%  Only one angle of the DOA:s (theta) can be used.
%  Focusing distance by the input parameter "focusDist" does not work. The
%  distance is allways infinity.
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%  [2]: Athley F.:"Tillmpningsexempel fr studie av modellbaserad
%    lobformning", Ericsson Microwave Systems 1996, No FY/XU-96:299.
%  [3]: Pettersson L., Danestig M., Sjstrm U.: "An Experimental S-Band
%    Digital Beamforming Antenna", IEEE AES Systems Magazine, November 1997,
%    p.19-26.
%  [4]: Bjrklund S.:"A MATLAB Toolbox for Radar Signal Processing",
%    Proceedings of Nordic MATLAB Conference `97,Stockholm 27-28 October 1997.
%  [5]: Heydarkhan A.: "Model Based Direction of Arrival Estimation Methods
%    Applied to Experimental Antenna Data", Master's Thesis, Chalmers
%    University of Technology 1997. FOA-R--97-00631-408--SE,FOA November 1997.
%  [6]: Stutzman W.L., Thiele G.A.: "Antenna Theory and Design", Wiley 1981,
%    ISBN 0-471-04458-X.
%  [7]: Pettersson L.: "versiktlig beskrivning av mtdataanalysprogram fr 
%    digital gruppantenn", FOA februari 1996, FOA-D--96-00226-3.2--SE.
%  []: Danestig M., Pettersson L., Sjstrm U.: "An Experimental S-Band
%    Digital Beamforming Antenna", IEEE International Symposium on Phased
%    Array Antennas, Boston, 15-18 October 1996.
%
%See Also:
%  dspastemat, The code of: doaspc1, doapar1, compsim2, pantpat3

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 961120 Svante Bjvrklund (svabj).
%  Latest change: $Date: 2000/10/21 14:06:14 $ (17:08) $Author: svabj $.
%  $Revision: 1.12 $
% *****************************************************************************

if (0)
  global thSign
  % Defines the reference direction for the theta angle in DOA:s.
  % Defined in "defant".
else
  thSign = 1;
end%if

% ----------------------------------------------------------------------- %
% Handle input parameters.
% ----------------------------------------------------------------------- %

% ****************** Add missing input parameters ******************

% Here should be some code also.

% ****************** Default values ******************

doas1 = zeros(2,size(doas,2)); 	% If phi is missing, extend "doas" with
				% a zero row
doas1(1:size(doas,1),:) = doas;
doas = doas1;

if (nargin < 4)
  focusDist = [];
end
if (nargin < 5)
  pointSubDoa = [];
end

if (isempty(focusDist))
  focusDist = Inf;
  %focusDist = Inf*ones(1,size(doas,2));
    % If there can be different focusing distances to the
    % different sources.
else
  focusDist = Inf;
    % Only if the focusing distance always is the same to all sources.
end%if

if (isempty(pointSubDoa))
  pointSubDoa = [0;0];
end%if


% ****************** Error check input parameters ******************

if (any(focusDist ~= Inf))
  error('DBT-Error: Only focus on infinity is implemented.')
end

% ----------------------------------------------------------------------- %
% Cell array of antenna definitions.
% ----------------------------------------------------------------------- %
if iscell(antenna)
  % In this code the input parameter "antenna" is a cell array with
  % antenna definitions. spastemat is called for each of the antennas.
  % The result is stacked in A with new rows for each new antenna.
  nDoas = size(doas,2);		% Number of DOA:s.
  nAnt=length(antenna);		% Number of antennas.
  nChan = 0;			% Total number of channels in all antennas.
  ixChanEnd = zeros(nAnt,1);	% End channel index in A for all antennas.
  for n = 1:nAnt
    nChan = nChan + antenna{n}.noElem;
    ixChanEnd(n) = nChan;
  end%for n
  A = zeros(nChan,nDoas);

if (0)	% If different pointing directions for different subarrays.
  if (size(pointSubDoa,2) > 1)	% If not the same definition of all elements.
    %Do nothing.
  else 	% Identical elements.
    pointSubDoa = repmat(pointSubDoa,1,nAnt);
  end%if
end%if (0)

  ixChanStart = 1;
  for n = 1:nAnt
    A(ixChanStart:ixChanEnd(n),:) = spastemat(antenna{n}, doas, lambda, ...
      focusDist, pointSubDoa);
    %A(ixChanStart:ixChanEnd(n),:) = spastemat(antenna{n}, doas, lambda, ...
    %  focusDist, pointSubDoa(n));
      % If different pointing directions for different subarrays.
    ixChanStart = ixChanEnd(n) + 1;
    %A = [A; spastemat(antenna{n}, doas, lambda, focusDist, pointSubDoa)]
  end%for

else % Not iscell(antenna)


% ----------------------------------------------------------------------- %
% Code common to all antenna types.
% ----------------------------------------------------------------------- %

  % ****************** Pick out fields from input parameters. ******************

  antennaType = antenna.antennaType;


% ----------------------------------------------------------------------- %
% Antenna types.
% ----------------------------------------------------------------------- %

% ----------------------------------------------------------------------- %
% Short cuts for special antenna types.
% ----------------------------------------------------------------------- %
  % --------------------------------------------------------------------- %
  % Isotropical ULA.
  %
  % Start: 980107 Svante Bjrklund (svabj).
  % --------------------------------------------------------------------- %
  if (strcmp(antenna.name,'isotropULA'))

    % Not implemented yet.

  % --------------------------------------------------------------------- %
  % AIMT Application Example.
  %
  % Start: 9xxxxx Fredrik Athley (freath).
  % --------------------------------------------------------------------- %
  elseif (strcmp(antenna.name,'aimtEx'))
    if (nargin < 5)
       pointSubDoa = [];
    end
    if (isempty(pointSubDoa))
      pointSubDoa = 0;
    else
      pointSubDoa = pointSubDoa(1,:);
    end%if
    poserr = 0;
    %lambda = antenna.lambda;
    d = antenna.distElem;
    K = antenna.noElem;
    k = 0:K-1;
    dmek = k*d + poserr;
    theta = doas(1,:);
    elementFactor = subpat(antenna,theta,pointSubDoa,poserr,[0.5 1 0.5],lambda);
    % The position errors in "poserr" will be the same for all subarrays.
    % This must be an error.
    %size(elementFactor)
    elFactorMatrix = diag(sparse(elementFactor));
    %elFactorMatrix = sparse(diag(elementFactor));
       % This generates a to large matrix
%    elFactorMatrix = diag(elementFactor);
%    arrayFactor = exp(thSign*i*2*pi*k'*d/lambda*sin(theta'));
    arrayFactor = exp(thSign*i*2*pi*dmek'/lambda*sin(theta));
    A = full(arrayFactor*elFactorMatrix);


  % --------------------------------------------------------------------- %
  % AIMT Experimental Antenna.
  %
  % What global variables are used and from where do the come?
  %   expCalVerG (from defant): Version of the calibration compensation
  %     information.
  %   c0 (from defant): The speed of light.
  %   cmpmetGlobal ():
  %   nearfieldMethodFlagGlobal ():
  %   Some more global variables are used in functions called by spastemat.
  %
  % Start: 97xxxx Svante Bjrklund (svabj).
  % --------------------------------------------------------------------- %
  elseif (strcmp(antenna.name,'expAnt'))
    global expCalVerG
    if (isempty(expCalVerG))
      dbterror('Missing calibration information.')
    end%if
    lpSign = +1;  % Because Lars has defined angels differently than DBT.
      % +1 : as Lars and as in dbt (now) at other places.
    doas = lpSign * doas;
    if ((expCalVerG == 2) | (expCalVerG == 3))
      global c0
      global cmpmetGlobal nearfieldMethodFlagGlobal
      %global Ccmp wctab sphitab wcmedel wcmat

      frequencyMhz = (c0/lambda)*1e-6;
      cmpmet = cmpmetGlobal;
      coupFlag = 1; % An empty matrix indicates a false value.
      wctabAngles = 19; % A dummy value.
      %wctabAngles = phitab;
      distElem    = antenna.distElem;
      noElem      = antenna.noElem;
      sphitest=sin(doas(1,:))';

      if (nearfieldMethodFlagGlobal == 1)
        wnfcmp=eincdbt(sphitest,frequencyMhz,'nearfield');
          % Calculate information for compensation of near field.
          %stest0=exp(j*k0*sphitest*xel).'/sqrt(nel);
      else %Do not compensate for near field.
        wnfcmp = [];
      end%if

      A0 = exp(j*(2*pi/lambda)*distElem*(0:noElem-1)' *sin(doas(1,:)));
        % The not compensated steering matrix.
        % "Utstyrt i sphix vid mittfrekvens" (larpet).
        % A0 is the steering matrix for an ULA with isotropical elements.
        % The sign in the exponent is maybe the opposite.
      A = wscordbt(A0,[],coupFlag,cmpmet,[],wnfcmp,[], doas(1,:));
        % Compensate the steering matrix for antenna element coupling,
        % amplitude and phase drift and near field.
    end%if expCalVerG


% ----------------------------------------------------------------------- %
% Antenna Elements (one channel antennas).
%
% Start: 980607 Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %
  elseif (strcmp(antennaType,'elem'))
    %disp('spastemat elem')
    nDoas = size(doas,2);				% Number of DOA:s.
    pType = antenna.patternType;
    if (strcmp(pType,'isotrop'))
      A = ones(1,nDoas);
    elseif (strcmp(pType,'func'))
      x = doas - repmat(antenna.elemOrient,1,size(doas,2));
      A = eval(antenna.patternFunc);  	% x is the evaluated variable.
    else
      error('Illegal pattern type for element.')
    end%if



% ----------------------------------------------------------------------- %
% Antenna elements.
%
% spastevec is called.
%
% Start: 9xxxxx David Rejdemyhr (davrej).
% ----------------------------------------------------------------------- %
  elseif ( strcmp(antennaType,'elem') )
    % This code should never be executed because of the if-branch above.
    dbtwarning('Calling "spastevec" for antenna elements.')
    M = size(doas,2); % No DOA:s.
    noElem   = antenna.noElem;	% Allways == 1.
    A = zeros(noElem, M);
    for n = 1:M
      A(:,n) = spastevec(antenna, doas(1,n), lambda, focusDist);
      %A(:,n) = spastevec(antenna, doas(1,n), lambda, focusDist(1,n));
        % If there can be different focusing distances to the
        % different sources, the commented line can be used.
    end%for


% ----------------------------------------------------------------------- %
% ULA and LA.
%
% spastevec is called.
%
% Start: 9xxxxx David Rejdemyhr (davrej).
% ----------------------------------------------------------------------- %
  elseif (strcmp(antennaType,'ULA') | strcmp(antennaType,'LA'))
    old = 1;
    if (old)
      %disp('Use spastevec')
      %dbtwarning('Calling "spastevec" for ULA or LA.')
      M = size(doas,2); % No DOA:s.
      noElem   = antenna.noElem;
      A = zeros(noElem, M);
      for n = 1:M
        A(:,n) = spastevec(antenna, doas(1,n), lambda, focusDist);
        %A(:,n) = spastevec(antenna, doas(1,n), lambda, focusDist(1,n));
          % If there can be different focusing distances to the
          % different sources, the commented line can be used.
      end%for
    elseif (0)
      noElem   = antenna.noElem;
      if (strcmp(antennaType,'ULA'))
        distElem = antenna.distElem;
        elemPos = distElem*(0:noElem-1).';
      else
        elemPos = antenna.elemPos;
      end%if
      a = exp(thSign*j*2*pi*(elemPos/lambda)*sin(doas(1,:)));
      % Array faktor.
      w = antenna.element.beamSpaceTrans;
      %a = a * (w' * spastevec(antenna.element, doas, lambda, focusDist));
      a = a * (w' * spastemat(antenna.element, doas, lambda, focusDist));
    else

      
    end%if (old)



% ----------------------------------------------------------------------- %
% Array antenna.
%
% Start: 980515 Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %
  elseif (strcmp(antennaType,'array'))
    if iscell(antenna.element)	% If not the same definition of all elements.
      Asub = spastemat(antenna.element, doas, lambda, focusDist, pointSubDoa);
        % Or a loop. Will this be faster?
    else 	% Identical elements.
      Asub = repmat(spastemat(antenna.element, doas, lambda, focusDist, ...
        pointSubDoa), antenna.noElem,1);
        % Any way to avoid a  whole matrix Asub and to gain execution time?
    end%if
    scanPhases = repmat(-1*antenna.elemPos.'*doa2wavevec(pointSubDoa,lambda),...
      1,size(doas,2));
      % "The array factor is maximum when the far-field contributions from
      % elements add in phase." ([6] p. 161). When the far-field contributions
      % from elements add in phase, the phase differences between elements
      % in the array factor are zero.
    AF = exp(-1*thSign * j * (antenna.elemPos.' *  doa2wavevec(doas,lambda) ...
      + scanPhases));
    %if (isempty(antenna.beamSpaceTrans))
      A = AF.*Asub;
    %else % This does not work because the antenna definition field
    %"noElem" is wrong. Shall "noElem" mean number of channels in or out?
    %  %size(antenna.beamSpaceTrans),size(AF),size(Asub)
    %  A = antenna.beamSpaceTrans' * (AF.*Asub);
    %end%if;


if(0)
% ----------------------------------------------------------------------- %
% Array antenna with beamforming.
%
% Start: 980530 Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %
  elseif (strcmp(antennaType,'BFArray'))
    % ... As antenna type 'array'.
    %A = AF.*(antenna.beamSpaceTrans' * Asub);
    A = antenna.beamSpaceTrans' * (AF.*Asub);
end%if(0)


% ----------------------------------------------------------------------- %
% Beamforming virtual antenna
%
% Start: 980520 Svante Bjrklund (svabj).
% ----------------------------------------------------------------------- %
  elseif (strcmp(antennaType,'beamform'))
    Asub = spastemat(antenna.element, doas, lambda, focusDist, pointSubDoa);
    A = antenna.beamSpaceTrans' * Asub;


% ----------------------------------------------------------------------- %
% The else branch.
% ----------------------------------------------------------------------- %
  else
    error('DBT-Error: Not implemented for desired antenna type.')
  end%if name


% ----------------------------------------------------------------------- %
% Normalization of the steering matrix.
% ----------------------------------------------------------------------- %

if (0)
  %disp('Normalizing A...')
  %r2d(doas(:,[1, 2, end-1, end]))
  %A(:,[1, 2, end-1, end])

  %ANorm = vecnorm2(A);
  %A = weightcols(A, 1./ANorm);

  %diag(A'*A).'
  %A(:,[1, 2, end-1, end])
end%if (0)

end%if iscell(antenna)
