function eeTimeVarOut = eetimebegin

%EETIMEBEGIN Begin estimation of execution time.
%
%--------
%Synopsis:
%  eeTimeVarOut = eetimebegin
%
%Description:
%  Begin estimation of execution time.
%  This function together with "eetimepreloop", eetimeloop" and "eetimeend"
%  is used to estimate the total execution time when executing MATLAB programs
%  with long execution times. The programs must have surrounding loops.
%  For the time estimation to work all trials must have about the same  
%  execution time.
%  The estimate is continously updated during the execution.
%
%  The purpose of the estimation of execution time is to as soon as possible
%  see how long the execution time will be in order to to know how long to
%  wait. If the execution time will be too long, some parameters have to be
%  changed, e.g. decreasing the number of trials in Monte Carlo simualtions,
%  in order to decrease the execution time.
%
%  The function "eetimebegin" must be placed at the beginning of the program.
%  The function "eetimepreloop" must be placed at just before the suurounding 
%    loops.
%  The function "eetimeloop" must be placed at the end of one of the loops.
%  The function "eetimeend" should be placed at the end of the program.
%
%  See the program "radio/wideex3" for an example of the use of these
%  functions.
%
%  The display on the screen can for example look like:
%    Done: fSig=2(2), mekAng=3(3), sampI=1(2), trialL=1(1), TotalL=8(12), 
%    E.time=0.9(1.3)hour.
%  The strings 'fSig', 'mekAng' and 'sampI' identify the outmost loops 
%  and are specified by the user. The numbers following them are the
%  current loop and maximum loop counts for respective loop.
%  'trialL' is the trial loop count. 'TotalL' is the total loop count
%  (of the innermost loop). The first number following 'E.time' is the 
%  elapsed time since start of execution and the second number is the
%  estimated total required time.
%
%Output and Input:
%  eeTimeVarOut (): A structure variable containing information for the 
%    functions "eetimebegin", "eetimepreloop", "eetimeloop" and "eetimeend"
%    and it should not be manipulated directly by the user.
%    This variable is next used as input to the function "eetimepreloop".
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%  See the example script file radio/wideex3.m for an example.
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  eetimepreloop, eetimeloop, eetimeend, radio/wideex3


%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 990222 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:40:11 $ $Author: svabj $.
%  $Revision: 1.9 $
% *****************************************************************************


eeTimeVarOut.startClock = clock;

eeTimeVarOut.begin = 1;	% For checking that the function are called in right order.

%The fields in eeTimeVarOut will eventually be:
%  startClock ():
%  elapTimePreLoop (IntScalarT): Elapsed execution time before the loops
%    [seconds].
%  estTotTime (IntScalarT): Estimated total time for the entire execution
%    [seconds].
%  noTrials (IntScalarT):
%  noLoop1 (IntScalarT):
%  noLoop2 (IntScalarT):
%  noLoop3 (IntScalarT):
%  noTotalLoop (IntScalarT):
