function eeTVOut = eetimepreloop(eeTVIn, noTrials, noLoop1, noLoop2, noLoop3)

%EETIMEPRELOOP Pre-loop preparation of estimation of execution time.
%
%--------
%Synopsis:
%  eeTVOut = eetimepreloop(eeTVIn, noTrials, noLoop1)
%  eeTVOut = eetimepreloop(eeTVIn, noTrials, noLoop1, noLoop2, noLoop3)
%
%Description:
%  Pre-loop preparation of estimation of execution time.
%  For the use of this function see the help text of function "eetimebegin".
%
%Output and Input:
%  eeTVOut ():  Same as "eeTVIn" but is output and is to be used by the
%    function "eetimeloop".
%  eeTVIn (): A structure variable containing information for the 
%    functions "eetimebegin", "eetimepreloop", "eetimeloop" and "eetimeend"
%    and it should not be manipulated directly by the user.
%    This variable is here the output of the function "eetimebegin".
%  noTrials (IntScalarT): The trial loop is the loop for repeated trials,
%    e.g. Monte Carlo simualtions, of the whole calculation. It is
%    expected that all trials need approximately the same execution time.
%  noLoop1 (IntScalarT): The number of turns in the loop, "loop one", 
%    enlcosing the trial loop.
%  noLoop2 [D](IntScalarT): The number of turns in the loop enlcosing loop one.
%  noLoop3 [D](IntScalarT): The number of turns in the loop enlcosing loop two.
%   If three loops are not enough, let loop three be the outermost loop and
%   the same change for the other loops.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in [1]
%  or by "help dbtdata".
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Examples:
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.)
%
%Known Bugs:
%
%References:
%  [1]: Bjrklund S.: "DBT, A MATLAB Toolbox for Radar Signal Processing.
%    Reference Guide", FOA-D--9x-00xxx-408--SE, To be published.
%
%See Also:
%  eetimebegin, eetimeloop, eetimeend, radio/wideex3


%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 990222 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:40:14 $ $Author: svabj $.
%  $Revision: 1.5 $
% *****************************************************************************

% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

% ****************** Add missing input parameters ******************


arginNo=3;
if (nargin < arginNo)
  error('DBT-Error: To few input parameters.')
end
arginNo = arginNo +1;

if (nargin < arginNo)
  noLoop2 = [];
end
arginNo = arginNo +1;
if (nargin < arginNo)
  noLoop3 = [];
end
arginNo = arginNo +1;


% ****************** Default values ******************

if isempty(noLoop2)
  noLoop2 = 1;
end%if
if isempty(noLoop3)
  noLoop3 = 1;
end%if


% ----------------------------------------------------------------------- %
% Do the actual work.
% ----------------------------------------------------------------------- %

if (~isfield(eeTVIn,'begin'))
  error('The function "eetimebegin" must be called before "eetimepreloop".')
end%if

% **************** Prepare execution time information ****************
% In: noBW, noTrials, startClock
% Out: estTotTime, noTotalLoop, clockPreLoop, elapTimePreLoop
%   This output information is suitable for a structure variable.

eeTVOut = eeTVIn;

eeTVOut.estTotTime = Inf;

clockPreLoop = clock;
eeTVOut.elapTimePreLoop =  etime(clockPreLoop,eeTVIn.startClock);

eeTVOut.noTrials = noTrials;
eeTVOut.noLoop1 = noLoop1;
eeTVOut.noLoop2 = noLoop2;
eeTVOut.noLoop3 = noLoop3;
eeTVOut.noTotalLoop = noTrials * noLoop1 * noLoop2 * noLoop3;


eeTVOut.preLoop = 1;  % For checking that the function are called in right order.
