function sigOut = videoint(sigIn, method)

%VIDEOINT Video integration of a radar signal.
%
%--------
%Synopsis:
%  sigOut = videoint(sigIn)
%
%Description:
%  If method = 'abs' (default), video integration of the signal from
%  several coherent processing intervals (CPI) is performed. This means
%  that the absolute values of the signal for several CPI:s are added.
%  This is also termed "incoherent" integration.
%
%  If method = 'sig', the summation of the signal from several coherent 
%  processing intervals (CPI) is performed.
%
%  The summation for both methods is normed by dividing with the number of
%  CPIs in the sum
%
%Output and Input:
%  sigOut (RxRadarSigT): Output radar signal.
%  sigIn (RxRadarSigT): Input radar signal.
%  method [D](StringT): Method.
%    = 'abs': Absolute values of the signal for several CPI:s are added 
%      (default). This is equal to noncoherent integration.
%    = 'sig': The signals for several CPI:s are added. No absloute values
%      are calculated. This is equal to coherent integration.
%
%--------
%Notations:
%  Data type names are shown in parentheses and they start with a capital
%  letter and end with a capital T. Data type definitions can be found in ...
%  [D] = This parameter can be omitted and then a default value is used.
%  When the [D]-input parameter is not the last used in the call, it must be
%  given the value [], i.e. an empty matrix.
%  ... = There can be more parameters. They are explained under respective
%  metod or choice.
%
%Software Quality:
%  (About what is done to ascertain software quality. What tests are done.
%   Known bugs.)
%
%Known Bugs:
%
%See Also:
%  

%   *  DBT, A Matlab Toolbox for Radar Signal Processing  *
% (c) FOA 1994-2000. See the file dbtright.m for copyright notice.
%
%  Start        : 981029 Svante Bjrklund (svabj).
%  Latest change: $Date: 2000/10/16 15:22:20 $ $Author: svabj $.
%  $Revision: 1.12 $
% *****************************************************************************

%  Start        : 961120 Fredrik Athley (freath).
%ant = sigIn.antenna;


% ----------------------------------------------------------------------- %
% Handle input parameters
% ----------------------------------------------------------------------- %

arginNo=1;
if (nargin < arginNo)
  dbterror('To few input parameters.')
end
arginNo = arginNo +1;

% ****************** Add missing input parameters ******************

arginNo=2;
if (nargin < arginNo)
  method = [];
end
arginNo = arginNo +1;

% ****************** Default values ******************

if isempty(method)
  method = 'abs';
end%if

% ****************** Error check input parameters ******************

chkdtype(sigIn, 'RxRadarSigT')
chkdtype(method, 'StringT')


% ----------------------------------------------------------------------- %
% Do the work.
% ----------------------------------------------------------------------- %

if (strcmp(method,'abs'))
  absXX =  abs(sigIn.signals);
elseif (strcmp(method,'sig'))
  absXX =  sigIn.signals;
else
  dbterror('Unknown method in videoint.')
end%if

s = sizem(absXX);
noCPIs = s(5);

if (0)
noDopplerCh = s(1);
noRangeBins = s(2);
noBeams     = s(3);
noNotUsed   = s(4);
noTrial     = s(6);

YY = zerosm([s(1:4) 1 s(6)]);
for trial = 1:noTrial
  for range=1:noRangeBins
    Y = zeros(noBeams,noDopplerCh);
    for cpi=1:noCPIs
      X = getm(absXX,':',range,':',1,cpi);
      X = reshape(X,noBeams,noDopplerCh);
      Y = Y + X;
    end%for cpi
    YY = setm(YY,Y.',':',range,':',1,1);
  end%for range
end%for trial

%sigOut.dataType = 'RxRadarSigT';
%sigOut.antenna = ant;
%sigOut.signals = YY;
end%if (0)

sNew = s;
%notUsedIx = 4;
cpiIx = 5;
sNew(cpiIx) = 1;
YY = zeros(sNew);
for cpi=1:noCPIs
  YY = YY + absXX(:,:,:,:,cpi,:,:);
  infoStr = sprintf('videoint: cpi = %d(%d)\r',cpi,noCPIs);
  dbtinfo(infoStr,1);
end%for cpi
dbtinfo('');	%New line.

YY = YY ./ noCPIs;

sigOut = sigIn;  % Must copy all information in the input signal.
sigOut.signals = YY; % Modify part of the input signal.

